import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dialog;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.WindowEvent;
import java.util.Vector;

/**
 * Dialog for summarizing sets of events, or creating a generalized model.
 */
public class AbstractionDialog extends Dialog {

	// Call AbstractionDialog.setSummarizing(true/false) before constructing.

	Panel AbstractionDialogPanel = new Panel();

	Panel abstractionArea = new Panel();

	Panel incorporateArea = new Panel();

	Panel doneArea = new Panel();

	Panel nameArea = new Panel();

	Panel incorporateButtonArea = new Panel();

	Label namingBoxTitle = new Label();

	Label summarizableTitle = new Label();

	TextField abbreviationField = new TextField(15);

	TextArea concreteEventDescription = new TextArea("", 3, 25,
			TextArea.SCROLLBARS_VERTICAL_ONLY);

	TextArea generalizedEventDescription = new TextArea("", 3, 25,
			TextArea.SCROLLBARS_VERTICAL_ONLY);

	Button doneButton = new Button();

	Button incorporateButton = new Button();

	Button instantiatesButton = new Button();

	Choice concreteDeedsChoice = new Choice();

	Choice generalizedDeedsChoice = new Choice();

	java.awt.List summarizableSequencesBox = new java.awt.List();

	BorderLayout abstractionDialogLayout = new BorderLayout();

	BorderLayout incorporateAreaLayout = new BorderLayout();

	BorderLayout abstractionAreaLayout1 = new BorderLayout();

	FlowLayout abstractionAreaLayout2 = new FlowLayout(FlowLayout.CENTER, 15,
			15);

	/** setSummarizing is called before invoking dialog. */
	static public boolean summarizing;

	Vector setsOfSummarizableSequences;

	String priorGeneralizationName;

	public AbstractionDialog(Frame frame, String title, boolean modal) {
		super(frame, title, modal);
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		try {
			jbInit();
			add(AbstractionDialogPanel);
			pack();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public AbstractionDialog(Frame frame) {
		this(frame, "", true);
	}

	void jbInit() throws Exception {
		AbstractionDialogPanel.setBackground(Color.yellow);
		abstractionArea.setBackground(Color.yellow);
		incorporateArea.setBackground(Color.yellow);
		doneArea.setBackground(Color.yellow);
		nameArea.setBackground(Color.yellow);
		incorporateButtonArea.setBackground(Color.yellow);
		namingBoxTitle.setBackground(Color.yellow);
		summarizableSequencesBox.setBackground(Color.yellow);
		instantiatesButton.setBackground(Color.cyan);
		incorporateButton.setBackground(Color.magenta);
		doneButton.setBackground(Color.cyan);

		doneButton.setLabel(Ethno.interfaceTexts.getString("doneButtonText"));
		instantiatesButton.setLabel(Ethno.interfaceTexts
				.getString("instantiatesButtonText"));
		summarizableTitle.setText(Ethno.interfaceTexts
				.getString("summarizableText"));

		AbstractionDialogPanel.setLayout(abstractionDialogLayout);
		incorporateArea.setLayout(incorporateAreaLayout);
		namingBoxTitle.setAlignment(Label.CENTER);
		summarizableTitle.setAlignment(Label.CENTER);

		AbstractionDialogPanel.add(abstractionArea, BorderLayout.NORTH);
		if (summarizing) {
			abstractionArea.setLayout(abstractionAreaLayout1);
			abstractionArea.add(summarizableTitle, BorderLayout.NORTH);
			abstractionArea.add(summarizableSequencesBox, BorderLayout.CENTER);
			namingBoxTitle.setText(Ethno.interfaceTexts
					.getString("summaryNameText"));
			incorporateButton.setLabel(Ethno.interfaceTexts
					.getString("replaceSequenceText"));
			findSummarizableSequences();
			Ethno.graphCard.setCursor(Cursor
					.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
		} else {
			// Generalize.
			Deed aDeed, concreteDeed, generalizedDeed;
			Relation r;
			int index;
			String deedName;
			namingBoxTitle.setText(Ethno.interfaceTexts
					.getString("generalizationNameText"));
			incorporateButton.setLabel(Ethno.interfaceTexts
					.getString("addGeneralizationText"));
			abstractionArea.setLayout(abstractionAreaLayout2);
			abstractionArea.add(concreteDeedsChoice, null);
			abstractionArea.add(instantiatesButton, null);
			abstractionArea.add(generalizedDeedsChoice, null);
			for (int i = 0; i < Ethno.deeds.size(); i++) {
				aDeed = (Deed) Ethno.deeds.elementAt(i);
				if (aDeed.isGeneralization) {
					generalizedDeedsChoice.addItem(aDeed.shortName);
					if (i == 0) {
						priorGeneralizationName = aDeed.shortName;
					}
				} else {
					if (!aDeed.isSummarized) {
						concreteDeedsChoice.addItem(aDeed.shortName);
					}
				}
			}
			AbstractionDialogPanel.add(concreteEventDescription,
					BorderLayout.WEST);
			AbstractionDialogPanel.add(generalizedEventDescription,
					BorderLayout.EAST);
			concreteEventDescription.setEditable(false);
			generalizedEventDescription.setEditable(true);
			if (concreteDeedsChoice.getItemCount() > 0) {
				deedName = concreteDeedsChoice.getItem(0);
				index = Ethno.deeds.findPositionOfElement(deedName);
				concreteDeed = (Deed) Ethno.deeds.elementAt(index);
				// Show full description:
				concreteEventDescription.setText(concreteDeed.verboseEventName);
				if (generalizedDeedsChoice.getItemCount() > 0) {
					deedName = generalizedDeedsChoice.getItem(0);
					priorGeneralizationName = deedName;
					index = Ethno.deeds.findPositionOfElement(deedName);
					generalizedDeed = (Deed) Ethno.deeds.elementAt(index);
					generalizedEventDescription
							.setText(generalizedDeed.verboseEventName);
					// Reset Instantiate button name if the visible relation is
					// an instantiation.
					for (int i = 0; i < concreteDeed.generalizations.size(); i++) {
						r = (Relation) concreteDeed.generalizations
								.elementAt(i);
						if (deedName.equals(r.implicated)) {
							instantiatesButton.setLabel(Ethno.interfaceTexts
									.getString("deleteLinkText"));
						}
					}
				}
			}
		}
		// Add the naming field.
		AbstractionDialogPanel.add(incorporateArea, BorderLayout.CENTER);
		incorporateArea.add(namingBoxTitle, BorderLayout.NORTH);
		incorporateArea.add(nameArea, BorderLayout.CENTER);
		nameArea.add(abbreviationField, null);
		// Add an Incorporate button.
		incorporateArea.add(incorporateButtonArea, BorderLayout.SOUTH);
		incorporateButtonArea.add(incorporateButton, null);
		// Add a Done button.
		AbstractionDialogPanel.add(doneArea, BorderLayout.SOUTH);
		doneArea.add(doneButton, null);

		instantiatesButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						changeInstantiation();
					}
				});
		incorporateButton
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						incorporateNewDeed();
					}
				});
		doneButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exitGeneralizing();
			}
		});
		summarizableSequencesBox
				.addItemListener(new java.awt.event.ItemListener() {
					public void itemStateChanged(ItemEvent e) {
						showSummarizableSet();
					}
				});
		concreteDeedsChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				setUpSelectionOfConcreteAndGeneralizedDeeds();
			}
		});
		generalizedDeedsChoice
				.addItemListener(new java.awt.event.ItemListener() {
					public void itemStateChanged(ItemEvent e) {
						setUpSelectionOfConcreteAndGeneralizedDeeds();
					}
				});
		abbreviationField
				.addActionListener(new java.awt.event.ActionListener() {
					public void actionPerformed(ActionEvent e) {
						incorporateNewDeed();
					}
				});
	} // End jbInit.

	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			cancel();
		}
		super.processWindowEvent(e);
	}

	void cancel() {
		dispose();
	}

	/**
	 * Search through the entire graph to find sets of events that could be
	 * summarized as one event.
	 */
	void findSummarizableSequences() {
		Deed aDeed, lowerNode, upperNode;
		PerceptList deedsWithLinks;
		Vector upFromUpper, upFromLower, downFromUpper, downFromLower, aVector;
		boolean upwardsEqualsDownwards;
		DeedWithLinks aLinkRecord;
		// Create lists of DeedWithLinks for faster processing.
		deedsWithLinks = new PerceptList();
		for (int ix = 0; ix < Ethno.graphCard.theGraph.graphNodes.size(); ix++) {
			aDeed = (Deed) Ethno.graphCard.theGraph.graphNodes.elementAt(ix);
			if (aDeed.shortName == null) {
				continue;
			} // Dummy node.
			aDeed.isSelected = false;
			deedsWithLinks.addElement(new DeedWithLinks(aDeed));
		}
		// Find and list the summarizable sequences.
		setsOfSummarizableSequences = new Vector();
		for (int i = 0; i < deedsWithLinks.size(); i++) {
			aLinkRecord = (DeedWithLinks) deedsWithLinks.elementAt(i);
			upperNode = (Deed) aLinkRecord.theDeed;
			NextLower: for (int j = i + 1; j < deedsWithLinks.size(); j++) {
				aLinkRecord = (DeedWithLinks) deedsWithLinks.elementAt(j);
				lowerNode = (Deed) aLinkRecord.theDeed;
				// Only consider sequences at least three events long.
				if ((upperNode.tier + 2) > lowerNode.tier) {
					continue;
				}
				// Get the nodes upward from lowerNode - nodes upward from
				// upperNode.
				upFromUpper = deedsWithLinks.quickLinks(upperNode,
						PerceptList.SEARCH_UP);
				upFromUpper.addElement(upperNode);
				upFromLower = deedsWithLinks.quickLinks(lowerNode,
						PerceptList.SEARCH_UP);
				if (!upFromLower.contains(upperNode)) {
					continue;
				} // For speed.
				for (int k = 0; k < upFromUpper.size(); k++) {
					upFromLower.removeElement(upFromUpper.elementAt(k));
				}
				for (int kk = 0; kk < upFromLower.size(); kk++) {
					// For speed.
					aDeed = (Deed) upFromLower.elementAt(kk);
					if (aDeed.tier <= upperNode.tier) {
						continue NextLower;
					}
				}
				// Get the nodes downward from upperNode - nodes downward from
				// lowerNode.
				downFromUpper = deedsWithLinks.quickLinks(upperNode,
						PerceptList.SEARCH_DOWN);
				downFromLower = deedsWithLinks.quickLinks(lowerNode,
						PerceptList.SEARCH_DOWN);
				downFromLower.addElement(lowerNode);
				for (int kk = 0; kk < downFromLower.size(); kk++) {
					downFromUpper.removeElement(downFromLower.elementAt(kk));
				}
				if ((downFromUpper.size() == 0) || (upFromLower.size() == 0)) {
					continue;
				}
				// Are the nodes going up the same as the nodes going down?
				upwardsEqualsDownwards = true;
				for (int m = 0; m < downFromUpper.size(); m++) {
					// removeElement returns false if element not found.
					upwardsEqualsDownwards = upwardsEqualsDownwards
							& upFromLower.removeElement(downFromUpper
									.elementAt(m));
				}
				if (upwardsEqualsDownwards && (upFromLower.size() == 0)) {
					// Yes, this is a subgraph.
					downFromUpper.insertElementAt(upperNode, 0);
					downFromUpper.addElement(lowerNode);
					setsOfSummarizableSequences.addElement(downFromUpper);
					summarizableSequencesBox.add(upperNode.shortName + " - "
							+ lowerNode.shortName);
				}
			}
		}
		if (summarizableSequencesBox.getItemCount() > 0) {
			summarizableSequencesBox.select(0);
			aVector = (Vector) setsOfSummarizableSequences.elementAt(0);
			for (int node = 0; node < aVector.size(); node++) {
				aDeed = (Deed) aVector.elementAt(node);
				aDeed.isSelected = true;
			}
			Ethno.graphCard.theGraph.paint(Ethno.graphCard.theGraph
					.getGraphics());
		}
	} // End findSummarizableSequences.

	/** Set the flag indicating whether we are summarizing or generalizing. */
	static public void setSummarizing(boolean newSummarizing) {
		summarizing = newSummarizing;
	}

	/** Try creating a new event with this name. */
	void incorporateNewDeed() {
		Deed topNode, bottomNode, aDeed, newDeed;
		String theName = Ethno.cleanTheString(abbreviationField.getText());
		if (theName.equals("")) { return; }
		newDeed = new Deed(theName, "");
		if (Ethno.deeds.insertAlphabetically(newDeed) < 0) {
			CancelOkDialog whoops = new CancelOkDialog(
					(Frame) this.getParent().getParent(), Ethno.interfaceTexts
							.getString("problemEncounteredText"),
					Ethno.interfaceTexts.getString("duplicateDeedError"), "");
			whoops.setVisible(true);
			return;
		}
		// Perform the proper kind of abstraction.
		if (!summarizing) {
			// New generalization.
			newDeed.isGeneralization = true;
			generalizedDeedsChoice.addItem(newDeed.shortName);
			if (generalizedDeedsChoice.getItemCount() == 1) {
				priorGeneralizationName = newDeed.shortName;
			}
			abbreviationField.setText("");
			Ethno.controls.rebuildEventsChoice();
			this.pack();
			// Generalizations do not appear on the graph until they are linked.
		} else {
			// New Summary.
			Relation r;
			// Recall the set of deeds to be replaced.
			int index = summarizableSequencesBox.getSelectedIndex();
			Vector aVector = (Vector) setsOfSummarizableSequences
					.elementAt(index);
			// Record new links.
			for (int i = 0; i < aVector.size(); i++) {
				Ethno.linkingHistory = Ethno.linkingHistory
						+ ((Deed) aVector.elementAt(i)).shortName
						+ Ethno.interfaceTexts.getString("summarizedIn")
						+ newDeed.shortName
						+ Ethno.interfaceTexts.getString("newLine");
				Ethno.defineLinksCard.elicitationLog
						.setText(Ethno.linkingHistory);
			}
			topNode = (Deed) aVector.firstElement();
			bottomNode = (Deed) aVector.lastElement();
			// Define newDeed's parameters from end nodes.
			newDeed.isDisjunctive = topNode.isDisjunctive;
			newDeed.implications = (Vector) topNode.implications.clone();
			// Transfer bottomNode's consequences to newDeed.
			for (int k = 0; k < Ethno.deeds.size(); k++) {
				aDeed = (Deed) Ethno.deeds.elementAt(k);
				for (int kk = 0; kk < aDeed.implications.size(); kk++) {
					r = (Relation) aDeed.implications.elementAt(kk);
					if (r.implicated.equals(bottomNode.shortName)) {
						r.implicated = theName;
					}
				}
			}
			// Remember bottomNode in case summarization is deleted.
			newDeed.linkToFinalSummarizedAction = new Relation(
					bottomNode.shortName, false, false);
			newDeed.isLinked = true;
			// Follow bottomNode with the summary deed in the sequence of deeds.
			// Work backwards to allow for insertions:
			for (int m = Ethno.sequence.size() - 1; m >= 0; m--) {
				if (Ethno.sequence.elementAt(m) == bottomNode) {
					Ethno.sequence.insertElementAt(newDeed, m + 1);
				}
			}
			for (int n = 0; n < aVector.size(); n++) {
				aDeed = (Deed) aVector.elementAt(n);
				// Transfer generalizations to summarizer deed.
				String generalizationOfOld, generalizationOfNew;
				for (int i = 0; i < aDeed.generalizations.size(); i++) {
					for (int j = 0; j < newDeed.generalizations.size(); j++) {
						generalizationOfOld = ((Relation) aDeed.generalizations
								.elementAt(i)).implicated;
						generalizationOfNew = ((Relation) newDeed.generalizations
								.elementAt(j)).implicated;
						if (!generalizationOfOld
								.equalsIgnoreCase(generalizationOfNew)) {
							newDeed.generalizations.addElement(new Relation(
									generalizationOfOld, false, false));
						}
					}
				}
				// Mark summarized deeds.
				aDeed.isSummarized = true;
				newDeed.summarizedActions.addElement(new Relation(
						aDeed.shortName, false, false));
			}
			abbreviationField.setText("");
			Ethno.controls.rebuildEventsChoice();
			// Ethno.needNewGraph = true;
			Ethno.graphCard.theGraph.update(Ethno.graphCard.theGraph
					.getGraphics());
		} // End of preparing new summary.
	} // End incorporateNewDeed.

	/** Clean up from generalizing and exit from the dialog. */
	void exitGeneralizing() {
		incorporateGeneralizationDescription(generalizedDeedsChoice
				.getSelectedItem());
		Ethno.graphCard.theGraph.repaint();
		Ethno.controls.rebuildEventsChoice();
		Ethno.controls.rebuildOperationsChoice();
		cancel();
	} // exitGeneralizing.

	/** Add or delete an instantiation link for a generalized event. */
	void changeInstantiation() {
		int index;
		Deed concreteDeed, generalizedDeed, aDeed;
		if (generalizedDeedsChoice.getItemCount() == 0) { return; }
		String concreteName = concreteDeedsChoice.getSelectedItem();
		index = Ethno.deeds.findPositionOfElement(concreteName);
		concreteDeed = (Deed) Ethno.deeds.elementAt(index);
		String generalizationName = generalizedDeedsChoice.getSelectedItem();
		Relation implies = new Relation(generalizationName, false, false);
		String instantiateOrDeleteLinkButtonName = Ethno.interfaceTexts
				.getString("deleteLinkText");
		if (instantiatesButton.getLabel().equalsIgnoreCase(
				instantiateOrDeleteLinkButtonName)) {
			// Delete the existing link.
			for (int i = 0; i < concreteDeed.generalizations.size(); i++) {
				if (generalizationName
						.equalsIgnoreCase(((Relation) concreteDeed.generalizations
								.elementAt(i)).implicated)) {
					concreteDeed.generalizations.removeElementAt(i);
					Ethno.linkingHistory = Ethno.linkingHistory
							+ concreteDeed.shortName
							+ Ethno.interfaceTexts
									.getString("cutInstantiation")
							+ generalizationName + Ethno.interfaceTexts
							.getString("newLine");
					Ethno.defineLinksCard.elicitationLog
							.setText(Ethno.linkingHistory);
					break;
				}
			}
			instantiateOrDeleteLinkButtonName = Ethno.interfaceTexts
					.getString("instantiatesButtonText");
			// Delete the generalized deed after each instance of concrete deed
			// in sequence.
			for (int i = Ethno.sequence.size() - 1; i >= 0; i--) {
				aDeed = (Deed) Ethno.sequence.elementAt(i);
				if (aDeed == concreteDeed) {
					Ethno.sequence.removeElementAt(i + 1);
				}
			}
		} else {
			// Add the generalization link.
			concreteDeed.generalizations.addElement(implies);
			Ethno.linkingHistory = Ethno.linkingHistory
					+ concreteDeed.shortName
					+ Ethno.interfaceTexts.getString("addInstantiation")
					+ generalizationName + Ethno.interfaceTexts
					.getString("newLine");
			Ethno.defineLinksCard.elicitationLog.setText(Ethno.linkingHistory);
			instantiateOrDeleteLinkButtonName = Ethno.interfaceTexts
					.getString("deleteLinkText");
			index = Ethno.deeds.findPositionOfElement(generalizationName);
			generalizedDeed = (Deed) Ethno.deeds.elementAt(index);
			generalizedDeed.isInstantiated = true;
			// Add the generalized deed after each instance of concrete deed in
			// sequence.
			for (int i = Ethno.sequence.size() - 1; i >= 0; i--) {
				aDeed = (Deed) Ethno.sequence.elementAt(i);
				if (aDeed == concreteDeed) {
					Ethno.sequence.insertElementAt(generalizedDeed, i + 1);
				}
			}
		}
		instantiatesButton.invalidate();
		instantiatesButton.setLabel(instantiateOrDeleteLinkButtonName);
		instantiatesButton.validate();
		Ethno.graphCard.theGraph.repaint();
		Ethno.controls.rebuildEventsChoice();
	} // End changeInstantiation.

	/** Respond to selection within lists of concrete or generalized events. */
	void setUpSelectionOfConcreteAndGeneralizedDeeds() {
		Deed concreteDeed, generalizedDeed;
		Relation r;
		int index;
		String buttonName, concreteName, generalizationName;
		buttonName = Ethno.interfaceTexts.getString("instantiatesButtonText");
		// Deal with new selection of concrete deed.
		concreteName = concreteDeedsChoice.getSelectedItem();
		index = Ethno.deeds.findPositionOfElement(concreteName);
		concreteDeed = (Deed) Ethno.deeds.elementAt(index);
		concreteEventDescription.setText(concreteDeed.verboseEventName);
		// Deal with new selection of generalized deed.
		if (generalizedDeedsChoice.getItemCount() > 0) {
			incorporateGeneralizationDescription(priorGeneralizationName);
			// Set up newly selected deed.
			generalizationName = generalizedDeedsChoice.getSelectedItem();
			priorGeneralizationName = generalizationName;
			index = Ethno.deeds.findPositionOfElement(generalizationName);
			generalizedDeed = (Deed) Ethno.deeds.elementAt(index);
			generalizedEventDescription
					.setText(generalizedDeed.verboseEventName);
			// Determine if concrete deed instantiates generalized deed.
			for (int i = 0; i < concreteDeed.generalizations.size(); i++) {
				r = (Relation) concreteDeed.generalizations.elementAt(i);
				if (generalizationName.equals(r.implicated)) {
					buttonName = Ethno.interfaceTexts
							.getString("deleteLinkText");
				}
			}
		}
		instantiatesButton.setLabel(buttonName);
		concreteEventDescription.invalidate();
		generalizedEventDescription.invalidate();
		instantiatesButton.getParent().invalidate();
		instantiatesButton.getParent().validate();
	} // End setUpSelectionOfConcreteAndGeneralizedDeeds.

	/** Select the events in a summarizable set of events. */
	void showSummarizableSet() {
		Deed aDeed;
		// Deselect the previous summary set on the graph.
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			aDeed = (Deed) Ethno.deeds.elementAt(i);
			aDeed.isSelected = false;
		}
		// Highlight the selected summary set.
		int index = summarizableSequencesBox.getSelectedIndex();
		Vector aVector = (Vector) setsOfSummarizableSequences.elementAt(index);
		for (int j = 0; j < aVector.size(); j++) {
			aDeed = (Deed) aVector.elementAt(j);
			aDeed.isSelected = true;
		}
		Ethno.graphCard.theGraph.paint(Ethno.graphCard.theGraph.getGraphics());
	} // End showSummarizableSet().

	/**
	 * Incorporate description for prior selected event, because the description
	 * may have been modified by user.
	 */
	void incorporateGeneralizationDescription(String generalizationName) {
		int index = Ethno.deeds.findPositionOfElement(generalizationName);
		if (index >= 0) {
			Deed generalizedDeed = (Deed) Ethno.deeds.elementAt(index);
			generalizedDeed.verboseEventName = generalizedEventDescription
					.getText();
		}
	}

} // End AbstractionDialog.
