
import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;

/** Dialog for setting constraints when analyzing an element's associations. */
public class AssociationsDialog extends Dialog {

	Panel dialogPanel = new Panel();

	Panel pLeft = new Panel();

	Panel pRight = new Panel();

	Panel leftBoxes = new Panel();

	Panel rightBoxes = new Panel();

	Panel pBottom = new Panel();

	Panel instructionPanel = new Panel();

	Label instruction1 = new Label();

	Label instruction2 = new Label();

	Label instruction3 = new Label();

	Label leftTitle = new Label();

	Label rightTitle = new Label();

	Checkbox[] leftCheckbox, rightCheckbox;

	Button goBack = new Button();

	public AssociationsDialog(Frame frame, String title, boolean modal) {
		super(frame, title, modal);
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		try {
			jbInit();
			add(dialogPanel);
			pack();
			Point upperLeftCorner = frame.getLocation();
			Dimension parentSize = frame.getSize();
			Dimension dialogSize = this.getSize();
			upperLeftCorner.translate(
				(parentSize.width - dialogSize.width) / 2,
				(parentSize.height - dialogSize.height) / 2);
			this.setLocation(upperLeftCorner);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	public AssociationsDialog(Frame frame) {
		this(frame, "", false);
	}

	void jbInit() throws Exception {
		dialogPanel.setLayout(new BorderLayout(10, 10));
		pLeft.setLayout(new BorderLayout(10, 10));
		pRight.setLayout(new BorderLayout(10, 10));
		leftBoxes.setLayout(new GridLayout(8, 1, 5, 5));
		rightBoxes.setLayout(new GridLayout(8, 1, 5, 5));
		pBottom.setLayout(new FlowLayout(FlowLayout.CENTER, 15, 15));
		dialogPanel.setBackground(Color.yellow);

		// Put the message label at the top of the window.
		// message = new
		// MultiLineLabel(esa2.interfaceTexts.getString("instruction"));
		dialogPanel.add("North", instructionPanel);
		instructionPanel.setLayout(new BorderLayout());
		instructionPanel.add("North", instruction1);
		instructionPanel.add("Center", instruction2);
		instructionPanel.add("South", instruction3);
		instruction1.setText(Ethno.interfaceTexts
			.getString("instructionConstrainDialog1"));
		instruction2.setText(Ethno.interfaceTexts
			.getString("instructionConstrainDialog2"));
		instruction3.setText(Ethno.interfaceTexts
			.getString("instructionConstrainDialog3"));
		instruction1.setAlignment(Label.CENTER);
		instruction2.setAlignment(Label.CENTER);
		instruction3.setAlignment(Label.CENTER);
		// Create two panels of choices, on the left and right of the window.
		dialogPanel.add("West", pLeft);
		dialogPanel.add("East", pRight);
		leftTitle.setText(Ethno.interfaceTexts.getString("leftTitle"));
		rightTitle.setText(Ethno.interfaceTexts.getString("rightTitle"));
		pLeft.add("North", leftTitle);
		pRight.add("North", rightTitle);
		pLeft.add("Center", leftBoxes);
		pRight.add("Center", rightBoxes);
		leftCheckbox = new Checkbox[8];
		rightCheckbox = new Checkbox[8];
		for (int i = 0; i < 8; i++) {
			leftCheckbox[i] = new Checkbox(Ethno.eventFrameName[i]);
			rightCheckbox[i] = new Checkbox(Ethno.eventFrameName[i]);
			leftBoxes.add(leftCheckbox[i]);
			rightBoxes.add(rightCheckbox[i]);
			leftCheckbox[i]
				.setState(Ethno.showElementCard.wantFocusElementInFrameCategory[i]);
			rightCheckbox[i]
				.setState(Ethno.showElementCard.wantOtherElementInFrameCategory[i]);
		}
		// Create bottom panel for centered return button.
		dialogPanel.add("South", pBottom);
		goBack.setLabel(Ethno.interfaceTexts.getString("continueButtonLabel"));
		pBottom.add(goBack);

		goBack.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeDialog();
			}
		});
	} // End jbInit.

	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			dispose();
		}
		super.processWindowEvent(e);
	}

	void removeDialog() {
		if (cleanedUp()) {
			dispose();
		} else {
			// Tell user to check each side.
			this.invalidate();
			instruction1.setText(Ethno.interfaceTexts
				.getString("constrainingDialogErrorMsg"));
			instruction2.setText("");
			instruction3.setText("");
			this.validate();
		}
	} // End removeDialog.

	public boolean cleanedUp() {
		boolean state1, state2, test1, test2;
		state1 = false;
		state2 = false;
		test1 = false;
		test2 = false;
		for (int i = 0; i < 8; i++) {
			// Must have at least one check on each side.
			state1 = leftCheckbox[i].getState();
			state2 = rightCheckbox[i].getState();
			test1 = test1 || state1;
			test2 = test2 || state2;
			// Store the results on the ShowElement card.
			Ethno.showElementCard.wantFocusElementInFrameCategory[i] = state1;
			Ethno.showElementCard.wantOtherElementInFrameCategory[i] = state2;
		}
		return (test1 && test2);
	} // End cleanedUp.

}
