
import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.event.MouseEvent;

/** Produce a graph of event implications. */
public class Chart extends Canvas {

	static boolean showingNamelessNodes;

	/* Using printing format. */
	static boolean nowPrinting = false;

	static boolean dragging = false;

	PerceptList graphNodes;

	int fontHeight;

	private final int TIER_HEIGHT = 60;

	private final int STANDARD_WIDTH = 760;

	private final int STANDARD_HEIGHT = 500;

	private final int THUMBNAIL_WIDTH = 200;

	private final int THUMBNAIL_HEIGHT = 280;

	private final int TAIL_SIZE_ON_UNLINKED_GENERALIZATION = 15;

	private final int VERBOSE_NAME_LINE_LENGTH = 60;

	private final int MINIMUM_NAME_WIDTH = 5; // For calculating chart width

	private final int NAME_SPACING = 4;

	private final int FONT_SIZE = 11;

	private final String FONT_NAME = "SansSerif";

	public Chart() { // Constructor method.
		super();
		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
	} // End constructor.

	private void jbInit() throws Exception {
	}

	/** Set the size of this chart. */
	public Dimension getPreferredSize() {
		int width =
			Math.max(STANDARD_WIDTH,
				MINIMUM_NAME_WIDTH * Ethno.deeds.getMaxEventsOnAnyTier());
		int height = STANDARD_HEIGHT;
		if (amDrawingOnChartPage()) {
			if (nowPrinting) {
				return new Dimension(ControlMenus.printWidth,
					ControlMenus.printHeight);
			} else {
				height =
					Math.max(height, Ethno.deeds.getNumberOfTiers()
						* TIER_HEIGHT);
				width =
					Math.max(width, Ethno.deeds.getMaxEventsOnAnyTier()
						* Ethno.deeds.getAverageNameLength());
				return new Dimension(width, height);
			}
		} else {
			return new Dimension(THUMBNAIL_WIDTH, THUMBNAIL_HEIGHT);
		}
	}

	/** Position each deed on the chart. */
	void setNodePositions(PerceptList topoSortedNodes) {
		Deed focalDeed;
		FontMetrics fontM;
		int fontStyle;
		if (Ethno.deeds.size() < 2) return;

		int numberOfTiers = ((Deed) topoSortedNodes.lastElement()).tier;
		int maxNodesOnTier = Ethno.deeds.getMaxEventsOnAnyTier();
		int rowIncrement = this.getPreferredSize().height / (numberOfTiers);
		for (int i = 0; i < topoSortedNodes.size(); i++) {
			focalDeed = (Deed) topoSortedNodes.elementAt(i);
			// Vertical.
			if (focalDeed.isGeneralization) {
				// Place generalizations between concrete lines.
				focalDeed.verticalPosition = rowIncrement * focalDeed.tier;
			} else {
				focalDeed.verticalPosition =
					rowIncrement / 2 + (rowIncrement * (focalDeed.tier - 1));
			}
			// Set the half-width of each node.
			if (showingNamelessNodes) {
				focalDeed.halfNameWidth =
					THUMBNAIL_WIDTH / (2 * maxNodesOnTier);
			} else { // Drawing a graph with named nodes.
				fontStyle = Font.PLAIN;
				if (focalDeed.isDisjunctive) {
					fontStyle = fontStyle | Font.ITALIC;
				}
				if (focalDeed.isGeneralization) {
					fontStyle = fontStyle | Font.BOLD;
				}
				fontM =
					this.getFontMetrics(new Font(FONT_NAME, fontStyle, this
						.getFont().getSize()));
				focalDeed.halfNameWidth =
					(NAME_SPACING + fontM.stringWidth(focalDeed.shortName)) / 2;
			}
		}
	} // End setNodePositions.

	/** Paint the chart. */
	public void paint(Graphics g) {
		Deed deed1, deed2;
		int index, offset;
		Relation r;
		if (Ethno.deeds.size() < 2) { return; }
		g.setFont(new Font("SansSerif", Font.PLAIN, FONT_SIZE));
		if ((Ethno.deeds.size() > 0) && (!dragging)) {
			// Prepare the data.
			// Create topographically-ordered list, including dummy nodes.
			graphNodes = toposortedDeedsWithDummies();
			// Rank nodes in each tier for minimum crossed lines.
			graphNodes.baryRank();
			if ((ControlMenus.graphMode == ControlMenus.TESTING_IN_GRAPH)
				|| (ControlMenus.graphMode == ControlMenus.EXPERIMENTING_IN_GRAPH)) {
				// Show potential deeds in yellow:
				Ethno.graphCard.findFeasibleDeeds();
				if (ControlMenus.graphMode != ControlMenus.EXPERIMENTING_IN_GRAPH) {
					// Show initially occurring event in red.
					deed1 = (Deed) Ethno.sequence.elementAt(0);
					deed1.isInProcess = true;
					for (int i = 0; i < deed1.generalizations.size(); i++) {
						// Along with its generalizations.
						r = (Relation) deed1.generalizations.elementAt(i);
						index = Ethno.deeds.findPositionOfElement(r.implicated);
						((Deed) Ethno.deeds.elementAt(index)).isInProcess =
							true;
					}
				}
			}
			setNodePositions(graphNodes);
			graphNodes.baryPositionDeeds(this.getPreferredSize().width,
				this.getPreferredSize().height, showingNamelessNodes);
		}
		dragging = false;
		// Draw the lines.
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			deed1 = (Deed) Ethno.deeds.elementAt(i);
			if (deed1.isSummarized) {
				continue;
			}
			for (int j = 0; j < deed1.implications.size(); j++) {
				r = (Relation) deed1.implications.elementAt(j);
				index = Ethno.deeds.findPositionOfElement(r.implicated);
				deed2 = (Deed) Ethno.deeds.elementAt(index);
				if (deed2.isSummarized) {
					continue;
				}
				if (deed1.isLinked && deed2.isLinked) {
					if (r.commutes) {
						g.setColor(Color.magenta);
						offset = 3;
					} else {
						g.setColor(Color.black);
						offset = 0;
					}
					g.drawLine(deed1.horizontalPosition + offset,
						deed1.verticalPosition, deed2.horizontalPosition
							+ offset, deed2.verticalPosition);
				}
			}
			for (int j = 0; j < deed1.generalizations.size(); j++) {
				r = (Relation) deed1.generalizations.elementAt(j);
				index = Ethno.deeds.findPositionOfElement(r.implicated);
				deed2 = (Deed) Ethno.deeds.elementAt(index);
				g.setColor(Color.orange);
				offset = 0;
				if (deed1.isSummarized || deed2.isSummarized) {
					continue;
				}
				if (deed1.isLinked && deed2.isLinked) {
					g.drawLine(deed1.horizontalPosition + offset,
						deed1.verticalPosition, deed2.horizontalPosition
							+ offset, deed2.verticalPosition);
				} else {
					// Show a tag on generalized items until linking shows
					// lines.
					g.drawLine(deed1.horizontalPosition,
						deed1.verticalPosition, deed1.horizontalPosition
							+ TAIL_SIZE_ON_UNLINKED_GENERALIZATION,
						deed1.verticalPosition
							- TAIL_SIZE_ON_UNLINKED_GENERALIZATION);
				}
			}
		}
		// Write the names.
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			deed1 = (Deed) Ethno.deeds.elementAt(i);
			if (deed1.isLinked) {
				printNameOfDeed(deed1, g);
			}
		}
	} // End paint.

	/** Print the names of all nodes. */
	void displayAllNodes() {
		Deed theDeed;
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			theDeed = (Deed) Ethno.deeds.elementAt(i);
			if (theDeed.isLinked) {
				printNameOfDeed(theDeed, this.getGraphics());
			}
		}
	}

	/** Format and print the deed's name on the chart. */
	void printNameOfDeed(Deed theDeed, Graphics g) {
		Font f = g.getFont();
		if (theDeed.isSummarized || theDeed.isADummy) return;
		Font priorFont;
		FontMetrics fontM;
		int fontStyle, fontHalfHeight;
		Color priorForeGrnd, foreColor, fillColor;
		priorForeGrnd = this.getForeground();
		priorFont = g.getFont();
		foreColor = Color.black;
		fillColor = this.getBackground();
		fontStyle = Font.PLAIN;
		if (theDeed.isInProcess) {
			foreColor = Color.white;
			fillColor = Color.red;
		} else if (theDeed.isAccomplished) {
			foreColor = Color.black;
			fillColor = Color.cyan;
		} else if (theDeed.isFeasible) {
			foreColor = Color.black;
			fillColor = Color.yellow;
		}
		if (theDeed.isSelected) {
			foreColor = Color.white;
			fillColor = Color.black;
		}
		if (theDeed.isDisjunctive) {
			fontStyle = Font.ITALIC;
		}
		if (theDeed.isGeneralization) {
			fontStyle = fontStyle | Font.BOLD;
			fillColor = Color.lightGray;
		}
		g.setFont(new Font(f.getName(), fontStyle, FONT_SIZE));
		fontM = g.getFontMetrics();
		fontHalfHeight = (fontM.getHeight() / 2) - fontM.getDescent();
		fontHeight = fontM.getHeight();
		// Truncate name to avoid overlapping with neighbor on right.
		String truncatedName = theDeed.shortName;
		int stringLength = truncatedName.length() + 1;
		// Truncate the event's short name to fit in name box.
		while ((--stringLength > 0)
			& (fontM.stringWidth(truncatedName.substring(0, stringLength)) > ((2 * theDeed.halfNameWidth) - NAME_SPACING)))
			;
		if (stringLength < 1) {
			truncatedName = "*";
		} else {
			truncatedName = truncatedName.substring(0, stringLength);
		}
		if (amDrawingOnChartPage()) {
			// Represent the node with the deed's short name..
			g.setColor(fillColor);
			g.fillRect(theDeed.horizontalPosition - theDeed.halfNameWidth,
				theDeed.verticalPosition - (fontM.getHeight() / 2), 2
					* (theDeed.halfNameWidth) - Math.round(NAME_SPACING / 2),
				fontHeight);
			g.setColor(foreColor);
			g.drawString(truncatedName, theDeed.horizontalPosition
				- theDeed.halfNameWidth + NAME_SPACING,
				theDeed.verticalPosition + fontHalfHeight);
		} else { // Working on the link-events form.
			if (theDeed.isSelected) {
				g.setColor(Color.green);
			} else if (theDeed.isAnswered) {
				g.setColor(Color.orange);
			} else {
				if (fillColor == this.getBackground()) {
					g.setColor(Color.black);
				} else {
					g.setColor(fillColor);
				}
			}
			// Mark nodes with filled squares on thumbnail graph.
			if (!theDeed.isGeneralization) {
				// This is a concrete event.
				if (fontStyle == Font.PLAIN) { // Conjunctive
					g.fillRect(theDeed.horizontalPosition - 2,
						theDeed.verticalPosition - 2, 4, 4);
				} else {
					g.fillArc(theDeed.horizontalPosition - 4,
						theDeed.verticalPosition - 4, 8, 8, 0, -180);
				}
			} else {
				// This is an abstract event.
				if (fontStyle == Font.PLAIN) { // Conjunctive
					g.drawRect(theDeed.horizontalPosition - 2,
						theDeed.verticalPosition - 2, 4, 4);
				} else {
					g.drawArc(theDeed.horizontalPosition - 4,
						theDeed.verticalPosition - 4, 8, 8, 0, -180);
				}
			}
		}
		g.setFont(priorFont);
		g.setColor(priorForeGrnd);
	} // End showNode.

	/**
	 * Create a toposorted list of linked deeds, including dummy nodes in
	 * skipped tiers.
	 */
	PerceptList toposortedDeedsWithDummies() {
		PerceptList topoSortedNodes;
		Deed focalDeed, theOtherDeed, chainedDeed, lastChainedDeed, testDeed;
		Relation r;
		int nodeCount, index, insertionPoint;
		topoSortedNodes = new PerceptList();
		for (int i = 0; i < Ethno.deeds.size(); i++) { // Assemble linked
			// deeds.
			focalDeed = (Deed) Ethno.deeds.elementAt(i);
			if (focalDeed.isSummarized) {
				continue; // Summarized deeds are not visible.
			}
			if (focalDeed.isLinked) {
				// Display only linked deeds.
				topoSortedNodes.addElement(focalDeed);
			}
		}
		// Define the hierarchy of linked deeds:
		topoSortedNodes = topoSortedNodes.topoSort();
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			focalDeed = (Deed) Ethno.deeds.elementAt(i);
			if (focalDeed.isSummarized) {
				continue; // Summarized deeds are not visible.
			}
			// Clear pointers to connected nodes.
			focalDeed.upNodes.removeAllElements();
			focalDeed.downNodes.removeAllElements();
		}
		// Define up and down links, and divide multi-tier links with dummy
		// deeds.
		nodeCount = -1;
		while (++nodeCount < topoSortedNodes.size()) {
			focalDeed = (Deed) topoSortedNodes.elementAt(nodeCount);
			for (int j = 0; j < focalDeed.implications.size(); j++) {
				r = (Relation) focalDeed.implications.elementAt(j);
				index = Ethno.deeds.findPositionOfElement(r.implicated);
				theOtherDeed = (Deed) Ethno.deeds.elementAt(index);
				lastChainedDeed = theOtherDeed;
				for (int iTier = theOtherDeed.tier; iTier < focalDeed.tier; iTier++) {
					if (iTier == (focalDeed.tier - 1)) {
						// Connect to theOtherDeed.
						lastChainedDeed.downNodes.addElement(focalDeed);
						focalDeed.upNodes.addElement(lastChainedDeed);
					} else {
						// Insert a dummy deed.
						chainedDeed = new Deed();
						chainedDeed.isADummy = true;
						chainedDeed.tier = iTier + 1;
						chainedDeed.upNodes.addElement(lastChainedDeed);
						lastChainedDeed.downNodes.addElement(chainedDeed);
						chainedDeed.halfNameWidth = 1;
						for (insertionPoint = 0; insertionPoint < topoSortedNodes
							.size(); insertionPoint++) {
							testDeed =
								(Deed) topoSortedNodes
									.elementAt(insertionPoint);
							if (testDeed.tier == chainedDeed.tier) {
								break;
							}
						}
						topoSortedNodes.insertElementAt(chainedDeed,
							insertionPoint);
						lastChainedDeed = chainedDeed;
						// Keep the index on focalDeed -- dummy always is prior:
						nodeCount++;
					}
				}
			}
		}
		return topoSortedNodes;
	} // End toposortedDeedsWithDummies.

	/**
	 * CTRL-Clicking a node pops a dialog showing the deed description, if there
	 * is one.
	 */
	void showFullNameInDialog(MouseEvent e) {
		Point p = e.getPoint();
		Rectangle r;
		Deed trialDeed;
		String longName1, longName2;
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			trialDeed = (Deed) Ethno.deeds.elementAt(i);
			r =
				new Rectangle(trialDeed.horizontalPosition - 10,
					trialDeed.verticalPosition - 6, 20, 10);
			if (r.contains(p)) {
				if (trialDeed.verboseEventName.length() > 0) {
					if ((trialDeed.verboseEventName.length() > VERBOSE_NAME_LINE_LENGTH)
						& (trialDeed.verboseEventName.indexOf(" ",
							VERBOSE_NAME_LINE_LENGTH) > 0)) {
						int breakPoint =
							trialDeed.verboseEventName.indexOf(" ",
								VERBOSE_NAME_LINE_LENGTH);
						longName1 =
							trialDeed.verboseEventName.substring(0, breakPoint);
						longName2 =
							trialDeed.verboseEventName.substring(breakPoint);
					} else {
						longName1 = "";
						longName2 = trialDeed.verboseEventName;
					}
				} else {
					longName1 = "";
					longName2 = "";
				}
				CancelOkDialog fullName =
					new CancelOkDialog((Frame) this.getParent().getParent().getParent().getParent(),
						trialDeed.shortName, longName1, longName2);
				fullName.setVisible(true);
				break;
			}
		}
	} // End showFullNameInDialog.

	/** True if the task now is GRAPH. */
	boolean amDrawingOnChartPage() {
		return (Ethno.theTaskNow == Ethno.GRAPH);
	} // End drawingBigChart.

}