
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Event;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.Rectangle;
import java.awt.ScrollPane;
import java.awt.event.MouseEvent;
import java.util.Vector;

/** Page showing graph of event implications in a scroll window. */
public class ChartCard extends Panel {

	static boolean automaticEvents = false;

	static boolean adoptedSomeSolution;

	Deed lastClickedDeed;

	int priorSelectedNode = -1;

	boolean primed, experimenting;

	ScrollPane scrollWindow = new ScrollPane(ScrollPane.SCROLLBARS_AS_NEEDED);

	Chart theGraph = new Chart();

	BorderLayout graphPageBorderLayout = new BorderLayout();

	public ChartCard() {
		try {
			jbInit();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void jbInit() throws Exception {
		this.setLayout(graphPageBorderLayout);
		scrollWindow.setBackground(Color.white);
		this.add(scrollWindow, BorderLayout.CENTER);
		scrollWindow.add(theGraph, null);
		Chart.showingNamelessNodes = false;
		automaticEvents = false;
		experimenting = false;
		theGraph
			.addMouseMotionListener(new java.awt.event.MouseMotionAdapter() {
				public void mouseDragged(MouseEvent e) {
					moveDeedOnGraph(e);
				}

				public void mouseMoved(MouseEvent e) {
				}
			});
		theGraph.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
			}

			public void mouseEntered(MouseEvent e) {
			}

			public void mouseExited(MouseEvent e) {
			}

			public void mousePressed(MouseEvent e) {
				lastClickedDeed = identifyClickedDeed(e);
			}

			public void mouseReleased(MouseEvent e) {
				respondToMouseClick(e);
			}
		});
	} // End jbInit.

	/** Reset the variables used to test a model. */
	static void initializeGraphVariables() {
		Deed theDeed;
		// Must not change graph variables while testing a model:
		automaticEvents = false;
		Ethno.currentEvent = 0;
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			theDeed = (Deed) Ethno.deeds.elementAt(i);
			theDeed.isAccomplished = false;
			theDeed.isFeasible = false;
			theDeed.isInProcess = false;
			theDeed.isSelected = false;
		}
	} // End initializeGraph.

	/**
	 * Accomplish the last deed, and set up the next deed if the series is not
	 * ended.
	 */
	public void moveAheadInSequence() {
		Deed occurringDeed, nextEvent;
		if (experimenting) {
			occurringDeed = lastClickedDeed;
		} else {
			occurringDeed = (Deed) Ethno.sequence.elementAt(Ethno.currentEvent);
		}
		// Make last event accomplished, not inProcess, prerequisites depleted.
		performDeed(occurringDeed);
		findFeasibleDeeds();
		this.theGraph.displayAllNodes();
		if (experimenting) { return; }
		++Ethno.currentEvent; // Go to next event in sequence.
		for (int i = 0; i < occurringDeed.generalizations.size(); i++) {
			// After generalized occurrences.
			++Ethno.currentEvent;
		}
		if (Ethno.currentEvent == Ethno.sequence.size()) {
			// Finished testing.
			CancelOkDialog done =
					new CancelOkDialog((Frame) this.getParent().getParent(),
					Ethno.interfaceTexts.getString("finishedAllText"),
					Ethno.interfaceTexts.getString(""),
					Ethno.interfaceTexts.getString("produceStatisticsText"));
			done.setVisible(true);
			produceEpisodeStatistics();
			initializeGraphVariables();
			theGraph.repaint();
		} else {
			// Continue processing events.
			nextEvent = (Deed) Ethno.sequence.elementAt(Ethno.currentEvent);
			// Make deed in Process if feasible, else RefineModelDialog.
			startDoing(nextEvent);
		}
	} // End moveAheadInSequence.

	/**
	 * If the current deed is feasible show it and its generalizations as being
	 * in process; otherwise show the RefineModelDialog.
	 */
	public void startDoing(Deed theDeed) {
		Deed generalizedDeed;
		Relation r;
		int index, priorPositionInSequence;
		if (theDeed.isFeasible) {
			theDeed.isInProcess = true;
			theGraph.printNameOfDeed(theDeed, theGraph.getGraphics());
			// Do generalized deeds that this instantiates.
			for (int i = 0; i < theDeed.generalizations.size(); i++) {
				r = (Relation) theDeed.generalizations.elementAt(i);
				index = Ethno.deeds.findPositionOfElement(r.implicated);
				generalizedDeed = (Deed) Ethno.deeds.elementAt(index);
				startDoing(generalizedDeed); // Recursive.
			}
		} else {
			RefineModelDialog changeModelDialog =
				new RefineModelDialog((Frame) this.getParent().getParent(), theDeed);
			changeModelDialog.setVisible(true);
			theGraph.repaint();
			automaticEvents = false;
			if (adoptedSomeSolution) {
				adoptedSomeSolution = false;
				priorPositionInSequence = Ethno.currentEvent;
				initializeGraphVariables();
				findFeasibleDeeds();
				this.theGraph.displayAllNodes();
				if (!experimenting) {
					// Redo events through Ethno.currentEvent.
					for (int i = 0; i < priorPositionInSequence; i++) {
						respondToMouseClick(new MouseEvent(this, 0, 0, 0, 0, 0,
							0, false));
						this.theGraph.displayAllNodes();
					}
				}
			} else {
				// Allow user to try the problem event again.
				if (experimenting) {
					this.theGraph.repaint();
				} else {
					Ethno.currentEvent--;
				}
				this.theGraph.displayAllNodes();
			}
		}
	} // End startDoing.

	/** Find deeds that are adequately primed and not blocked. */
	public void findFeasibleDeeds() {
		Deed theDeed, impliedDeed, consequentDeed, occurringDeed;
		Relation r;
		boolean conjunction;
		int index;
		if (experimenting) {
			occurringDeed = lastClickedDeed;
		} else {
			occurringDeed = (Deed) Ethno.sequence.elementAt(Ethno.currentEvent);
		}
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			// Examine the feasibility of every deed.
			theDeed = (Deed) Ethno.deeds.elementAt(i);
			theDeed.isBlocked = theDeed.isAccomplished & !theDeed.isRepeatable;
			primed = false;
			if ((theDeed.implications.size() == 0)) {// &
				// theDeed.isRepeatable)
				// {
				primed = true; // Repeatable exogenous always feasible.
			}
			// Gets consequent deeds:
			DeedWithLinks theLinkedDeed = new DeedWithLinks(theDeed);
			// A deed with no consequences cannot be used or unused:
			theDeed.isBlocked =
				theDeed.isBlocked & (theLinkedDeed.downLinks.size() != 0);
			for (int j = 0; j < theLinkedDeed.downLinks.size(); j++) {
				// Allow flip-flops to prime backwards when upper node is
				// disjunctive.
				consequentDeed = (Deed) theLinkedDeed.downLinks.elementAt(j);
				for (int k = 0; k < consequentDeed.implications.size(); k++) {
					r = (Relation) consequentDeed.implications.elementAt(k);
					if (r.implicated.equalsIgnoreCase(theDeed.shortName)) {
						if (theDeed.isDisjunctive & r.commutes
							& consequentDeed.isAccomplished) {
							primed = true;
						}
					}
				}
			}
			// Deed is feasible if immediate prerequisites are accomplished, or
			// occurring now.
			conjunction = true;
			for (int j2 = 0; j2 < theDeed.implications.size(); j2++) {
				r = (Relation) theDeed.implications.elementAt(j2);
				index = Ethno.deeds.findPositionOfElement(r.implicated);
				impliedDeed = (Deed) Ethno.deeds.elementAt(index);
				if (theDeed.isDisjunctive
					& (impliedDeed.isAccomplished | impliedDeed == occurringDeed)) {
					// One priming is enough for disjunctive.
					primed = true;
					break;
				}
				conjunction =
					conjunction
						& (impliedDeed.isAccomplished | impliedDeed == occurringDeed);
			}
			primed = primed | conjunction;
			// Conclusion.
			theDeed.isFeasible = primed & !theDeed.isBlocked;
		}
	} // End feasibleDeeds.

	/**
	 * Perform the deed by depleting its prerequisites and by performing its
	 * generalizations.
	 */
	public void performDeed(Deed theDeed) {
		Deed impliedDeed, generalizedDeed, aDeed;
		Relation r;
		int index, occurringDeedIndex;
		theDeed.isAccomplished = true;
		theDeed.isInProcess = false;
		theDeed.isSelected = false;
		// Do event counts.
		occurringDeedIndex =
			Ethno.deeds.findPositionOfElement(theDeed.shortName);
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			aDeed = (Deed) Ethno.deeds.elementAt(i);
			if (aDeed.isFeasible
				&& !Deed
					.deedsAreAtDifferentGeneralizationLevels(theDeed, aDeed)) {
				++Ethno.eventCounts[occurringDeedIndex][i];
			}
		}
		// Deplete immediate prerequisites.
		for (int i = 0; i < theDeed.implications.size(); i++) {
			r = (Relation) theDeed.implications.elementAt(i);
			index = Ethno.deeds.findPositionOfElement(r.implicated);
			impliedDeed = (Deed) Ethno.deeds.elementAt(index);
			if (r.depletes) {
				impliedDeed.isAccomplished = false;
			}
		}
		// Look for a commuter that is depleted.
		for (int j = 0; j < Ethno.deeds.size(); j++) {
			aDeed = (Deed) Ethno.deeds.elementAt(j);
			for (int k = 0; k < aDeed.implications.size(); k++) {
				r = (Relation) aDeed.implications.elementAt(k);
				index = Ethno.deeds.findPositionOfElement(r.implicated);
				impliedDeed = (Deed) Ethno.deeds.elementAt(index);
				if (r.commutes
					& impliedDeed.shortName.equals(theDeed.shortName)
					& r.depletes) {
					aDeed.isAccomplished = false;
				}
			}
		}
		// Deal with generalizations.
		for (int i = 0; i < theDeed.generalizations.size(); i++) {
			r = (Relation) theDeed.generalizations.elementAt(i);
			index = Ethno.deeds.findPositionOfElement(r.implicated);
			generalizedDeed = (Deed) Ethno.deeds.elementAt(index);
			performDeed(generalizedDeed); // Recursive.
		}
	} // End performDeed.

	/**
	 * Search through all deeds and return the one displayed at the clicked
	 * point.
	 */
	Deed identifyClickedDeed(MouseEvent e) {
		Deed aDeed;
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			aDeed = (Deed) Ethno.deeds.elementAt(i);
			Rectangle rect =
				new Rectangle(aDeed.horizontalPosition - aDeed.halfNameWidth,
					aDeed.verticalPosition - theGraph.fontHeight / 2,
					2 * aDeed.halfNameWidth, theGraph.fontHeight);
			if (rect.contains(e.getX(), e.getY())) { return aDeed; }
		}
		return null;
	} // End clickedDeed.

	/**
	 * Search through all deeds and return the index for the one displayed at
	 * the clicked point.
	 */
	int getClickedDeedIndex(MouseEvent e) {
		Deed aDeed;
		int i;
		for (i = 0; i < Ethno.deeds.size(); i++) {
			aDeed = (Deed) Ethno.deeds.elementAt(i);
			Rectangle rect =
				new Rectangle(aDeed.horizontalPosition - aDeed.halfNameWidth,
					aDeed.verticalPosition - theGraph.fontHeight / 2,
					2 * aDeed.halfNameWidth, theGraph.fontHeight);
			if (rect.contains(e.getX(), e.getY())) { return i; }
		}
		return -1;
	} // End clickedDeedIndex.

	/**
	 * Find the contextual meaning of a mouse click and initiate required
	 * actions..
	 */
	void respondToMouseClick(MouseEvent e) {
		Deed aDeed, deed1, deed2;
		Vector connectedNodes;
		int thisSelectedNode;
		boolean shiftKeyDown = (e.getModifiers() & Event.SHIFT_MASK) != 0;
		int currentGraphMode =
			Ethno.controls.graphModeChoice.getSelectedIndex();
		if (e.isControlDown()) {
			// If Control key down, show event name and description.
			theGraph.showFullNameInDialog(e);
			return;
		}
		if (currentGraphMode == ControlMenus.INSPECTING_GRAPH) {
			// Clear selections or select connected events.
			aDeed = identifyClickedDeed(e);
			if (aDeed == null) {
				// Didn't click on a node so clear all selections.
				for (int i = 0; i < Ethno.deeds.size(); i++) {
					aDeed = (Deed) Ethno.deeds.elementAt(i);
					aDeed.isSelected = false;
					theGraph.printNameOfDeed(aDeed, theGraph.getGraphics());
				}
				return;
			}
			// Select nodes connected to clicked node.
			connectedNodes = Ethno.deeds.downwardLinks(aDeed);
			for (int i = 0; i < connectedNodes.size(); i++) {
				deed1 = (Deed) connectedNodes.elementAt(i);
				deed1.isSelected = true;
				theGraph.printNameOfDeed(deed1, theGraph.getGraphics());
			}
			connectedNodes = Ethno.deeds.upwardLinks(aDeed);
			for (int i = 0; i < connectedNodes.size(); i++) {
				deed1 = (Deed) connectedNodes.elementAt(i);
				deed1.isSelected = true;
				theGraph.printNameOfDeed(deed1, theGraph.getGraphics());
			}
			aDeed.isSelected = true;
			theGraph.printNameOfDeed(aDeed, theGraph.getGraphics());
			return;
		}
		if (currentGraphMode == ControlMenus.EDITING_GRAPH) {
			// Select the deed that was clicked.
			thisSelectedNode = getClickedDeedIndex(e);
			if (thisSelectedNode >= 0) {
				aDeed = identifyClickedDeed(e);
				aDeed.isSelected = true;
				theGraph.printNameOfDeed(aDeed, theGraph.getGraphics());
			} else {
				priorSelectedNode = -1;
				initializeGraphVariables();
				theGraph.repaint();
				return;
			}
			if (priorSelectedNode == -1) {
				// Make this the first of a pair of selections.
				priorSelectedNode = thisSelectedNode;
			} else {
				// Add or delete a link between the pair of deeds.
				if (priorSelectedNode == thisSelectedNode) { return; }
				deed1 = (Deed) Ethno.deeds.elementAt(priorSelectedNode);
				deed2 = (Deed) Ethno.deeds.elementAt(thisSelectedNode);
				priorSelectedNode = -1;
				// Force an extra link if Shift down:
				RelationDialog editLink =
					new RelationDialog((Frame) this.getParent().getParent(), deed1, deed2,
						shiftKeyDown);
				editLink.setVisible(true);
			}
			return;
		}
		// User clicked the screen to test the next event.
		if (currentGraphMode == ControlMenus.TESTING_IN_GRAPH) {
			if (shiftKeyDown) {
				automaticEvents = true;
			}
			moveAheadInSequence();
			if (automaticEvents) {
				respondToMouseClick(e);
			}
		}
		// User clicked the screen to try a hypothetical event.
		if (currentGraphMode == ControlMenus.EXPERIMENTING_IN_GRAPH) {
			experimenting = true;
			lastClickedDeed = identifyClickedDeed(e);
			if (lastClickedDeed == null) {
				theGraph.repaint();
				produceEpisodeStatistics();
			} else {
				// Make deed inProcess if feasible, else RefineModelDialog.
				startDoing(lastClickedDeed);
				moveAheadInSequence();
			}
			experimenting = false;
		}
	} // End bigGraph_mouseClicked.

	/** Drag a graph node. */
	void moveDeedOnGraph(MouseEvent e) {
		if (lastClickedDeed != null) {
			lastClickedDeed.horizontalPosition = e.getX();
			lastClickedDeed.verticalPosition = e.getY();
			Chart.dragging = true;
			theGraph.repaint();
		}
	} // End bigGraph_mouseDragged.

	public void produceEpisodeStatistics() {
		Deed aDeed;
		String GENERALIZATION_MARKER = "_";
		String heading, number, endField;
		int total;
		int FIELD = 9;
		String blanks4 = "    ";
		String padding = "";
		for (int i = 0; i < FIELD; i++) {
			padding = padding + " ";
		}
		Ethno.reportPage.results.append(
			Ethno.interfaceTexts
			.getString("episodeStatisticsTitle")
			+ "\n" 
			+ Ethno.interfaceTexts
			.getString("numberOfDefinedEvents")
			+ (new Integer(Ethno.deeds.size())).toString()
			+ "\n"
			+ Ethno.interfaceTexts
			.getString("numberOfEventOccurrences")
			+ (new Integer(Ethno.sequence.size())).toString()
			+ "\n");
		// Table.
		endField = Ethno.interfaceTexts.getString("endField");
		heading = Ethno.interfaceTexts.getString("frequencyTitle");
		Ethno.reportPage.results.append(
			(padding + heading + endField).substring(heading.length()));
		heading = Ethno.interfaceTexts.getString("priorityTitle");
		Ethno.reportPage.results.append(
			(padding + heading + endField).substring(heading.length()));
		heading = blanks4 + Ethno.interfaceTexts.getString("nameTitle");
		Ethno.reportPage.results.append(
			blanks4 + heading + "\n");
		// Use the event counts from a graph testing in the narrative box.
		for (int j = 0; j < Ethno.deeds.size(); j++) { // For each column.
			total = 0;
			for (int i = 0; i < Ethno.deeds.size(); i++) {
				total = total + Ethno.eventCounts[i][j];
			}
			number = (new Integer(Ethno.eventCounts[j][j])).toString(); // Frequency.
			Ethno.reportPage.results.append(
				(padding + number + endField).substring(number.length()));
			number =
				(new Long(Math.round((double) (100 * Ethno.eventCounts[j][j])
					/ (total + 1)))).toString(); // Priority.
			Ethno.reportPage.results.append(
				(padding + number + endField).substring(number.length()));
			aDeed = (Deed) Ethno.deeds.elementAt(j);
			if (aDeed.isGeneralization) {
				Ethno.reportPage.results.append(
					(blanks4 + GENERALIZATION_MARKER
					+ aDeed.shortName + "\n"));
			} else {
				Ethno.reportPage.results.append(
					blanks4 + aDeed.shortName + "\n");
			}
		}
	}

}