
import java.awt.Choice;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Frame;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.event.ItemEvent;
import java.awt.print.Book;
import java.awt.print.PageFormat;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

/** Menus for help, operations, events, elements, and graph modes. */
public class ControlMenus extends Panel {

	static final int TITLE_LINE = 0;

	/** Corresponds to operationLineName[1] in interfaceTexts. */
	static final int DEFINE_SEQUENCE = 1;

	/** Corresponds to operationLineName[2] in interfaceTexts. */
	static final int LINK_EVENTS = 2;

	/** Corresponds to operationLineName[3] in interfaceTexts. */
	static final int WORK_WITH_CHART = 3;

	/** Corresponds to operationLineName[4] in interfaceTexts. */
	static final int IMPORT_EXPORT = 4;

	/** Corresponds to operationLineName[5] in interfaceTexts. */
	static final int ANALYSES = 5;

	/** Corresponds to operationLineName[5] in interfaceTexts. */
	static final int EXPORT_ABSTRACT_MODEL = 6;

	/** Corresponds to graphModeLine[0] in interfaceTexts. */
	static final int INSPECTING_GRAPH = 0;

	/** Corresponds to graphModeLine[1] in interfaceTexts. */
	static final int EDITING_GRAPH = 1;

	/** Corresponds to graphModeLine[2] in interfaceTexts. */
	static final int SUMMARIZING_IN_GRAPH = 2;

	/** Corresponds to graphModeLine[3] in interfaceTexts. */
	static final int GENERALIZING_IN_GRAPH = 3;

	/** Corresponds to graphModeLine[4] in interfaceTexts. */
	static final int TESTING_IN_GRAPH = 4;

	/** Corresponds to graphModeLine[5] in interfaceTexts. */
	static final int EXPERIMENTING_IN_GRAPH = 5;

	/** Corresponds to graphModeLine[6] in interfaceTexts. */
	static final int PRINTING_GRAPH = 6;

	static final int MAIN_MENU = 1;

	static final int ELEMENTS_MENU = 2;

	static final int GRAPH_MENU = 3;

	static int graphMode;

	static int printWidth, printHeight;

	String singleMenuTasks;

	Panel generalPanel = new Panel();

	Panel elementPanel = new Panel();

	Panel graphmodePanel = new Panel();

	Choice operationsChoice = new Choice();

	Choice eventsChoice = new Choice();

	Choice entitiesChoice = new Choice();

	Choice peopleChoice = new Choice();

	Choice actionsChoice = new Choice();

	Choice graphModeChoice = new Choice();

	GridBagLayout gridBagLayout1 = new GridBagLayout();

	GridLayout gridLayout1 = new GridLayout(3, 1);

	public ControlMenus() {
		try {
			jbInit();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void jbInit() throws Exception {
		this.setBackground(Color.darkGray);
		this.setLayout(gridLayout1);
		this.add(generalPanel, null);
		this.add(elementPanel, null);
		this.add(graphmodePanel, null);
		generalPanel.setBackground(Color.green);
		elementPanel.setBackground(Color.green);
		graphmodePanel.setBackground(Color.green);

		operationsChoice.setBackground(Color.white);
		eventsChoice.setBackground(Color.white);
		entitiesChoice.setBackground(Color.white);
		peopleChoice.setBackground(Color.white);
		actionsChoice.setBackground(Color.white);
		graphModeChoice.setBackground(Color.white);
		
		generalPanel.add(operationsChoice, null);
		generalPanel.add(eventsChoice, null);
		rebuildOperationsChoice();
		eventsChoice.add(Ethno.interfaceTexts.getString("eventsTitleText"));

		elementPanel.add(entitiesChoice, null);
		elementPanel.add(peopleChoice, null);
		elementPanel.add(actionsChoice, null);
		entitiesChoice.add(Ethno.interfaceTexts.getString("newEntityText"));
		peopleChoice.add(Ethno.interfaceTexts.getString("newPersonText"));
		actionsChoice.add(Ethno.interfaceTexts.getString("newActionText"));

		graphmodePanel.add(graphModeChoice, null);
		rebuildGraphModeChoice();

		entitiesChoice.setVisible(false);
		peopleChoice.setVisible(false);
		actionsChoice.setVisible(false);
		graphModeChoice.setVisible(false);

		operationsChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				showNewAnalysisPage();
			}
		});
		eventsChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				goToPageForAnEvent();
			}
		});
		entitiesChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				dealWithEntityChoice();
			}
		});
		peopleChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				dealWithPersonChoice();
			}
		});
		actionsChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				dealWithActionChoice();
			}
		});
		graphModeChoice.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				conductGraphTask();
			}
		});
	} // End jbInit.

	/** Implement user's request for a new analysis page. */
	void showNewAnalysisPage() {
		incorporateDeedOrPerceptChanges();
		int requestedTask = operationsChoice.getSelectedIndex();
		if (requestedTask == 0) { // Clicked on title.
			return;
		} else if (requestedTask == DEFINE_SEQUENCE) {
			Ethno.theTaskNow = Ethno.DEFINE_SEQUENCE;
		} else if (requestedTask == LINK_EVENTS) {
			Ethno.theTaskNow = Ethno.DEFINE_LINKS;
		} else if (requestedTask == WORK_WITH_CHART) {
			Ethno.theTaskNow = Ethno.GRAPH;
			ChartCard.initializeGraphVariables();
		} else if (requestedTask == IMPORT_EXPORT) {
			Ethno.theTaskNow = Ethno.IMPORT_EXPORT;
		} else if (requestedTask == EXPORT_ABSTRACT_MODEL) {
			Ethno.theTaskNow = Ethno.IMPORT_EXPORT;
		} else if (requestedTask == ANALYSES) {
			Ethno.theTaskNow = Ethno.ANALYSES;
		} else {
			Ethno.reportPage.results.append(
			"\n\nERROR IN OPERATIONS MENU\n\n");
		}
		eventsChoice.select(0); // Restore menu title.
		// Import-Export data.
		if ((requestedTask == IMPORT_EXPORT)
			|| (requestedTask == EXPORT_ABSTRACT_MODEL)) {
			// Save the analysis.
			boolean generalizedModel = (requestedTask == EXPORT_ABSTRACT_MODEL);
			Ethno.importExportCard.displayModelAsString(generalizedModel);
			// Show the import-export card.
			Ethno.theTaskNow = Ethno.IMPORT_EXPORT; 
		}
		// Go to Sequence card.
		if (requestedTask == DEFINE_SEQUENCE) { // Nothing required.
		}
		// Go to Linking card.
		if (requestedTask == LINK_EVENTS) {
			// Show card first as background for possible can't-link dialog.
			Ethno.card.show(Ethno.displayPanel, Ethno.theTaskNow);
			// Set up for linking.
			Ethno.defineLinksCard.startANewElicitation();
		}
		// Go to Graph card.
		if (Ethno.theTaskNow == Ethno.GRAPH) {
			// Add the graph-mode menu when viewing the graph.
			redisplayControlArea(GRAPH_MENU);
			// Reflect any changes in graph size:
			Ethno.graphCard.theGraph.invalidate();
		} else {
			// In case we return to the graph after viewing another card
			// set up inspection mode:
			graphModeChoice.select(INSPECTING_GRAPH);
			conductGraphTask(); // Repaint the graph in inspection mode.
			// Remove the graph-mode menu before going to other cards.
			redisplayControlArea(MAIN_MENU);
		}
		// Show the relevant display:
		Ethno.card.show(Ethno.displayPanel, Ethno.theTaskNow);
	} // End showNewAnalysisPage.

	/** Implement user's request for details on an event. */
	void goToPageForAnEvent() {
		String theLine = eventsChoice.getSelectedItem();
		operationsChoice.select(0); // Show menu title.
		incorporateDeedOrPerceptChanges();
		if (theLine.equals(Ethno.interfaceTexts.getString("eventsTitleText"))) { return; }
		redisplayControlArea(ELEMENTS_MENU);
		Ethno.theTaskNow = Ethno.SHOW_DEED;
		Ethno.showDeedCard.fetchData(theLine);
	} // End response to eventsChoice_itemStateChanged.

	/** Implement user's selection of an entity. */
	void dealWithEntityChoice() {
		String theLine = entitiesChoice.getSelectedItem();
//		incorporateDeedOrPerceptChanges();
//		entitiesChoice.select(0); // Restore menu title.
		// Add the selected item to the checked Event Frame container,
		// if one is checked.
		int index = Ethno.showDeedCard.getCheckedCategoryInEventFrame();
		int ACTION = 1; // Actions are the second list in eventFrameListing.
		if ((index >= 0)
			&& (index != ACTION)
			&& (!theLine
				.equals(Ethno.interfaceTexts.getString("newEntityText")))) {
			// Insert a defined entity in event frame category.
			Ethno.showDeedCard.eventFrameListing[index].add(theLine);
			incorporateDeedOrPerceptChanges();
			Ethno.showDeedCard.eventFrameCheckbox[8].setState(true); // Clear
			// radiobuttons
		} else {
			// Define a new entity.
			Ethno.showElementCard.inputsPanel.invalidate();
			Ethno.showElementCard.setCurrentElementType(Ethno.ENTITY);
			showClearedElementCard();
			Ethno.showElementCard.rebuildElementAbstractor(Ethno.entities);
			Ethno.theTaskNow = Ethno.SHOW_ELEMENT;
			if (theLine.equals(Ethno.interfaceTexts.getString("newEntityText"))) {
				Ethno.showElementCard.indexToElement = -1;
				Ethno.showElementCard.nameTextBox.requestFocus();
			} else {
				Ethno.showElementCard.fetchPerceptData(theLine, Ethno.entities);
			}
			generalPanel.setVisible(false);
			elementPanel.setVisible(false);
			Ethno.showElementCard.inputsPanel.validate();
		}
	} // End dealWithEntityChoice.

	/** Implement user's selection of a person. */
	void dealWithPersonChoice() {
		String theLine = peopleChoice.getSelectedItem();
//		repositionEditedPercept();
//		peopleChoice.select(0); // Restore menu title.
		// Add the selected item to the checked Event Frame container,
		// if one is checked.
		int index = Ethno.showDeedCard.getCheckedCategoryInEventFrame();
		int ACTION = 1; // Actions are the second list in eventFrameListing.
		if ((index >= 0)
			&& (index != ACTION)
			&& (!theLine
				.equals(Ethno.interfaceTexts.getString("newPersonText")))) {
			// Insert a defined person in event frame category.
			Ethno.showDeedCard.eventFrameListing[index].add(theLine);
			incorporateDeedOrPerceptChanges();
			// Clear radiobuttons:
			Ethno.showDeedCard.eventFrameCheckbox[8].setState(true);
		} else {
			// Define a new person.
//			rebuildElementChoices();
			Ethno.showElementCard.inputsPanel.invalidate();
			Ethno.showElementCard.setCurrentElementType(Ethno.PERSON);
			showClearedElementCard();
			Ethno.showElementCard.rebuildElementAbstractor(Ethno.people);
			Ethno.theTaskNow = Ethno.SHOW_ELEMENT;
			if (theLine.equals(Ethno.interfaceTexts.getString("newPersonText"))) {
				Ethno.showElementCard.indexToElement = -1;
				Ethno.showElementCard.nameTextBox.requestFocus();
			} else {
				Ethno.showElementCard.fetchPerceptData(theLine, Ethno.people);
			}
			generalPanel.setVisible(false);
			elementPanel.setVisible(false);
			Ethno.showElementCard.inputsPanel.validate();
		}
	} // End dealWithPersonChoice.

	/** Implement user's selection of an action. */
	void dealWithActionChoice() {
		String theLine = actionsChoice.getSelectedItem();
//		incorporateDeedOrPerceptChanges();
//		actionsChoice.select(0); // Restore menu title.
		// Add the selected item to the checked Event Frame container, if one is
		// checked.
		int index = Ethno.showDeedCard.getCheckedCategoryInEventFrame();
		int ACTION = 1; // Actions are the second list in eventFrameListing.
		if ((index >= 0)
			&& (index == ACTION)
			&& (!theLine
				.equals(Ethno.interfaceTexts.getString("newActionText")))) {
			// Insert a defined action in event frame category.
			Ethno.showDeedCard.eventFrameListing[index].add(theLine);
			incorporateDeedOrPerceptChanges();
			// Clear radiobuttons:
			Ethno.showDeedCard.eventFrameCheckbox[8].setState(true);
		} else {
			// Define a new action.
			Ethno.showElementCard.inputsPanel.invalidate();
			Ethno.showElementCard.setCurrentElementType(Ethno.ACTION);
			showClearedElementCard();
			Ethno.showElementCard.rebuildElementAbstractor(Ethno.actions);
			Ethno.theTaskNow = Ethno.SHOW_ELEMENT;
			if (theLine.equals(Ethno.interfaceTexts.getString("newActionText"))) {
				Ethno.showElementCard.indexToElement = -1;
				Ethno.showElementCard.nameTextBox.requestFocus();
			} else {
				Ethno.showElementCard.fetchPerceptData(theLine, Ethno.actions);
			}
			generalPanel.setVisible(false);
			elementPanel.setVisible(false);
			Ethno.showElementCard.inputsPanel.validate();
		}
	} // End dealWithActionChoice.

	/** Implement user's selection of a graph-related task. */
	void conductGraphTask() {
		if (Ethno.deeds.size() < 2) { return; }
		ChartCard.initializeGraphVariables();
		// ChartCard.suspendClearingGraphVariables = false;
		graphMode = graphModeChoice.getSelectedIndex();
		incorporateDeedOrPerceptChanges();
		if ((graphMode == INSPECTING_GRAPH) || (graphMode == EDITING_GRAPH)) {
		} else if (graphMode == SUMMARIZING_IN_GRAPH) {
			Ethno.graphCard.setCursor(Cursor
				.getPredefinedCursor(Cursor.WAIT_CURSOR)); // Set a wait
			// cursor.
			AbstractionDialog.setSummarizing(true);
			AbstractionDialog summary =
				new AbstractionDialog((Frame) this.getParent().getParent());
			summary.setVisible(true);
			graphModeChoice.select(INSPECTING_GRAPH);
		} else if (graphMode == GENERALIZING_IN_GRAPH) {
			AbstractionDialog.setSummarizing(false);
			AbstractionDialog summary =
				new AbstractionDialog((Frame) this.getParent().getParent());
			summary.setVisible(true);
			graphModeChoice.select(INSPECTING_GRAPH);
		} else if ((graphMode == TESTING_IN_GRAPH)
			|| (graphMode == EXPERIMENTING_IN_GRAPH)) {
			Ethno.eventCounts = new int[Ethno.deeds.size()][Ethno.deeds.size()];
			for (int i = 0; i < Ethno.deeds.size(); i++) {
				for (int j = 0; j < Ethno.deeds.size(); j++) {
					Ethno.eventCounts[i][j] = 0;
				}
			}
		} else if (graphMode == PRINTING_GRAPH) {
			Chart.nowPrinting = true;
			// Fetch printer information:
			PrinterJob graphPrintJob = PrinterJob.getPrinterJob();
			PageFormat pageFormat = graphPrintJob.defaultPage();
			pageFormat = graphPrintJob.pageDialog(pageFormat);
			printWidth = (int) pageFormat.getImageableWidth();
			printHeight = (int) pageFormat.getImageableHeight();
			Book book = new Book(); // Create a new book for page.
			book.append(new PrintGraph(), pageFormat); // Add the chart page.
			graphPrintJob.setPageable(book); // Use the Book as the Pageable
			// object
			if (graphPrintJob.printDialog()) { // Show a Print dialog.
				try {
					graphPrintJob.print(); // Print to paper.
				} catch (PrinterException exception) {
					Ethno.reportPage.results.append(
					"\n\nPRINTING ERROR: " + exception + "\n\n");
				}
			}
			// Restore the screen image.
			Chart.nowPrinting = false;
			Ethno.graphCard.theGraph.invalidate();
			Ethno.graphCard.validate();
		}
		Ethno.graphCard.theGraph.repaint();
	} // End conductGraphTask.

	/** Rebuild the operations menu. */
	public void rebuildOperationsChoice() {
		operationsChoice.invalidate();
		operationsChoice.removeAll();
		for (int i = 0; i < EXPORT_ABSTRACT_MODEL; i++) {
			operationsChoice.add(Ethno.operationLine[i]);
		}
		if (Ethno.deeds.isGeneralizedModel()) {
			operationsChoice.add(Ethno.operationLine[EXPORT_ABSTRACT_MODEL]);
		}
		operationsChoice.select(getLineForTASK_NOW(Ethno.theTaskNow));
	} // End rebuildOperationsChoice.

	/** Rebuild the graph tasks menu. */
	public void rebuildGraphModeChoice() {
		graphModeChoice.invalidate();
		graphModeChoice.removeAll();
		for (int i = 0; i < Ethno.graphModeLine.length; i++) {
			graphModeChoice.add(Ethno.graphModeLine[i]);
		}
	} // End rebuildGraphModeChoice.

	/** Rebuild the events menu. */
	public void rebuildEventsChoice() {
		Deed theDeed;
		String theName;
		eventsChoice.invalidate();
		eventsChoice.removeAll();
		eventsChoice.add(Ethno.interfaceTexts.getString("eventsTitleText"));
		// Do the repeats menu on DefineSequenceCard, too.
		Ethno.defineSequenceCard.eventChoiceMenu.invalidate();
		Ethno.defineSequenceCard.eventChoiceMenu.removeAll();
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			theDeed = (Deed) Ethno.deeds.elementAt(i);
			theName = theDeed.shortName;
			if (theDeed.isSummarized) {
				theName =
					Ethno.interfaceTexts.getString("summarizedMark") + theName;
			} else {
				// Include only non-summarized deeds in sequence repeats menu.
				Ethno.defineSequenceCard.eventChoiceMenu.addItem(theName);
			}
			eventsChoice.addItem(theName);
		}
		Ethno.controls.validate();
		Ethno.defineSequenceCard.eventChoiceMenu.validate();
	} // End rebuildEventsChoice.

	/** Rebuild the menus for choosing entities, people, and actions. */
	public void rebuildElementChoices() {
		// This reconstructs menus to work around
		// JAVA BUG: http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=7171412
		Percept x;
		entitiesChoice.removeAll();
		peopleChoice.removeAll();
		actionsChoice.removeAll();
		elementPanel.removeAll();
		elementPanel.add(entitiesChoice, null);
		elementPanel.add(peopleChoice, null);
		elementPanel.add(actionsChoice, null);
		entitiesChoice.add(Ethno.interfaceTexts.getString("newEntityText"));
		peopleChoice.add(Ethno.interfaceTexts.getString("newPersonText"));
		actionsChoice.add(Ethno.interfaceTexts.getString("newActionText"));
		for (int i = 0; i < Ethno.entities.size(); i++) {
			x = (Percept) Ethno.entities.elementAt(i);
			entitiesChoice.addItem(x.shortName);
		}
		for (int i = 0; i < Ethno.people.size(); i++) {
			x = (Percept) Ethno.people.elementAt(i);
			peopleChoice.addItem(x.shortName);
		}
		for (int i = 0; i < Ethno.actions.size(); i++) {
			x = (Percept) Ethno.actions.elementAt(i);
			actionsChoice.addItem(x.shortName);
		}
	} // End rebuildElementChoices.

	/** Reconstruct the menu bar. */
	void redisplayControlArea(int menuNeeded) {
		entitiesChoice.setVisible(false);
		peopleChoice.setVisible(false);
		actionsChoice.setVisible(false);
		graphModeChoice.setVisible(false);
		if (menuNeeded == ELEMENTS_MENU) {
			// Frame submenus.
			entitiesChoice.setVisible(true);
			peopleChoice.setVisible(true);
			actionsChoice.setVisible(true);
		} else if (menuNeeded == GRAPH_MENU) {
			// Graph submenu.
			graphModeChoice.setVisible(true);
		}
		Ethno.controls.invalidate();
		Ethno.displayPanel.invalidate();
		entitiesChoice.invalidate();
		peopleChoice.invalidate();
		actionsChoice.invalidate();
		graphModeChoice.invalidate();
		entitiesChoice.validate();
		peopleChoice.validate();
		actionsChoice.validate();
		Ethno.controls.validate();
		Ethno.displayPanel.validate();
	} // End reconstructControlArea.

	/**
	 * Incorporate editing on deed card or element card before a menu action can
	 * occur.
	 */
	void incorporateDeedOrPerceptChanges() {
		// Incorporate input on DeedCard or PerceptCard.
		Deed theDeed, aDeed;
		Relation r;
		Percept focalElement, targetElement, elementBeingDeleted, anElement;
		PerceptList elementList;
		String oldName, newName, dataString;
		java.awt.List anEventFrameListing;
		int index;
		if (Ethno.theTaskNow.equals(Ethno.SHOW_DEED)) {
			// Incorporate data from deed card.
			// Get the old event, the original short name, and the new one.
			theDeed =
				(Deed) Ethno.deeds.elementAt(Ethno.showDeedCard.indexToDeed);
			oldName = theDeed.shortName;
			newName =
				Ethno.cleanTheString(Ethno.showDeedCard.showAbbreviation
					.getText());
			if (!oldName.equals(newName)) {
				// Check for duplication and reposition alphabetically.
				theDeed.shortName = newName;
				index =
					Ethno.deeds.repositionElement(
						Ethno.showDeedCard.indexToDeed, (Deed) theDeed);
				if (index < 0) {
					// Duplicate short name.
					CancelOkDialog whoops =
						new CancelOkDialog((Frame) this.getParent(),
							Ethno.interfaceTexts
								.getString("problemEncounteredText"),
							Ethno.interfaceTexts
								.getString("duplicateDeedError"), "");
					whoops.setVisible(true);
					theDeed.shortName = oldName;
					return;
				} else {
					Ethno.showDeedCard.indexToDeed = index;
					// Correct linkages.
					for (int m = 0; m < Ethno.deeds.size(); m++) {
						aDeed = (Deed) Ethno.deeds.elementAt(m);
						for (int na = 0; na < aDeed.implications.size(); na++) {
							r = (Relation) aDeed.implications.elementAt(na);
							if (r.implicated.equals(oldName)) {
								r.implicated = newName;
							}
						}
						for (int nb = 0; nb < aDeed.generalizations.size(); nb++) {
							r = (Relation) aDeed.generalizations.elementAt(nb);
							if (r.implicated.equals(oldName)) {
								r.implicated = newName;
							}
						}
					}
					rebuildEventsChoice();
				}
			}
			theDeed.verboseEventName =
				Ethno.cleanTheString(Ethno.showDeedCard.eventDescription
					.getText());
			theDeed.sourceText =
				Ethno.cleanTheString(Ethno.showDeedCard.quote.getText());
			theDeed.comment =
				Ethno.cleanTheString(Ethno.showDeedCard.comment.getText());
			// Booleans.
			theDeed.isDisjunctive =
				Ethno.showDeedCard.disjunctiveCheckbox.getState();
			theDeed.isRepeatable =
				Ethno.showDeedCard.repeatableCheckbox.getState();
			// Store Event Frame.
			targetElement = null; // Arbitrary initialization for compiler.
			for (int i = 0; i < 8; i++) {
				theDeed.eventFrame[i].removeAllElements();
				anEventFrameListing = Ethno.showDeedCard.eventFrameListing[i];
				for (int j = 0; j < anEventFrameListing.getItemCount(); j++) {
					dataString = anEventFrameListing.getItem(j);
					index = Ethno.entities.findPositionOfElement(dataString);
					if (index >= 0) {
						targetElement =
							(Percept) Ethno.entities.elementAt(index);
					} else {
						index = Ethno.people.findPositionOfElement(dataString);
						if (index >= 0) {
							targetElement =
								(Percept) Ethno.people.elementAt(index);
						} else {
							index =
								Ethno.actions.findPositionOfElement(dataString);
							if (index >= 0) {
								targetElement =
									(Percept) Ethno.actions.elementAt(index);
							}
						}
					}
					theDeed.eventFrame[i].addElement(targetElement);
				}
			}
			// Logical relations cannot be changed on this form.
			// Revise Sequence list.
			for (int i = 0; i < Ethno.sequence.size(); i++) {
				aDeed = (Deed) Ethno.sequence.elementAt(i);
				if (oldName.equals(aDeed.shortName)) {
					aDeed.shortName = newName;
				}
			}
			if (!newName.equals(oldName)) {
				rebuildEventsChoice();
			}
		} // End incorporating from deed card.
		
		else if (Ethno.theTaskNow.equals(Ethno.SHOW_ELEMENT)) {
			// Incorporate data from element card.
			newName =
				Ethno.cleanTheString(Ethno.showElementCard.nameTextBox
					.getText()); // Get the element name.
			// Skip if the name field is blank:
			if (newName.equals("")) { return; }
			// Align to the proper vector:
			elementList =
				PerceptList.getPerceptList(ShowPerceptCard.currentElementType);
			if ((Ethno.showElementCard.indexToElement >= 0)
				&& newName.equalsIgnoreCase(Ethno.interfaceTexts
					.getString("deletionWord"))) {
				// Delete the element.
				elementBeingDeleted =
					(Percept) elementList
						.elementAt(Ethno.showElementCard.indexToElement);
				// Transfer implications.
				for (int i = 0; i < elementList.size(); i++) {
					// For each element of this type.
					anElement = (Percept) elementList.elementAt(i);
					if (anElement == elementBeingDeleted) {
						continue;
					}
					for (int j = 0; j < anElement.implications.size(); j++) {
						// Does the element imply elementBeingDeleted?
						r = (Relation) anElement.implications.elementAt(j);
						if (r.implicated
							.equalsIgnoreCase(elementBeingDeleted.shortName)) {
							for (int k = 0; k < elementBeingDeleted.implications
								.size(); k++) {
								// Yes, so transfer elementBeingDeleted's
								// implications.
								anElement.implications
									.addElement((Relation) elementBeingDeleted.implications
										.elementAt(k));
							}
						}
					}
				}
				// Remove from Event Frame of each event.
				for (int i = 0; i < Ethno.deeds.size(); i++) { // For each
					// deed.
					aDeed = (Deed) Ethno.deeds.elementAt(i);
					for (int j = 0; j < aDeed.eventFrame.length; j++) {
						// For each Event Frame category.
						for (int k = aDeed.eventFrame[j].size() - 1; k >= 0; k--) {
							// For each percept in the category.
							anElement =
								(Percept) aDeed.eventFrame[j].elementAt(k);
							if (anElement == elementBeingDeleted) {
								aDeed.eventFrame[j].removeElementAt(k);
							}
						}
					}
				}
				elementList
					.removeElementAt(Ethno.showElementCard.indexToElement);
				Ethno.showElementCard.indexToElement = -1;
				showClearedElementCard();
				// End of deleting.
				
			} else if (Ethno.showElementCard.indexToElement >= 0) {
				// Edit an existing element.
				focalElement =
					(Percept) elementList
						.elementAt(Ethno.showElementCard.indexToElement);
				oldName = focalElement.shortName;
				focalElement.shortName = newName;
				index =
					elementList.repositionElement(
						Ethno.showElementCard.indexToElement, focalElement);
				if (index < 0) {
					// Duplicate short name.
					CancelOkDialog whoops =
						new CancelOkDialog((Frame) this.getParent(),
							Ethno.interfaceTexts
								.getString("problemEncounteredText"),
							Ethno.interfaceTexts
								.getString("duplicateDeedError"), "");
					whoops.setVisible(true);
					focalElement.shortName = oldName;
					return;
					// End of duplicate element.
				} else
					Ethno.showElementCard.indexToElement = index;
				// End of editing an element.
				
			} else {
				// Emplace a new element.
				oldName = "";
				focalElement = new Percept();
				focalElement.shortName = newName;
				Ethno.showElementCard.indexToElement =
					elementList.insertAlphabetically(focalElement);
				if (Ethno.showElementCard.indexToElement < 0) {
					// Duplicate short name.
					CancelOkDialog whoops =
						new CancelOkDialog((Frame) this.getParent(),
							Ethno.interfaceTexts
								.getString("problemEncounteredText"),
							Ethno.interfaceTexts
								.getString("duplicateDeedError"), "");
					whoops.setVisible(true);
					focalElement = null;
					return;
				}
			}
			rebuildElementChoices();
			redisplayControlArea(ELEMENTS_MENU);
		} // End incorporating data from show-element card.
		Ethno.defineSequenceCard.reconstructSequenceDisplay();
	} // End incorporateDeedOrPerceptChanges.

	void showClearedElementCard() {
		Ethno.showElementCard.abstractionsList.removeAll();
		Ethno.showElementCard.actionAssociations.removeAll();
		Ethno.showElementCard.chooseGeneralizationMenu.removeAll();
		Ethno.showElementCard.concreteEventsCheckbox.setState(true);
		Ethno.showElementCard.concreteFormsList.removeAll();
		Ethno.showElementCard.containingEvents.removeAll();
		Ethno.showElementCard.entityAssociations.removeAll();
		Ethno.showElementCard.generalizeCheckbox.setState(false);
		Ethno.showElementCard.nameTextBox.setText("");
		Ethno.showElementCard.personAssociations.removeAll();
		Ethno.showElementCard.generalizedEventsCheckbox.setEnabled(Ethno.deeds
			.isGeneralizedModel());
		Ethno.card.show(Ethno.displayPanel, Ethno.SHOW_ELEMENT);
	} // End showClearedElementCard.

	/** Recall what the current task is after rebuilding the menu. */
	public int getLineForTASK_NOW(String theTaskNow) {
		if (theTaskNow.equalsIgnoreCase(Ethno.DEFINE_SEQUENCE)) {
			return DEFINE_SEQUENCE;
		} else if (theTaskNow.equalsIgnoreCase(Ethno.DEFINE_LINKS)) {
			return LINK_EVENTS;
		} else if (theTaskNow.equalsIgnoreCase(Ethno.GRAPH)) {
			return WORK_WITH_CHART;
		} else if (theTaskNow.equalsIgnoreCase(Ethno.SHOW_DEED)) {
			return TITLE_LINE;
		} else if (theTaskNow.equalsIgnoreCase(Ethno.SHOW_ELEMENT)) {
			return TITLE_LINE;
		} else if (theTaskNow.equalsIgnoreCase(Ethno.IMPORT_EXPORT)) { return IMPORT_EXPORT; }
		return -1;
	} // End getLineForTASK_NOW.

} // End ControlMenus.
