
import java.util.Vector;

/** Variables and methods for an event. */
class Deed extends Percept {

	/*
	 * Fields inherited from Percept: String shortName; Vector implications;
	 */

	/** Verbose description of element. */
	String verboseEventName;

	/** Quote in original text describing event. */
	String sourceText;

	/** Researcher's remark about an element. */
	String comment;

	/** Buckets for action, entity, person percepts. */
	Vector[] eventFrame;

	/** Bucket for generalizing relations. */
	Vector generalizations;

	/** Bucket of demoted deeds if this is a summary deed. */
	Vector summarizedActions;

	/** Bottom-most node in a set of summarized nodes. */
	Relation linkToFinalSummarizedAction;

	/** A characteristic of an event. */
	boolean isLinked, isDisjunctive, isRepeatable, isGeneralization,
			isInstantiated, isSummarized;

	/** Graphing variable. */
	int halfNameWidth, upCount, downCount, tier, tierRank, horizontalPosition,
			verticalPosition;

	/** Nodes directly or indirectly linked to focal node. */
	Vector upNodes, downNodes;

	/** A property of an event on the graph page. */
	boolean isAccomplished, isFeasible, isBlocked, isInProcess, isSelected,
			isADummy;

	/** A property of an event during an elicitation. */
	boolean isAnswered;

	public Deed() { // Constructor method.
		// Establish the text variables.
		shortName = new String();
		verboseEventName = new String();
		sourceText = new String();
		comment = new String();
		verboseEventName = "";
		sourceText = "";
		comment = "";
		// Establish the Event Frame buckets, with initial space for one.
		eventFrame = new Vector[8];
		for (int i = 0; i < 8; i++) {
			eventFrame[i] = new Vector(1);
		}
		// Establish the relation buckets, with initial space for two.
		implications = new Vector(2);
		generalizations = new Vector(2);
		summarizedActions = new Vector(2);
		linkToFinalSummarizedAction = new Relation("", false, false);
		// Initialize graphing variables.
		halfNameWidth = upCount = downCount = horizontalPosition = 0;
		tier = -1;
		upNodes = new Vector();
		downNodes = new Vector();
		// Initialize the boolean variables.
		isLinked =
			isDisjunctive =
				isRepeatable =
					isGeneralization = isSummarized = isInstantiated = false;
		isFeasible =
			isAccomplished =
				isInProcess = isSelected = isADummy = isAnswered = false;
	} // End Deed constructor 1.

	public Deed(String briefName, String quote) {
		// Constructor 2 method.
		this(); // Invokes the first constructor method.
		shortName = briefName;
		sourceText = quote;
	} // End Deed constructor 2.

	public Deed(String briefName, String quote, String verboseName) {
		// Constructor 3 method.
		this(); // Invokes the first constructor method.
		shortName = briefName;
		verboseEventName = verboseName;
		sourceText = quote;
	} // End Deed constructor 2.

	/** Transform a deed to textual form; using toXML after 2006. */
	public String toString(boolean generalizedModel) {
		Percept p;
		Relation link;
		String output;
		output = "#" + shortName + "\n";
		output = output + verboseEventName + "\n";
		output = output + sourceText + "\n";
		output = output + comment + "\n";
		output = output + isLinked + "\n";
		output = output + isDisjunctive + "\n";
		output = output + isRepeatable + "\n";
		output = output + isGeneralization + "\n";
		if (generalizedModel) {
			output = output + "false\n";
		} else {
			output = output + isInstantiated + "\n";
		}
		output = output + isSummarized + "\n";
		for (int i = 0; i < 8; i++) {
			output = output + "#F " + (i + 1) + "\n";
			for (int j = 0; j < eventFrame[i].size(); j++) {
				p = (Percept) eventFrame[i].elementAt(j);
				output = output + p.shortName + "\n";
			}
		}
		output = output + "#IMPLIES\n";
		for (int j = 0; j < implications.size(); j++) {
			link = (Relation) implications.elementAt(j);
			output = output + link.implicated + "\n";
			output = output + link.depletes + "\n";
			output = output + link.commutes + "\n";
		}
		output = output + "#GENERALIZES TO\n";
		for (int j = 0; j < generalizations.size(); j++) {
			link = (Relation) generalizations.elementAt(j);
			output = output + "@ " + link.implicated + "\n";
			output = output + link.depletes + "\n";
			output = output + link.commutes + "\n";
		}
		output = output + "#SUMMARIZES\n";
		for (int j = 0; j < summarizedActions.size(); j++) {
			link = (Relation) summarizedActions.elementAt(j);
			output = output + "@ " + link.implicated + "\n";
			output = output + link.depletes + "\n";
			output = output + link.commutes + "\n";
		}
		return output;
	}

	/** Transform a textual record into a deed; kept for legacy datasets.. */
	public int fromString(String data, int first) {
		int last, index, eof;
		String implicated, elementName;
		boolean depletes, commutes;
		eof = data.length() - 1;
		if (data.charAt(first) != '#') {
			Ethno.reportPage.results.append(
					"\n\nNo # for event.\n\n");
			return (eof);
		}
		first++;
		if (data.charAt(first) == '\n') {
			Ethno.reportPage.results.append(
				"\n\nEvent has no short name.\n\n");
			return (eof);
		}
		// Get short name.
		last = data.indexOf('\n', first);
		this.shortName = data.substring(first, last);
		// Get event description.
		first = ++last;
		if (data.charAt(first) == '\n') {
			// No description provided.
			this.verboseEventName = "";
			last = first;
		} else {
			last = data.indexOf('\n', first);
			this.verboseEventName = data.substring(first, last);
		}
		// Get source text.
		first = ++last;
		if (data.charAt(first) == '\n') {
			// None provided.
			this.sourceText = "";
			last = first;
		} else {
			last = data.indexOf('\n', first);
			this.sourceText = data.substring(first, last);
		}
		// Get comment.
		first = ++last;
		if (data.charAt(first) == '\n') {
			// None provided.
			this.comment = "";
			last = first;
		} else {
			last = data.indexOf('\n', first);
			this.comment = data.substring(first, last);
		}
		// Get boolean variables.
		first = ++last;
		last = data.indexOf('\n', first);
		this.isLinked = (data.substring(first, last).equals("true"));
		first = ++last;
		last = data.indexOf('\n', first);
		this.isDisjunctive = (data.substring(first, last).equals("true"));
		first = ++last;
		last = data.indexOf('\n', first);
		this.isRepeatable = (data.substring(first, last).equals("true"));
		first = ++last;
		last = data.indexOf('\n', first);
		this.isGeneralization = (data.substring(first, last).equals("true"));
		first = ++last;
		last = data.indexOf('\n', first);
		this.isInstantiated = (data.substring(first, last).equals("true"));
		first = ++last;
		last = data.indexOf('\n', first);
		this.isSummarized = (data.substring(first, last).equals("true"));
		first = ++last;
		// Get event frame codings.
		for (int i = 0; i < 8; i++) {
			last = data.indexOf('\n', first); // Skip the title line.
			first = ++last;
			while (data.charAt(first) != '#') {
				last = data.indexOf('\n', first);
				elementName = data.substring(first, last);
				// Find the unique element name, and put it in the proper
				// bucket.
				index = Ethno.entities.findPositionOfElement(elementName);
				if (index >= 0) {
					eventFrame[i].addElement(Ethno.entities.elementAt(index));
				} else {
					index = Ethno.people.findPositionOfElement(elementName);
					if (index >= 0) {
						eventFrame[i].addElement(Ethno.people.elementAt(index));
					} else {
						index =
							Ethno.actions.findPositionOfElement(elementName);
						if (index >= 0) {
							eventFrame[i].addElement(Ethno.actions
								.elementAt(index));
						} else {
							Ethno.reportPage.results.append(
								"\n\nCan't find element: "
								+ elementName + "\n\n");
						}
					}
				}
				first = ++last;
			}
		}
		last = data.indexOf('\n', first); // Skip IMPLIES title.
		first = ++last;
		while (data.charAt(first) != '#') {
			last = data.indexOf('\n', first);
			implicated = data.substring(first, last);
			first = ++last;
			last = data.indexOf('\n', first);
			depletes = (data.substring(first, last).equals("true"));
			first = ++last;
			last = data.indexOf('\n', first);
			commutes = (data.substring(first, last).equals("true"));
			this.implications.addElement(new Relation(implicated, depletes,
				commutes));
			first = ++last;
		}
		last = data.indexOf('\n', first); // Skip GENERALIZES TO title.
		first = ++last;
		if (!data.substring(first, data.indexOf('\n', first)).equalsIgnoreCase(
			"@SEQUENCE")) { // Allow for old formatting.
			while (data.charAt(first) != '#') {
				last = data.indexOf('\n', first);
				// +2 skips "@ " at beginning of name:
				implicated = data.substring(first + 2, last);
				first = ++last;
				last = data.indexOf('\n', first);
				depletes = (data.substring(first, last).equals("true"));
				first = ++last;
				last = data.indexOf('\n', first);
				commutes = (data.substring(first, last).equals("true"));
				this.generalizations.addElement(new Relation(implicated,
					depletes, commutes));
				first = ++last;
			}
		}
		last = data.indexOf('\n', first); // Skip SUMMARIZES title.
		first = ++last;
		while ((data.charAt(first) != '#') && (data.charAt(first) != '@')) {
			last = data.indexOf('\n', first);
			implicated = data.substring(first, last);
			first = ++last;
			last = data.indexOf('\n', first);
			depletes = (data.substring(first, last).equals("true"));
			first = ++last;
			last = data.indexOf('\n', first);
			commutes = (data.substring(first, last).equals("true"));
			this.summarizedActions.addElement(new Relation(implicated,
				depletes, commutes));
			first = ++last;
		}
		return first;
	} // End fromString.

	/** Encode a deed into the XML text describing the deed. */
	public String toXML(boolean generalizedModel) {
		Percept p;
		Relation link;
		String output;
		output =
			"<" + SavedData.name + ">" + shortName + "</" + SavedData.name
				+ ">\n";
		if (verboseEventName.length() != 0) {
			output =
				output + "<" + SavedData.longName + ">" + verboseEventName
					+ "</" + SavedData.longName + ">\n";
		}
		if (sourceText.length() != 0) {
			output =
				output + "<" + SavedData.sourceText + ">" + sourceText + "</"
					+ SavedData.sourceText + ">\n";
		}
		if (comment.length() != 0) {
			output =
				output + "<" + SavedData.comment + ">" + comment + "</"
					+ SavedData.comment + ">\n";
		}
		output =
			output + "<" + SavedData.isLinked + ">" + isLinked + "</"
				+ SavedData.isLinked + ">\n";
		output =
			output + "<" + SavedData.isDisjunctive + ">" + isDisjunctive + "</"
				+ SavedData.isDisjunctive + ">\n";
		output =
			output + "<" + SavedData.isRepeatable + ">" + isRepeatable + "</"
				+ SavedData.isRepeatable + ">\n";
		output =
			output + "<" + SavedData.isGeneralization + ">" + isGeneralization
				+ "</" + SavedData.isGeneralization + ">\n";
		if (generalizedModel) {
			output =
				output + "<" + SavedData.isInstantiated + ">" + "false" + "</"
					+ SavedData.isInstantiated + ">\n";
		} else {
			output =
				output + "<" + SavedData.isInstantiated + ">" + isInstantiated
					+ "</" + SavedData.isInstantiated + ">\n";
		}
		if (isSummarized) {
			output =
				output + "<" + SavedData.isSummarized + ">" + isSummarized
					+ "</" + SavedData.isSummarized + ">\n";
		}
		String[] frameStart =
			{ "<" + SavedData.agent + ">", "<" + SavedData.act + ">",
				"<" + SavedData.object + ">", "<" + SavedData.instrument + ">",
				"<" + SavedData.alignment + ">", "<" + SavedData.setting + ">",
				"<" + SavedData.product + ">",
				"<" + SavedData.beneficiary + ">" };
		String[] frameEnd =
			{ "</" + SavedData.agent + ">", "</" + SavedData.act + ">",
				"</" + SavedData.object + ">",
				"</" + SavedData.instrument + ">",
				"</" + SavedData.alignment + ">",
				"</" + SavedData.setting + ">", "</" + SavedData.product + ">",
				"</" + SavedData.beneficiary + ">" };
		boolean usingEventFrame = false;
		for (int i = 0; i < 8; i++) {
			if (eventFrame[i].size() > 0) {
				usingEventFrame = true;
			}
		}
		if (usingEventFrame) {
			output = output + "<" + SavedData.eventFrame + ">\n";
			for (int i = 0; i < 8; i++) {
				for (int j = 0; j < eventFrame[i].size(); j++) {
					output = output + frameStart[i];
					p = (Percept) eventFrame[i].elementAt(j);
					output = output + p.shortName;
					output = output + frameEnd[i] + "\n";
				}
			}
			output = output + "</" + SavedData.eventFrame + ">\n";
		}
		if (implications.size() > 0) {
			output = output + "<" + SavedData.implies + ">\n";
			for (int j = 0; j < implications.size(); j++) {
				link = (Relation) implications.elementAt(j);
				output =
					output + "<" + SavedData.antecedent + ">" + link.implicated
						+ "</" + SavedData.antecedent + ">\n";
				output =
					output + "<" + SavedData.depletesAntecedent + ">"
						+ link.depletes + "</" + SavedData.depletesAntecedent
						+ ">\n";
				output =
					output + "<" + SavedData.commutesWithAntecedent + ">"
						+ link.commutes + "</"
						+ SavedData.commutesWithAntecedent + ">\n";
			}
			output = output + "</" + SavedData.implies + ">\n";
		}
		for (int j = 0; j < generalizations.size(); j++) {
			link = (Relation) generalizations.elementAt(j);
			output =
				output + "<" + SavedData.generalizesTo + ">" + link.implicated
					+ "</" + SavedData.generalizesTo + ">\n";
		}
		for (int j = 0; j < summarizedActions.size(); j++) {
			link = (Relation) summarizedActions.elementAt(j);
			output =
				output + "<" + SavedData.summarizes + ">" + link.implicated
					+ "</" + SavedData.summarizes + ">\n";
		}
		return output;
	} // End toXML.

	/** Extract a deed from the XML text describing the deed. */
	void fromXMLString(String data) {
		String name;
		if (!(data.substring(0, SavedData.open(SavedData.name).length()))
			.equalsIgnoreCase(SavedData.open(SavedData.name))) {
			Ethno.reportPage.results.append(
				Ethno.interfaceTexts.getString("unnamedDeed") + "\n");
			return;
		}
		// Lop off <NAME>
		data =
			data.substring(data.indexOf(SavedData.open(SavedData.name))
				+ (SavedData.open(SavedData.name)).length());
		// Get the name.
		name = data.substring(0, data.indexOf(SavedData.close(SavedData.name)));
		if (name.length() == 0) {
			Ethno.reportPage.results.append(
				Ethno.interfaceTexts.getString("unnamedDeed") + "\n");
			return;
		}
		this.shortName = name;
		// Get optional texts.
		if (data.indexOf(SavedData.open(SavedData.longName)) != -1) {
			// Lop off <LONG_NAME>
			data =
				data.substring(data.indexOf(SavedData.open(SavedData.longName))
					+ (SavedData.open(SavedData.longName)).length());
			// Get the long name.
			name =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.longName)));
			this.verboseEventName = name;
		}
		if (data.indexOf(SavedData.open(SavedData.sourceText)) != -1) {
			// Lop off <SOURCE_TEXT>
			data =
				data.substring(data.indexOf(SavedData
					.open(SavedData.sourceText))
					+ (SavedData.open(SavedData.sourceText)).length());
			// Get the long name.
			name =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.sourceText)));
			this.sourceText = name;
		}
		if (data.indexOf(SavedData.open(SavedData.comment)) != -1) {
			// Lop off <COMMENT>
			data =
				data.substring(data.indexOf(SavedData.open(SavedData.comment))
					+ (SavedData.open(SavedData.comment)).length());
			// Get the long name.
			name =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.comment)));
			this.comment = name;
		}
		// Get required boolean variables.
		// Lop off <IS_LINKED>
		data =
			data.substring(data.indexOf(SavedData.open(SavedData.isLinked))
				+ (SavedData.open(SavedData.isLinked)).length());
		this.isLinked =
			data.substring(0, data.indexOf(SavedData.close(SavedData.isLinked)))
				.equalsIgnoreCase("true");
		// Lop off <IS_DISJUNCTIVE>
		data =
			data.substring(data.indexOf(SavedData.open(SavedData.isDisjunctive))
				+ (SavedData.open(SavedData.isDisjunctive)).length());
		this.isDisjunctive =
			data.substring(0,
				data.indexOf(SavedData.close(SavedData.isDisjunctive)))
				.equalsIgnoreCase("true");
		// Lop off <IS_REPEATABLE>
		data =
			data.substring(data.indexOf(SavedData.open(SavedData.isRepeatable))
				+ (SavedData.open(SavedData.isRepeatable)).length());
		this.isRepeatable =
			data.substring(0,
				data.indexOf(SavedData.close(SavedData.isRepeatable)))
				.equalsIgnoreCase("true");
		// Lop off <IS_GENERALIZATION>
		data =
			data.substring(data.indexOf(SavedData
				.open(SavedData.isGeneralization))
				+ (SavedData.open(SavedData.isGeneralization)).length());
		this.isGeneralization =
			data.substring(0,
				data.indexOf(SavedData.close(SavedData.isGeneralization)))
				.equalsIgnoreCase("true");
		// Instantiation boolean variable only occurs for generalized events.
		if (this.isGeneralization) {
			// Lop off <IS_INSTANTIATED>
			data =
				data.substring(data.indexOf(SavedData
					.open(SavedData.isInstantiated))
					+ (SavedData.open(SavedData.isInstantiated)).length());
			this.isInstantiated =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.isInstantiated)))
					.equalsIgnoreCase("true");
		}
		// Summarized boolean variable is optional.
		if (data.indexOf(SavedData.open(SavedData.isSummarized)) != -1) {
			// Lop off <IS_SUMMARIZED>
			data =
				data.substring(data.indexOf(SavedData
					.open(SavedData.isSummarized))
					+ (SavedData.open(SavedData.isSummarized)).length());
			this.isSummarized =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.isSummarized)))
					.equalsIgnoreCase("true");
		}

		// Get Event Frame codings.
		if (data.indexOf(SavedData.open(SavedData.eventFrame)) != -1) {
			while (data.indexOf(SavedData.open(SavedData.agent)) != -1) {
				// Lop off <AGENT>
				data =
					data.substring(data.indexOf(SavedData.open(SavedData.agent))
						+ (SavedData.open(SavedData.agent)).length());
				// Get the name.
				name =
					data.substring(0,
						data.indexOf(SavedData.close(SavedData.agent)));
				putInEventFrame(name, 0);
			}
			while (data.indexOf(SavedData.open(SavedData.act)) != -1) {
				// Lop off <ACT>
				data =
					data.substring(data.indexOf(SavedData.open(SavedData.act))
						+ SavedData.open(SavedData.act).length());
				// Get the name.
				name =
					data.substring(0,
						data.indexOf(SavedData.close(SavedData.act)));
				putInEventFrame(name, 1);
			}
			while (data.indexOf(SavedData.open(SavedData.object)) != -1) {
				// Lop off <OBJECT>
				data =
					data.substring(data.indexOf(SavedData
						.open(SavedData.object))
						+ SavedData.open(SavedData.object).length());
				// Get the name.
				name =
					data.substring(0,
						data.indexOf(SavedData.close(SavedData.object)));
				putInEventFrame(name, 2);
			}
			while (data.indexOf(SavedData.open(SavedData.instrument)) != -1) {
				// Lop off <INSTRUMENT>
				data =
					data.substring(data.indexOf(SavedData
						.open(SavedData.instrument))
						+ SavedData.open(SavedData.instrument).length());
				// Get the name.
				name =
					data.substring(0,
						data.indexOf(SavedData.close(SavedData.instrument)));
				putInEventFrame(name, 3);
			}
			while (data.indexOf(SavedData.open(SavedData.alignment)) != -1) {
				// Lop off <ALIGNMENT>
				data =
					data.substring(data.indexOf(SavedData
						.open(SavedData.alignment))
						+ SavedData.open(SavedData.alignment).length());
				// Get the name.
				name =
					data.substring(0,
						data.indexOf(SavedData.close(SavedData.alignment)));
				putInEventFrame(name, 4);
			}
			while (data.indexOf(SavedData.open(SavedData.setting)) != -1) {
				// Lop off <SETTING>
				data =
					data.substring(data.indexOf(SavedData
						.open(SavedData.setting))
						+ SavedData.open(SavedData.setting).length());
				// Get the name.
				name =
					data.substring(0,
						data.indexOf(SavedData.close(SavedData.setting)));
				putInEventFrame(name, 5);
			}
			while (data.indexOf(SavedData.open(SavedData.product)) != -1) {
				// Lop off <PRODUCT>
				data =
					data.substring(data.indexOf(SavedData
						.open(SavedData.product))
						+ SavedData.open(SavedData.product).length());
				// Get the name.
				name =
					data.substring(0,
						data.indexOf(SavedData.close(SavedData.product)));
				putInEventFrame(name, 6);
			}
			while (data.indexOf(SavedData.open(SavedData.beneficiary)) != -1) {
				// Lop off <BENEFICIARY>
				data =
					data.substring(data.indexOf(SavedData
						.open(SavedData.beneficiary))
						+ SavedData.open(SavedData.beneficiary).length());
				// Get the name.
				name =
					data.substring(0,
						data.indexOf(SavedData.close(SavedData.beneficiary)));
				putInEventFrame(name, 7);
			}
		} // End Event Frame.

		// Get implied deeds.
		// Ignore <IMPLIES>.
		while (data.indexOf(SavedData.open(SavedData.antecedent)) != -1) {
			// Lop off <(ANTECEDENT>
			data =
				data.substring(data.indexOf(SavedData
					.open(SavedData.antecedent))
					+ SavedData.open(SavedData.antecedent).length());
			// Get the implied deed's name.
			name =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.antecedent)));
			// Lop off <(_DEPLETES>
			data =
				data.substring(data.indexOf(SavedData
					.open(SavedData.depletesAntecedent))
					+ SavedData.open(SavedData.depletesAntecedent).length());
			boolean depleter =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.depletesAntecedent)))
					.equalsIgnoreCase("true");
			// Lop off <(_COMMUTES>
			data =
				data.substring(data.indexOf(SavedData
					.open(SavedData.commutesWithAntecedent))
					+ SavedData.open(SavedData.commutesWithAntecedent).length());
			boolean commuter =
				data.substring(
					0,
					data.indexOf(SavedData
						.close(SavedData.commutesWithAntecedent)))
					.equalsIgnoreCase("true");
			this.implications
				.addElement(new Relation(name, depleter, commuter));
		} // End of getting implied deeds.
			// Get generalizations.
		while (data.indexOf(SavedData.open(SavedData.generalizesTo)) != -1) {
			// Lop off <(GENERALIZES_TO>
			data =
				data.substring(data.indexOf(SavedData
					.open(SavedData.generalizesTo))
					+ SavedData.open(SavedData.generalizesTo).length());
			// Get the generalization's name.
			name =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.generalizesTo)));
			this.generalizations.addElement(new Relation(name, false, false));
		}
		// Get concrete deeds summarized by this deed.
		while (data.indexOf(SavedData.open(SavedData.summarizes)) != -1) {
			// Lop off <(SUMMARIZES>
			data =
				data.substring(data.indexOf(SavedData
					.open(SavedData.summarizes))
					+ SavedData.open(SavedData.summarizes).length());
			// Get the concrete deed's name.
			name =
				data.substring(0,
					data.indexOf(SavedData.close(SavedData.summarizes)));
			this.summarizedActions.addElement(new Relation(name, false, false));
		}
	} // End fromXMLString.

	boolean isSummaryEvent() {
		// Summary events always summarize something.
		if (this.summarizedActions.size() == 0) {
			return false;
		} else {
			return true;
		}
	} // End isSummaryEvent.

	/**
	 * Find concrete events that instantiate <code>this</code> event, if
	 * <code>this</code> is a generalization.
	 */
	Vector getInstantiators() {
		Vector eventsGeneralized = new Vector();
		Deed aDeed;
		Relation r;
		if (this.isGeneralization) {
			for (int i = 0; i < Ethno.deeds.size(); i++) {
				aDeed = (Deed) Ethno.deeds.elementAt(i);
				for (int j = 0; j < aDeed.generalizations.size(); j++) {
					r = (Relation) aDeed.generalizations.elementAt(j);
					if (r.implicated.equalsIgnoreCase(this.shortName)) {
						eventsGeneralized.addElement(aDeed);
					}
				}
			}
		}
		return eventsGeneralized;
	} // End instantiators.

	static boolean deedsAreAtDifferentGeneralizationLevels(
		Deed deed1, Deed deed2) {
		if ((deed1.isGeneralization & !deed2.isGeneralization)
			| (!deed1.isGeneralization & deed2.isGeneralization)) {
			return true;
		} else {
			return false;
		}
	} // End deedsAreAtDifferentGeneralizationLevels.

	private void putInEventFrame(String name, int classification) {
		int index = Ethno.entities.findPositionOfElement(name);
		if (index >= 0) {
			this.eventFrame[classification].addElement(Ethno.entities
				.elementAt(index));
		} else {
			index = Ethno.people.findPositionOfElement(name);
			if (index >= 0) {
				eventFrame[classification].addElement(Ethno.people
					.elementAt(index));
			} else {
				index = Ethno.actions.findPositionOfElement(name);
				if (index >= 0) {
					eventFrame[classification].addElement(Ethno.actions
						.elementAt(index));
				} else {
					Ethno.reportPage.results.append(
						Ethno.interfaceTexts
						.getString("missingElement") + name + "\n");
				}
			}
		}
	} // End putInEventFrame.

} // End class Deed.
