
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Choice;
import java.awt.FlowLayout;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.event.*;
import java.util.Vector;

/** Page for eliciting implications between Actions. */
public class DefineLinksCard extends Panel {

	/** List of actions used in elicitation. */
	PerceptList linkedDeeds, linkedDeedsToUseInQueries, linksToFocalDeed;

	/** Action involved in current question. */
	Deed deedBeingLinked, testDeed;

	/** The type of action being linked. */
	boolean analyzingConcreteDeeds, analyzingGeneralizedDeeds;

	/**
	 * Ask whether the focal action could have happened <b>without</b> the test
	 * action.
	 */
	boolean questionIsCounterfactual;

	/**
	 * Go up the hierarchy: start by asking whether the focal action requires
	 * the action that currently has the most prerequisites.
	 */
	boolean elicitingUpward;

	/** Items for the menu that determines phrasing of the linking question. */
	String questionText[][] = { Ethno.prerequisiteText, Ethno.implicationText,
		Ethno.causationText, Ethno.counterfactualText };

	String newLine = Ethno.interfaceTexts.getString("newLine");

	Panel manageElicitationArea = new Panel();

	Panel questionArea = new Panel();

	Panel yesNoArea = new Panel();

	Panel redoDoneArea = new Panel();

	Panel relinkArea = new Panel();

	Panel elicitationLogPanel = new Panel();

	Chart graphCanvas = new Chart();

	Label questionTypeLabel = new Label("", Label.CENTER);

	Label typeOfLogicLabel = new Label("", Label.CENTER);

	Label focalEventLabel = new Label("", Label.CENTER);

	Label[] elicitationQuestionLabels;

	Label elicitationLogLabel = new Label("", Label.CENTER);

	Label eraseLinksLabel = new Label("", Label.CENTER);

	Choice chooseQuestionType = new Choice();

	Choice linkedActionsMenu = new Choice();

	CheckboxGroup directionOfQuestioningGroup = new CheckboxGroup();

	Checkbox elicitFromBottomToTopCheckbox = new Checkbox("",
		directionOfQuestioningGroup, true);

	Checkbox elicitFromTopToBottomCheckbox = new Checkbox("",
		directionOfQuestioningGroup, false);

	Button redoThisQuestionButton = new Button();

	Button questioningCanEndButton = new Button();

	Button redoThisElicitationButton = new Button();

	Button yesButton = new Button();

	Button noButton = new Button();

	Button nextButton = new Button();

	TextArea topEvent = new TextArea("", 3, 25,
		TextArea.SCROLLBARS_VERTICAL_ONLY);

	TextArea bottomEvent = new TextArea("", 3, 25,
		TextArea.SCROLLBARS_VERTICAL_ONLY);

	TextArea elicitationLog =
		new TextArea("", 6, 100, TextArea.SCROLLBARS_BOTH);

	GridBagLayout manageElicitationLayout = new GridBagLayout();

	GridBagLayout relinkAreaLayout = new GridBagLayout();

	GridBagLayout questionAreaLayout = new GridBagLayout();

	FlowLayout yesNoAreaLayout = new FlowLayout();

	BorderLayout pageLayout = new BorderLayout();

	BorderLayout elicitationLogLayout = new BorderLayout();

	public DefineLinksCard() {
		try {
			jbInit();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void jbInit() throws Exception {
		yesNoArea.setLayout(yesNoAreaLayout);
		yesNoAreaLayout.setHgap(10);
		yesNoAreaLayout.setVgap(10);
		elicitationLogPanel.setLayout(elicitationLogLayout);

		pageLayout.setHgap(50);
		pageLayout.setVgap(10);
		this.setLayout(pageLayout);
		manageElicitationArea.setLayout(manageElicitationLayout);
		relinkArea.setLayout(relinkAreaLayout);
		questionArea.setLayout(questionAreaLayout);

		this.add(manageElicitationArea, BorderLayout.WEST);
		this.add(questionArea, BorderLayout.EAST);
		this.add(elicitationLogPanel, BorderLayout.SOUTH);
		this.add(graphCanvas, BorderLayout.CENTER);

		Chart.showingNamelessNodes = true;
		questionTypeLabel.setText(Ethno.interfaceTexts
			.getString("questionTypesLabelText"));
		for (int i = 0; i < Ethno.questionTypes.length; i++) {
			chooseQuestionType.addItem(Ethno.questionTypes[i]);
		}
		typeOfLogicLabel.setText(Ethno.interfaceTexts
			.getString("logicLabelText"));
		topEvent.setText("");
		bottomEvent.setText("");
		elicitationLog.setText(Ethno.linkingHistory);
		focalEventLabel.setText("");
		elicitFromBottomToTopCheckbox.setLabel(Ethno.interfaceTexts
			.getString("localText"));
		elicitFromTopToBottomCheckbox.setLabel(Ethno.interfaceTexts
			.getString("remoteText"));
		redoThisQuestionButton.setLabel(Ethno.interfaceTexts
			.getString("redoText"));
		questioningCanEndButton.setLabel(Ethno.interfaceTexts
			.getString("doneButtonText"));
		yesButton.setLabel(Ethno.interfaceTexts.getString("yesText"));
		noButton.setLabel(Ethno.interfaceTexts.getString("noText"));
		nextButton.setLabel(Ethno.interfaceTexts.getString("nextText"));
		eraseLinksLabel.setText(Ethno.interfaceTexts
			.getString("eraseTextTitle"));
		elicitationLogLabel.setAlignment(1);
		elicitationLogLabel.setText(Ethno.interfaceTexts
			.getString("answerRecordLabelText"));
		manageElicitationArea.add(questionTypeLabel,
			new SetOfGridBagConstraints(0, 1, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0));
		manageElicitationArea.add(chooseQuestionType,
			new SetOfGridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0));
		manageElicitationArea.add(typeOfLogicLabel,
			new SetOfGridBagConstraints(0, 3, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0));
		manageElicitationArea.add(elicitFromBottomToTopCheckbox,
			new SetOfGridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0));
		manageElicitationArea.add(elicitFromTopToBottomCheckbox,
			new SetOfGridBagConstraints(0, 5, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0));
		manageElicitationArea.add(focalEventLabel, new SetOfGridBagConstraints(
			0, 6, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
			GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		manageElicitationArea.add(redoDoneArea, new SetOfGridBagConstraints(0,
			7, 1, 1, 0.0, 0.0, GridBagConstraints.CENTER,
			GridBagConstraints.NONE, new Insets(0, 0, 0, 0), 0, 0));
		redoDoneArea.add(redoThisQuestionButton);
		redoThisQuestionButton.setEnabled(false);
		redoDoneArea.add(questioningCanEndButton);
		questioningCanEndButton.setEnabled(false);
		manageElicitationArea.add(relinkArea, new SetOfGridBagConstraints(0, 8,
			1, 1, 0.0, 2.0, GridBagConstraints.CENTER, GridBagConstraints.NONE,
			new Insets(0, 0, 0, 0), 0, 0));
		relinkArea.add(eraseLinksLabel, new SetOfGridBagConstraints(0, 1, 1, 1,
			0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NORTH,
			new Insets(0, 0, 0, 0), 0, 0));
		relinkArea.add(linkedActionsMenu, new SetOfGridBagConstraints(0, 2, 1,
			1, 0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.SOUTH,
			new Insets(0, 0, 0, 0), 0, 0));
		elicitationQuestionLabels = new Label[3];
		for (int i = 0; i < 3; i++) {
			elicitationQuestionLabels[i] = new Label();
		}
		questionArea.add(elicitationQuestionLabels[0],
			new SetOfGridBagConstraints(0, 0, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0));
		questionArea.add(topEvent, new SetOfGridBagConstraints(0, 1, 1, 1, 0.0,
			0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE,
			new Insets(0, 0, 0, 0), 0, 0));
		questionArea.add(elicitationQuestionLabels[1],
			new SetOfGridBagConstraints(0, 2, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0));
		questionArea.add(bottomEvent, new SetOfGridBagConstraints(0, 3, 1, 1,
			0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE,
			new Insets(0, 0, 0, 0), 0, 0));
		questionArea.add(elicitationQuestionLabels[2],
			new SetOfGridBagConstraints(0, 4, 1, 1, 0.0, 0.0,
				GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(
					0, 0, 0, 0), 0, 0));
		questionArea.add(yesNoArea, new SetOfGridBagConstraints(0, 5, 1, 1,
			0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE,
			new Insets(0, 0, 0, 0), 0, 0));
		yesNoArea.add(yesButton, null);
		yesNoArea.add(noButton, null);
		questionArea.add(nextButton, new SetOfGridBagConstraints(0, 6, 1, 1,
			0.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.NONE,
			new Insets(0, 0, 0, 0), 0, 0));
		yesButton.setEnabled(false);
		noButton.setEnabled(false);
		nextButton.setEnabled(false);
		elicitationLogPanel.add(elicitationLog, BorderLayout.CENTER);
		elicitationLogPanel.add(elicitationLogLabel, BorderLayout.NORTH);
		for (int i = 0; i < 3; i++) {
			elicitationQuestionLabels[i].setText(Ethno.prerequisiteText[i]);
		}
		analyzingConcreteDeeds = true;
		elicitingUpward = true;
		questionIsCounterfactual = false;
		linkedDeeds = new PerceptList();
		linkedDeedsToUseInQueries = new PerceptList();
		linksToFocalDeed = new PerceptList();
		deedBeingLinked = null;
		elicitationLog.addFocusListener(new FocusEventHandler());
		graphCanvas.addMouseListener(new java.awt.event.MouseAdapter() {
			public void mouseClicked(MouseEvent e) {
				showNameOfNodeThatWasCtrlClicked(e);
			}
		});
		chooseQuestionType.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				chooseQuestionType();
			}
		});
		redoThisQuestionButton
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					redoThisQuestion();
				}
			});
		questioningCanEndButton
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					stopQuestionsAboutFocalEvent();
				}
			});
		linkedActionsMenu.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				redoElicitation();
			}
		});
		yesButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				implementTheAnswerYes();
			}
		});
		nextButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				goToNextQuestion();
			}
		});
		noButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				implementTheAnswerNo();
			}
		});
		elicitFromBottomToTopCheckbox
			.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					redoThisQuestion();
				}
			});
		elicitFromTopToBottomCheckbox
			.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					redoThisQuestion();
				}
			});
	} // End jbInit.

	/**
	 * Establish links for focal action, and set up questioning for new focal
	 * action.
	 */
	void incorporateNewLinksAndSetUpNextQuestion() {
		Deed aDeed;
		int lastLinkedEvent;
		yesButton.setVisible(true);
		noButton.setVisible(true);
		if (linkedDeedsToUseInQueries.size() == 0) {
			// Done linking the focalDeed.
			linkedActionsMenu.add(deedBeingLinked.shortName);
			if (!elicitingUpward) {
				// Clear prerequisites of prerequisites.
				lastLinkedEvent = linksToFocalDeed.size() - 1;
				while (lastLinkedEvent > 0) {
					aDeed = (Deed) linksToFocalDeed.elementAt(lastLinkedEvent);
					Vector prerequisitesOfPrerequisite = new Vector();
					prerequisitesOfPrerequisite =
						linksToFocalDeed.upwardLinks(aDeed);
					for (int i = 0; i < prerequisitesOfPrerequisite.size(); i++) {
						aDeed = (Deed) prerequisitesOfPrerequisite.elementAt(i);
						linksToFocalDeed.removeElement(aDeed);
					}
					lastLinkedEvent =
						linksToFocalDeed.findPositionOfElement(aDeed.shortName) - 1;
				}
			}
			// Add the links just established for deedBeingLinked.
			for (int i = 0; i < linksToFocalDeed.size(); i++) {
				aDeed = (Deed) linksToFocalDeed.elementAt(i);
				deedBeingLinked.implications.addElement(new Relation(
					aDeed.shortName, true, false));
			}
			deedBeingLinked.isLinked = true;
			// Ethno.needNewGraph = true;
			setupLinking();
			deedBeingLinked = getNextUnlinkedDeed();
			graphCanvas.update(graphCanvas.getGraphics());
			if (deedBeingLinked == null) {
				linkingCompleted();
				return;
			}
		}
		presentQuestion();
	} // End incorporateNewLinksAndSetUpNextQuestion.

	/** Set up a new elicitation and present the first question. */
	void startANewElicitation() {
		if (Ethno.sequence.size() < 2) {
			// Too few actions are defined.
			// Ignore a linking request if there are 0 or 1 deeds.
			if (Ethno.sequence.size() < 2) {
				CancelOkDialog whoops =
					new CancelOkDialog((Frame) this.getParent().getParent(),
						Ethno.interfaceTexts.getString("cannotLink1"),
						Ethno.interfaceTexts.getString("cannotLink2"), "");
				whoops.setVisible(true);
			}
			return;
		}
		setupLinking();
		deedBeingLinked = getNextUnlinkedDeed();
		if (deedBeingLinked == null) { return; }
		presentQuestion();
	} // End elicit.

	/** Find the next unlinked action in the action sequence. */
	Deed getNextUnlinkedDeed() {
		Deed aDeed = null;
		boolean noneFound = true;
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			// Reset coloring of nodes on chart.
			aDeed = (Deed) Ethno.deeds.elementAt(i);
			aDeed.isAnswered = false;
		}
		for (int i = 0; i < Ethno.sequence.size(); i++) {
			aDeed = (Deed) Ethno.sequence.elementAt(i);
			if (analyzingConcreteDeeds && (!aDeed.isGeneralization)) {
				if (!aDeed.isLinked) {
					noneFound = false;
					break;
				}
			} else if (analyzingGeneralizedDeeds && aDeed.isGeneralization) {
				if (!aDeed.isLinked) {
					noneFound = false;
					break;
				}
			}
		}
		elicitationLog.append(newLine);
		if (noneFound) {
			aDeed = null;
			if (!Ethno.linkingHistory.contains(Ethno.interfaceTexts
				.getString("doneLinkingText"))) {
				elicitationLog.append(Ethno.interfaceTexts
					.getString("doneLinkingText") + newLine);
			}
		} else {
			elicitationLog.append(aDeed.shortName);
		}
		return aDeed;
	} // End getNextUnlinkedDeed.

	/**
	 * Get the actions to be used in the next series of questions, and set
	 * upward or downward questioning.
	 */
	void setupLinking() {
		Deed aDeed;
		linksToFocalDeed = new PerceptList();
		linkedActionsMenu.removeAll();
		// Reset coloring of nodes on chart.
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			aDeed = (Deed) Ethno.deeds.elementAt(i);
			aDeed.isAnswered = false;
		}
		// Collect the deeds that already are linked.
		analyzingConcreteDeeds = false;
		linkedDeedsToUseInQueries = new PerceptList();
		for (int i = 0; i < Ethno.sequence.size(); i++) {
			aDeed = (Deed) Ethno.sequence.elementAt(i);
			if ((!linkedDeedsToUseInQueries.contains(aDeed))
				&& (!aDeed.isGeneralization)) {
				// Skip repeats and generalizations:
				if (aDeed.isLinked) {
					linkedDeedsToUseInQueries.addElement(aDeed);
					linkedActionsMenu.add(aDeed.shortName);
				} else {
					// Found an unlinked concrete deed in the sequence list.
					analyzingConcreteDeeds = true;
				}
			}
		}
		// Link generalized deeds after concrete.
		analyzingGeneralizedDeeds = false;
		if ((!analyzingConcreteDeeds) && generalizationsAreAllInstantiated()) {
			linkedDeedsToUseInQueries.removeAllElements();
			for (int i = 0; i < Ethno.sequence.size(); i++) {
				aDeed = (Deed) Ethno.sequence.elementAt(i);
				if ((!linkedDeedsToUseInQueries.contains(aDeed))
					&& aDeed.isGeneralization) {
					// Skip repeats and concrete deeds.
					if (aDeed.isLinked) {
						linkedDeedsToUseInQueries.addElement(aDeed);
					} else {
						// Found an unlinked generalized deed in the sequence
						// list.
						analyzingGeneralizedDeeds = true;
					}
				}
			}
		}
		if ((linkedDeedsToUseInQueries.size() == 0)
			&& (analyzingConcreteDeeds || analyzingGeneralizedDeeds)) {
			// No deeds are linked yet. Start a new analysis with the first
			// deed.
			aDeed = getNextUnlinkedDeed();
			aDeed.isLinked = true;
			linkedDeedsToUseInQueries.addElement(aDeed);
		}
		linkedDeedsToUseInQueries = linkedDeedsToUseInQueries.topoSort();
		// Set local/remote logic.
		elicitingUpward = elicitFromBottomToTopCheckbox.getState();
	} // End setupLinking.

	/** Present the question, phrased according to type of query. */
	void presentQuestion() {
		int typeOfQuestion;
		String testText, focalText, answerRecordText;
		questionArea.invalidate();
		manageElicitationArea.invalidate();
		nextButton.invalidate();
		this.invalidate();
		typeOfQuestion = chooseQuestionType.getSelectedIndex();
		if (typeOfQuestion == Ethno.questionTypes.length - 1) {
			// Counterfactual is the last choice.
			questionIsCounterfactual = true;
			nextButton.setLabel(Ethno.interfaceTexts.getString("yesNextText"));
		} else {
			questionIsCounterfactual = false;
			nextButton.setLabel(Ethno.interfaceTexts.getString("noNextText"));
		}
		// Write the question frame.
		for (int i = 0; i < 3; i++) {
			elicitationQuestionLabels[i]
				.setText(questionText[typeOfQuestion][i]);
			elicitationQuestionLabels[i].invalidate();
		}
		if (deedBeingLinked == null) {
			questionArea.validate();
			manageElicitationArea.validate();
			return;
		}
		// Set up the redo / halt area.
		focalEventLabel.setText(deedBeingLinked.shortName);
		redoThisQuestionButton.setEnabled(true);
		questioningCanEndButton.setEnabled(true);
		focalEventLabel.invalidate();
		// Choose local or remote test deed.
		if (elicitingUpward) {
			testDeed = (Deed) linkedDeedsToUseInQueries.lastElement();
		} else {
			testDeed = (Deed) linkedDeedsToUseInQueries.firstElement();
		}
		// Use the longest text available.
		if (deedBeingLinked.verboseEventName.length() > 0) {
			focalText = deedBeingLinked.verboseEventName;
		} else {
			focalText = deedBeingLinked.shortName;
		}
		if (testDeed.verboseEventName.length() > 0) {
			testText = testDeed.verboseEventName;
		} else {
			testText = testDeed.shortName;
		}
		// Select test deed on chart.
		Deed visibleDeed;
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			visibleDeed = (Deed) Ethno.deeds.elementAt(i);
			if (visibleDeed.isADummy) continue;
			if (visibleDeed == testDeed) {
				visibleDeed.isSelected = true;
			} else {
				visibleDeed.isSelected = false;
			}
			graphCanvas.printNameOfDeed(visibleDeed, graphCanvas.getGraphics());
		}
		// Write the texts of the deeds.
		if (typeOfQuestion > 1) {
			// Historical causation or counterfactual.
			topEvent.setText(testText);
			bottomEvent.setText(focalText);
		} else {
			topEvent.setText(focalText);
			bottomEvent.setText(testText);
		}
		// Add info to answer record, but not when returning to card
		// where this already is done.
		testText = testDeed.shortName; // Use only the short name here.
		answerRecordText = elicitationLog.getText();
		if (!answerRecordText.endsWith(testText
			+ Ethno.interfaceTexts.getString("colonSpace"))) {
			elicitationLog.append(newLine
				+ Ethno.interfaceTexts.getString("indent") + testText
				+ Ethno.interfaceTexts.getString("colonSpace"));
		}
		chooseQuestionType.setEnabled(true);
		yesButton.setEnabled(true);
		noButton.setEnabled(true);
		nextButton.setEnabled(true);
		elicitFromBottomToTopCheckbox.setEnabled(true);
		elicitFromTopToBottomCheckbox.setEnabled(true);
		questionArea.validate();
		manageElicitationArea.validate();
	} // End presentQuestion.

	/** Clean up the page, and inform user that the elicitation is done. */
	void linkingCompleted() {
		topEvent.setText("");
		bottomEvent.setText("");
		redoThisQuestionButton.setEnabled(false);
		questioningCanEndButton.setEnabled(false);
		yesButton.setEnabled(false);
		noButton.setEnabled(false);
		nextButton.setEnabled(false);
		CancelOkDialog popupElicitationDone =
			new CancelOkDialog((Frame) this.getParent().getParent(),
				Ethno.interfaceTexts.getString("linkingCompletedTitle"),
				Ethno.interfaceTexts.getString("linkingCompletedText1"),
				Ethno.interfaceTexts.getString("linkingCompletedText2"));
		popupElicitationDone.setVisible(true);
		Chart.showingNamelessNodes = false;
	} // End linkingCompleted.

	/** Record the answer to the last elicitation question. */
	void addAnswerToLog(boolean testDeedRelatesToFocalDeed) {
		Deed aDeed;
		Vector deedsRequiringNoQuestion = new Vector();
		yesButton.setVisible(false);
		noButton.setVisible(false);
		String questionType = chooseQuestionType.getSelectedItem();
		testDeed.isAnswered = true;
		if (elicitingUpward) {
			if (testDeedRelatesToFocalDeed) {
				linksToFocalDeed.addElement(testDeed);
				deedsRequiringNoQuestion =
					linkedDeedsToUseInQueries.upwardLinks(testDeed);
				for (int i = 0; i < deedsRequiringNoQuestion.size(); i++) {
					// Reset coloring of forgone nodes on chart.
					aDeed = (Deed) deedsRequiringNoQuestion.elementAt(i);
					aDeed.isAnswered = true;
				}
				elicitationLog.append(Ethno.interfaceTexts
					.getString("linkedText")
					+ Ethno.interfaceTexts.getString("workingUpwardText")
					+ questionType
					+ Ethno.interfaceTexts.getString("questionEndPrenText"));
			} else { // Test deed unrelated to focal deed.
				elicitationLog.append(Ethno.interfaceTexts.getString("notText")
					+ Ethno.interfaceTexts.getString("linkedText")
					+ Ethno.interfaceTexts.getString("workingUpwardText")
					+ questionType
					+ Ethno.interfaceTexts.getString("questionEndPrenText"));
			}
		} else { // Eliciting downward.
			if (testDeedRelatesToFocalDeed) {
				linksToFocalDeed.addElement(testDeed);
				elicitationLog.append(Ethno.interfaceTexts
					.getString("linkedText")
					+ Ethno.interfaceTexts.getString("workingDownwardText")
					+ questionType
					+ Ethno.interfaceTexts.getString("questionEndPrenText"));
			} else { // Test deed unrelated to focal deed.
				deedsRequiringNoQuestion =
					linkedDeedsToUseInQueries.downwardLinks(testDeed);
				for (int i = 0; i < deedsRequiringNoQuestion.size(); i++) {
					// Reset coloring of forgone nodes on chart.
					aDeed = (Deed) deedsRequiringNoQuestion.elementAt(i);
					aDeed.isAnswered = true;
				}
				elicitationLog.append(Ethno.interfaceTexts.getString("notText")
					+ Ethno.interfaceTexts.getString("linkedText")
					+ Ethno.interfaceTexts.getString("workingDownwardText")
					+ questionType
					+ Ethno.interfaceTexts.getString("questionEndPrenText"));
			}
		}
		deedsRequiringNoQuestion.addElement(testDeed);
		for (int i = 0; i < deedsRequiringNoQuestion.size(); i++) {
			linkedDeedsToUseInQueries.removeElement(deedsRequiringNoQuestion
				.elementAt(i));
		}
		saveTheLog();
	} // End addAnswerToLog.

	/** Control-click means show full names. */
	void showNameOfNodeThatWasCtrlClicked(MouseEvent e) {
		if (e.isControlDown()) {
			graphCanvas.showFullNameInDialog(e);
		}
		graphCanvas.repaint();
	}

	/** Represent the question, phrased according to the selected type of query. */
	void chooseQuestionType() {
		presentQuestion();
	}

	/** Restart questions regarding the focal action. */
	void redoThisQuestion() {
		yesButton.setVisible(true);
		noButton.setVisible(true);
		elicitationLog.append(newLine
			+ Ethno.interfaceTexts.getString("indent")
			+ Ethno.interfaceTexts.getString("redoText"));
		setupLinking();
		presentQuestion();
	}

	/** Jettison some linking information already elicited, and redo. */
	void redoElicitation() {
		int first = linkedActionsMenu.getSelectedIndex();
		int last = linkedActionsMenu.getItemCount();
		yesButton.setVisible(true);
		noButton.setVisible(true);
		CancelOkDialog query =
			new CancelOkDialog((Frame) this.getParent().getParent(),
				Ethno.interfaceTexts.getString("cautionText"),
				Ethno.interfaceTexts.getString("eraseText"), "",
				Ethno.interfaceTexts.getString("okText"),
				Ethno.interfaceTexts.getString("cancelText"));
		query.setVisible(true);
		if (!Ethno.dialogCancel) {
			// Unlink selected action and subsequent ones
			Deed aDeed;
			elicitationLog.append(newLine
				+ Ethno.interfaceTexts.getString("restartText") + newLine);
			for (int i = first; i < last; i++) {
				String shortName = linkedActionsMenu.getItem(i);
				aDeed =
					(Deed) Ethno.deeds.elementAt(Ethno.deeds
						.findPositionOfElement(shortName));
				if (aDeed.isSummarized) {
					continue;
				}
				aDeed.isLinked = false;
				aDeed.halfNameWidth = aDeed.upCount = aDeed.downCount = 0;
				aDeed.horizontalPosition = aDeed.verticalPosition = 0;
				aDeed.tier = -1;
				if (Ethno.defineLinksCard.analyzingConcreteDeeds) {
					if (!aDeed.isGeneralization) {
						aDeed.implications.removeAllElements();
					}
				} else {
					if (aDeed.isGeneralization) {
						aDeed.implications.removeAllElements();
					}
				}
			}
			Ethno.defineLinksCard.linkedDeedsToUseInQueries.removeAllElements();
			// Ethno.needNewGraph = true;
			graphCanvas.repaint();
			startANewElicitation();
		}
	} // End of response to restartButton_actionPerformed.

	/** Respond to the user clicking the <i>Yes</i> button. */
	void implementTheAnswerYes() {
		// Yes=false for counterfactual, but yes=true when
		// counterfactual is false.
		addAnswerToLog(!questionIsCounterfactual);
		nextButton.setLabel(Ethno.interfaceTexts.getString("nextText"));
		chooseQuestionType.setEnabled(false);
		elicitFromBottomToTopCheckbox.setEnabled(false);
		elicitFromTopToBottomCheckbox.setEnabled(false);
	} // End of response to yesButton_actionPerformed.

	/** Respond to the user clicking the <i>No</i> button. */
	void implementTheAnswerNo() {
		// No=true for counterfactual, but no=false when
		// counterfactual is false.
		addAnswerToLog(questionIsCounterfactual);
		nextButton.setLabel(Ethno.interfaceTexts.getString("nextText"));
		chooseQuestionType.setEnabled(false);
		elicitFromBottomToTopCheckbox.setEnabled(false);
		elicitFromTopToBottomCheckbox.setEnabled(false);
	} // End of response to noButton_actionPerformed.

	/** Respond to the user clicking the <i>Next question</i> button. */
	void goToNextQuestion() {
		if (nextButton.getLabel().equalsIgnoreCase(
			Ethno.interfaceTexts.getString("yesNextText"))) {
			// Yes=false for counterfactual, but yes=true when counterfactual is
			// false.
			addAnswerToLog(!questionIsCounterfactual);
		} else if (nextButton.getLabel().equalsIgnoreCase(
			Ethno.interfaceTexts.getString("noNextText"))) {
			// No=true for counterfactual, but no=false when counterfactual is
			// false.
			addAnswerToLog(questionIsCounterfactual);
		} else { // For nextText.
		}
		incorporateNewLinksAndSetUpNextQuestion();
	} // End of response to nextButton_actionPerformed.

	/** User signals that there are no more links to be found for this action. */
	void stopQuestionsAboutFocalEvent() {
		linkedDeedsToUseInQueries.removeAllElements();
		incorporateNewLinksAndSetUpNextQuestion();
	} // End of response to doneButton_actionPerformed.

	/** Are there generalized actions, and are they all instantiated? */
	boolean generalizationsAreAllInstantiated() {
		boolean allInstantiated = true;
		boolean atLeastOneGeneralization = false;
		for (int i = 0; i < Ethno.deeds.size(); i++) {
			Deed aDeed = (Deed) Ethno.deeds.elementAt(i);
			if (aDeed.isGeneralization) {
				atLeastOneGeneralization = true;
				allInstantiated = allInstantiated && aDeed.isInstantiated;
			}
		}
		return allInstantiated && atLeastOneGeneralization;
	} // End generalizationsAllInstantiated.

	void saveTheLog() {
		Ethno.linkingHistory = this.elicitationLog.getText();
	} // End saveTheLog

	class FocusEventHandler extends FocusAdapter {
		public void focusLost(FocusEvent evt) {
			saveTheLog();
		}
	}

}