
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.*;
import java.util.Vector;

/** Page for defining events, and the event sequence. */
public class DefineSequenceCard extends Panel {

	/** Limit on short name length; overly long may not be formatted correctly. */
	static final int MAX_SHORT_NAME_LENGTH = 35;

	/** Index to selections on events list. */
	int lastSequenceSelection, currentSequenceSelection;

	Panel namingArea = new Panel();

	Panel sequenceArea = new Panel();

	Panel sequenceButtonsPanel = new Panel();

	Panel functionChoicePanel = new Panel();

	Panel buttonPanel = new Panel();

	Label shortNameTitle = new Label();

	Label shortNameInstruction = new Label();

	Label sequenceTitle = new Label();

	Label elaboratedText = new Label();

	TextField shortNameTextField = new TextField(MAX_SHORT_NAME_LENGTH);

	TextArea narrativeTextArea = new TextArea("", 10, 40,
		TextArea.SCROLLBARS_VERTICAL_ONLY);

	java.awt.List sequenceDisplay = new java.awt.List();

	Choice eventChoiceMenu = new Choice();

	Checkbox findFunctionCheckbox = new Checkbox();

	Checkbox repeatFunctionCheckbox = new Checkbox();

	CheckboxGroup popupFunctionGroup = new CheckboxGroup();

	Button cutButton = new Button();

	Button moveButton = new Button();

	BorderLayout pageLayout = new BorderLayout(5, 0);

	BorderLayout sequenceAreaLayout = new BorderLayout();

	BorderLayout sequenceButtonsPanelLayout = new BorderLayout();

	public DefineSequenceCard() {
		try {
			jbInit();
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	private void jbInit() throws Exception {
		this.setLayout(pageLayout);
		findFunctionCheckbox.setCheckboxGroup(popupFunctionGroup);
		repeatFunctionCheckbox.setCheckboxGroup(popupFunctionGroup);
		popupFunctionGroup.setSelectedCheckbox(repeatFunctionCheckbox);

		this.add(namingArea, BorderLayout.NORTH);
		this.add(sequenceArea, BorderLayout.WEST);
		this.add(narrativeTextArea, BorderLayout.CENTER);
		this.add(elaboratedText, BorderLayout.SOUTH);

		shortNameTitle.setText(Ethno.interfaceTexts
			.getString("abbreviationLabel"));
		shortNameTextField.setText("");
		shortNameInstruction.setText(Ethno.interfaceTexts
			.getString("abbreviationInstruction"));
		sequenceTitle.setText(Ethno.interfaceTexts.getString("sequenceTitle"));
		repeatFunctionCheckbox.setLabel(Ethno.interfaceTexts
			.getString("textRepeat"));
		findFunctionCheckbox.setLabel(Ethno.interfaceTexts
			.getString("textSelect"));
		cutButton.setLabel(Ethno.interfaceTexts.getString("textCut"));
		moveButton.setLabel(Ethno.interfaceTexts.getString("textMove"));
		narrativeTextArea.setText(Ethno.narrative);
		elaboratedText.setText(Ethno.interfaceTexts.getString("copyrightText"));
		narrativeTextArea.setFont(new Font("Monospaced", Font.PLAIN, 12));
		sequenceTitle.setAlignment(Label.CENTER);
		shortNameTitle.setAlignment(Label.RIGHT);
		elaboratedText.setAlignment(Label.RIGHT);

		namingArea.add(shortNameTitle, null);
		namingArea.add(shortNameTextField, null);
		namingArea.add(shortNameInstruction, null);
		sequenceArea.setLayout(sequenceAreaLayout);
		sequenceArea.add(sequenceTitle, BorderLayout.NORTH);
		sequenceArea.add(sequenceDisplay, BorderLayout.CENTER);
		sequenceArea.add(sequenceButtonsPanel, BorderLayout.SOUTH);
		sequenceButtonsPanel.setLayout(sequenceButtonsPanelLayout);
		sequenceButtonsPanel.add(buttonPanel, BorderLayout.NORTH);
		buttonPanel.add(moveButton, null);
		buttonPanel.add(cutButton, null);
		sequenceButtonsPanel.add(eventChoiceMenu, BorderLayout.CENTER);
		sequenceButtonsPanel.add(functionChoicePanel, BorderLayout.SOUTH);
		functionChoicePanel.add(repeatFunctionCheckbox, null);
		functionChoicePanel.add(findFunctionCheckbox, null);

		currentSequenceSelection = -1;

		shortNameTextField
			.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(ActionEvent e) {
					incorporateNewEvent();
				}
			});
		sequenceDisplay.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				processClickOnEvent();
			}
		});
		findFunctionCheckbox.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				popupMenuFunction();
			}
		});
		repeatFunctionCheckbox
			.addItemListener(new java.awt.event.ItemListener() {
				public void itemStateChanged(ItemEvent e) {
					popupMenuFunction();
				}
			});
		eventChoiceMenu.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				processChosenEvent();
			}
		});
		cutButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				removeSelectedEvents();
			}
		});
		moveButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				performFirstStepOfMove();
			}
		});
		narrativeTextArea.addFocusListener(new java.awt.event.FocusAdapter() {
			public void focusLost(FocusEvent e) {
				Ethno.narrative = narrativeTextArea.getText();
			}
		});
	} // End jbInit.

	/** Incorporate a new event, with quoted source text, into corpus. */
	void incorporateNewEvent() {
		String shortName = Ethno.cleanTheString(shortNameTextField.getText());
		String quote =
			Ethno.cleanTheString(narrativeTextArea.getSelectedText());
		if (shortName.equals("")) {
			CancelOkDialog whoops =
				new CancelOkDialog((Frame) this.getParent().getParent(),
					Ethno.interfaceTexts.getString("problemEncounteredText"),
					Ethno.interfaceTexts.getString("missingTextError"), "");
			whoops.setVisible(true);
		} else if (shortName.length() > MAX_SHORT_NAME_LENGTH) {
			CancelOkDialog whoops =
				new CancelOkDialog((Frame) this.getParent().getParent(),
					Ethno.interfaceTexts.getString("problemEncounteredText"),
					shortName,
					Ethno.interfaceTexts.getString("abbreviationTooLongError")
						+ (new Integer(MAX_SHORT_NAME_LENGTH)).toString());
			whoops.setVisible(true);
		} else {
			this.invalidate();
			// Incorporate the event in corpus.
			int indexToNewEvent =
				Ethno.deeds.insertAlphabetically(new Deed(shortName, quote));
			if (indexToNewEvent >= 0) {
				Deed x = (Deed) Ethno.deeds.elementAt(indexToNewEvent);
				Ethno.sequence.addElement(x);
				sequenceDisplay.add(shortName);
				Ethno.controls.rebuildEventsChoice();
				narrativeTextArea
					.select(narrativeTextArea.getSelectionEnd(), 0);
				shortNameTextField.setText("");
				shortNameTextField.select(0, 0);
				sequenceDisplay.makeVisible(sequenceDisplay.getItemCount() - 1);
			} else {
				// Returned index is minus, means duplication .
				CancelOkDialog whoops =
					new CancelOkDialog((Frame) this.getParent().getParent(),
						Ethno.interfaceTexts
							.getString("problemEncounteredText"),
						Ethno.interfaceTexts.getString("duplicateDeedError"),
						"");
				whoops.setVisible(true);
			}
			this.validate();
		}
	} // End response to shortNameTextField_actionPerformed.

	/** Deselect any selections in the events list. */
	void clearSelectionsInEventsList() {
		moveButton.setBackground(Color.lightGray);
		for (int i = 0; i < sequenceDisplay.getItemCount(); i++) {
			sequenceDisplay.deselect(i);
		}
	}

	/** Following a click in the event list, select, deselect, or move an event. */
	void processClickOnEvent() {
		// Find mode precludes working in list:
		if (findFunctionCheckbox.getState()) {
			clearSelectionsInEventsList();
			return;
		}
		if (moveButton.getBackground() == Color.red) {
			moveEvent();
			return;
		}
		// Select or deselect in sequence display.
		lastSequenceSelection = currentSequenceSelection;
		currentSequenceSelection = sequenceDisplay.getSelectedIndex();
		if (lastSequenceSelection == currentSequenceSelection) {
			// Deselect.
			sequenceDisplay.deselect(currentSequenceSelection);
			currentSequenceSelection = sequenceDisplay.getSelectedIndex();
			// Remove event description.
			elaboratedText.setText(Ethno.interfaceTexts
				.getString("copyrightText"));
			elaboratedText.setAlignment(Label.RIGHT);
		} else {
			// Show event description of selection at the bottom of screen.
			String shortName = sequenceDisplay.getSelectedItem();
			int selectedDeedNumber =
				Ethno.deeds.findPositionOfElement(shortName);
			Deed x = (Deed) Ethno.deeds.elementAt(selectedDeedNumber);
			if (x.verboseEventName.length() > 0) {
				elaboratedText.setText(x.verboseEventName);
				elaboratedText.setAlignment(Label.LEFT);
			} else {
				elaboratedText.setText(shortName);
				elaboratedText.setAlignment(Label.LEFT);
			}
		}
	} // End response to sequenceDisplay_itemStateChanged.

	/**
	 * User's choice of event in popup menu gets repeated in events list, or
	 * else all instances are highlighted.
	 */
	void processChosenEvent() {
		clearSelectionsInEventsList();
		String workString = eventChoiceMenu.getSelectedItem();
		if (repeatFunctionCheckbox.getState()) {
			// Repeat the chosen event.
			Ethno.sequence.addElement(Ethno.deeds.elementAt(Ethno.deeds
				.findPositionOfElement(workString)));
			sequenceDisplay.add(workString);
			sequenceDisplay.makeVisible(sequenceDisplay.getItemCount() - 1);
		} else {
			// Highlight instances of the chosen event.
			for (int i = 0; i < sequenceDisplay.getItemCount(); i++) {
				if (sequenceDisplay.getItem(i).equalsIgnoreCase(workString)) {
					sequenceDisplay.select(i);
				}
			}
		}
	} // End response to eventChoiceMenu_itemStateChanged.

	/** Color <i>Move</i> button red in preparation for step two of a move. */
	void performFirstStepOfMove() {
		if (currentSequenceSelection < 0) { return; }
		lastSequenceSelection = currentSequenceSelection;
		moveButton.setBackground(Color.red);
		findFunctionCheckbox.setEnabled(false);
	} // End moveButton_actionPerformed.

	/** Perform second step of an event move. */
	void moveEvent() {
		// Called from when movingAnEvent = true.
		currentSequenceSelection = sequenceDisplay.getSelectedIndex();
		Deed theDeedBeingMoved =
			(Deed) Ethno.sequence.elementAt(lastSequenceSelection);
		Ethno.sequence.insertElementAt(theDeedBeingMoved,
			currentSequenceSelection);
		if (currentSequenceSelection <= lastSequenceSelection) {
			Ethno.sequence.removeElementAt(lastSequenceSelection + 1);
		} else {
			Ethno.sequence.removeElementAt(lastSequenceSelection);
		}
		int newPosition = currentSequenceSelection;
		reconstructSequenceDisplay();
		moveButton.setBackground(Color.lightGray);
		sequenceDisplay.makeVisible(Math.min(newPosition,
			sequenceDisplay.getItemCount() - 1));
		findFunctionCheckbox.setEnabled(true);
	} // End moveEvent.

	/** Remove all selected instances of an event. */
	void removeSelectedEvents() {
		Deed aDeed = null;
		int arrayIndex, indexOfDeedToRemove = 0;
		int[] selectedLines = sequenceDisplay.getSelectedIndexes();
		if (selectedLines.length == 0) { return; // Nothing selected.
		}
		boolean thisIsSoleInstance = false;
		if (repeatFunctionCheckbox.getState()) {
			// In repeat mode have to check whether the selection is the only
			// instance in sequence because other instances might be unselected.
			// (In find mode all instances are selected.)
			String nameOfDeedToRemove = sequenceDisplay.getSelectedItem();
			aDeed =
				(Deed) Ethno.deeds.elementAt(Ethno.deeds
					.findPositionOfElement(nameOfDeedToRemove));
			int first = Ethno.sequence.indexOf(aDeed);
			int last = Ethno.sequence.lastIndexOf(aDeed);
			if (first == last) {
				thisIsSoleInstance = true;
			}
		}
		// Cut selected events.
		for (arrayIndex = selectedLines.length - 1; arrayIndex >= 0; arrayIndex--) {
			// Work backward in list of selections.
			String nameOfDeedToRemove =
				sequenceDisplay.getItem(selectedLines[arrayIndex]);
			aDeed =
				(Deed) Ethno.deeds.elementAt(Ethno.deeds
					.findPositionOfElement(nameOfDeedToRemove));
			if ((arrayIndex == 0) && findFunctionCheckbox.getState()) {
				// All repeats selected with find-function.
				thisIsSoleInstance = true;
			}
			if (thisIsSoleInstance) {
				// Last instance in sequence, so delete from alphabetic list of
				// events.
				purgeAnEvent(aDeed);
			} else {
				// Remove this instance from sequence.
				indexOfDeedToRemove = selectedLines[arrayIndex];
				aDeed = (Deed) Ethno.sequence.elementAt(indexOfDeedToRemove);
				if (aDeed.isSummarized) {
					continue; // Don't remove events that are in summary sets.
				}
				Ethno.sequence.removeElementAt(indexOfDeedToRemove);
			}// End if (thisIsSoleInstance) else.
		} // End for arrayIndex.
		reconstructSequenceDisplay();
		sequenceDisplay.makeVisible(Math.min(indexOfDeedToRemove,
			sequenceDisplay.getItemCount() - 1));
	} // End removeSelectedEvents.

	/** Rebuild the list showing the sequence of events. */
	public void reconstructSequenceDisplay() {
		Deed aDeedInTheSequence;
		clearSelectionsInEventsList();
		this.invalidate();
		sequenceDisplay.removeAll();
		for (int i = 0; i < Ethno.sequence.size(); i++) {
			aDeedInTheSequence = (Deed) Ethno.sequence.elementAt(i);
			if (!aDeedInTheSequence.isSummarized) {
				sequenceDisplay.add(aDeedInTheSequence.shortName);
			}
		}
		elaboratedText.setText(Ethno.interfaceTexts.getString("copyrightText"));
		elaboratedText.setAlignment(Label.RIGHT);
		this.validate();
		lastSequenceSelection = -1;
		currentSequenceSelection = -1;
	} // End reconstructSequenceDisplay.

	void purgeAnEvent(Deed deletedDeed) {
		CancelOkDialog danger =
			new CancelOkDialog((Frame) this.getParent().getParent(),
				Ethno.interfaceTexts.getString("cautionText"),
				Ethno.interfaceTexts.getString("removingDeedWarningText"),
				deletedDeed.shortName,
				Ethno.interfaceTexts.getString("removeText"),
				Ethno.interfaceTexts.getString("cancelText"));
		danger.setVisible(true);
		if (!Ethno.dialogCancel) {
			// Expunge the deed.
			Deed testDeed;
			Relation deletedDeedRelation;
			Vector implicandsOfTestDeed;
			String theImplicandOfDeletedDeed, anImplicandOfTestDeed, theImplicandOfTestDeed;
			String deletedDeedName = deletedDeed.shortName;
			boolean deletingSummarization =
				deletedDeed.summarizedActions.size() > 0;
			// Transfer implications of deletedDeed to deletedDeed's implicands.
			for (int i = 0; i < Ethno.deeds.size(); i++) {
				testDeed = (Deed) Ethno.deeds.elementAt(i);
				if (testDeed.equals(deletedDeed)) {
					continue;
				}
				int j = -1;
				while (++j < testDeed.implications.size()) { // Have to allow
					// for changing
					// size.
					anImplicandOfTestDeed =
						((Relation) testDeed.implications.elementAt(j)).implicated;
					if (anImplicandOfTestDeed.equalsIgnoreCase(deletedDeedName)) {
						// Transfer deletedDeed's implications.
						if (deletedDeed.summarizedActions.size() > 0) {
							// deletedDeed is a summarization, so transfer to
							// the original bottom node.
							deletingSummarization = true;
							((Relation) testDeed.implications.elementAt(j)).implicated =
								deletedDeed.linkToFinalSummarizedAction.implicated;
						} else {
							// Remove the link to deletedDeed.
							// Have/ to step back j after deletion:
							testDeed.implications.removeElementAt(j--);
							// Implicands without going through deletedDeed:
							implicandsOfTestDeed =
								Ethno.deeds.upwardLinks(testDeed);
							// Don't transfer implications that already exist.
							for (int k = 0; k < deletedDeed.implications.size(); k++) {
								boolean okToTransferImplication = true;
								deletedDeedRelation =
									(Relation) deletedDeed.implications
										.elementAt(k);
								theImplicandOfDeletedDeed =
									deletedDeedRelation.implicated;
								for (int m = 0; m < implicandsOfTestDeed.size(); m++) {
									theImplicandOfTestDeed =
										((Deed) implicandsOfTestDeed
											.elementAt(m)).shortName;
									if (theImplicandOfTestDeed
										.equalsIgnoreCase(theImplicandOfDeletedDeed)) {
										okToTransferImplication = false;
									}
								}
								if (okToTransferImplication) {
									testDeed.implications
										.addElement(deletedDeedRelation);
								}
							} // End transfering deletedDeed's implicands.
						} // End distinguishing summarizations.
					} // End if testDeed implicand = deletedDeed.
				} // End while traversal of testDeed.implications.
			} // End traversal of deeds.
			if (deletingSummarization) { // Restore summarized deeds if
				// deleting a summarization.
				for (int i = 0; i < deletedDeed.summarizedActions.size(); i++) {
					String nameOfDeedToRestore =
						((Relation) deletedDeed.summarizedActions.elementAt(i)).implicated;
					Deed restoredDeed =
						(Deed) Ethno.deeds.elementAt(Ethno.deeds
							.findPositionOfElement(nameOfDeedToRestore));
					restoredDeed.isSummarized = false;
				}
			}
			while (Ethno.sequence.removeElement(deletedDeed))
				;
			reconstructSequenceDisplay();
			Ethno.deeds.removeElement(deletedDeed); // Gone.
			Ethno.controls.rebuildEventsChoice();
			// Ethno.needNewGraph = true;
		}
	} // End purgeAnEvent.

	void popupMenuFunction() {
		moveButton.setBackground(Color.lightGray);
		clearSelectionsInEventsList();
		if (repeatFunctionCheckbox.getState()) {
			sequenceDisplay.setMultipleMode(false);
			moveButton.setEnabled(true);
		} else {
			sequenceDisplay.setMultipleMode(true);
			moveButton.setEnabled(false);
		}
	} // End popupMenuFunction.

}