
import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Frame;
import java.awt.Panel;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.Locale;
import java.util.ResourceBundle;

//Conversion to app from: http://www.cs.tcu.edu/10403/AppletToApplication.pdf

/**
 * An applet for analyzing logical relations in events and in series of events.
 * 
 * @author David Heise
 */

public class Ethno extends Frame implements WindowListener {

	/** Internal tag for a page (card) that is presented by the program. */
	static final String DEFINE_SEQUENCE = "DEFINE_SEQUENCE";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String DEFINE_LINKS = "DEFINE_LINKS";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String GRAPH = "GRAPH";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String SHOW_DEED = "SHOW_DEED";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String SHOW_ELEMENT = "SHOW_ELEMENT";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String IMPORT_EXPORT = "IMPORT_EXPORT";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String ANALYSES = "ANALYSES";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String ENTITIES = "ENTITIES";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String PEOPLE = "PEOPLE";

	/** Internal tag for a page (card) that is presented by the program. */
	static final String ACTIONS = "ACTIONS";

	/** Flag for one of the basic types of elements. */
	static final int ENTITY = 0;

	/** Flag for one of the basic types of elements. */
	static final int PERSON = 1;

	/** Flag for one of the basic types of elements. */
	static final int ACTION = 2;

	/** Menus at top of the display. */
	static ControlMenus controls;

	/** Page for saving or restoring models. */
	static ImportExportCard importExportCard;

	/** Page for inputting the sequence of events. */
	static DefineSequenceCard defineSequenceCard;

	/** Page for examining and changing an event's characteristics. */
	static ShowDeedCard showDeedCard;

	/** Page to examine or change an entity, person, or action. */
	static ShowPerceptCard showElementCard;

	/** Page for establishing how events imply one another. */
	static DefineLinksCard defineLinksCard;

	/** Page displaying event model, and allowing editing and tests. */
	static ChartCard graphCard;

	/** Page for showing results of analyses. */
	static ReportPage reportPage;

	// LOCALES

	static Locale America = new Locale("en", "US");

	static Locale currentLocale;


	/** Interface texts. */

	static ResourceBundle interfaceTexts;

	static Panel displayPanel = new Panel();

	static PerceptList deeds, entities, people, actions, sequence;

	static String linkingHistory = "";

	static String narrative;

	static boolean dialogCancel, generalizationsAllInstantiated;

	static String theTaskNow = DEFINE_SEQUENCE;

	static String[] eventFrameName, operationLine, graphModeLine, elementName;

	static String[] questionTypes, prerequisiteText, implicationText,
			causationText, counterfactualText;

	static int currentEvent;

	static int[][] eventCounts;

	/** Dimensions. */
	
	static int width, height;

	static CardLayout card = new CardLayout();

	BorderLayout ethnoLayout = new BorderLayout();

	/** Initialize */
	public Ethno() {

		// SET LOCALE.
		currentLocale = America;

		try {
			jbInit();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void main(String[] args){
		Ethno f = new Ethno();
		f.setSize(1200,700);
		f.setVisible(true);
	}

	public void windowClosing(WindowEvent e) {
		dispose();
		System.exit(0);
	}

	public void windowOpened(WindowEvent e) {
	}

	public void windowIconified(WindowEvent e) {
	}

	public void windowClosed(WindowEvent e) {
	}

	public void windowDeiconified(WindowEvent e) {
	}

	public void windowActivated(WindowEvent e) {
	}

	public void windowDeactivated(WindowEvent e) {
	}
	

	private void jbInit() throws Exception {
		this.addWindowListener(this);

		// Set color.
		this.setBackground(Color.white);

		// Save width and height.
		width = this.getSize().width;
		height = this.getSize().height;

		// Define the data vectors.
		sequence = new PerceptList();
		deeds = new PerceptList();
		entities = new PerceptList();
		people = new PerceptList();
		actions = new PerceptList();

		// Get interface texts.
		interfaceTexts =
			ResourceBundle.getBundle("InterfaceTexts", currentLocale);
		eventFrameName =
			(String[]) interfaceTexts.getStringArray("eventFrameName");
		operationLine =
			(String[]) interfaceTexts.getStringArray("operationLineName");
		elementName = (String[]) interfaceTexts.getStringArray("elementName");
		graphModeLine =
			(String[]) interfaceTexts.getStringArray("graphModeLine");
		questionTypes =
			(String[]) interfaceTexts.getStringArray("questionTypes");
		prerequisiteText =
			(String[]) interfaceTexts.getStringArray("PrerequisiteText");
		implicationText =
			(String[]) interfaceTexts.getStringArray("ImplicationText");
		causationText =
			(String[]) interfaceTexts.getStringArray("CausationText");
		counterfactualText =
			(String[]) interfaceTexts.getStringArray("CounterfactualText");
		narrative = Ethno.interfaceTexts.getString("initialNarrativeText");
		// Define the cards.
		defineSequenceCard = new DefineSequenceCard();
		defineLinksCard = new DefineLinksCard();
		graphCard = new ChartCard();
		showDeedCard = new ShowDeedCard();
		showElementCard = new ShowPerceptCard();
		importExportCard = new ImportExportCard();
		reportPage = new ReportPage();

		// Layout a control area above card content.
		controls = new ControlMenus();
		this.setLayout(ethnoLayout);
		this.setBackground(Color.white);
		this.add(displayPanel, BorderLayout.CENTER); // This first to put
		// focus in text field.
		this.add(controls, BorderLayout.NORTH);
		displayPanel.setLayout(card);

		displayPanel.add(DEFINE_SEQUENCE, defineSequenceCard);
		displayPanel.add(DEFINE_LINKS, defineLinksCard);
		displayPanel.add(GRAPH, graphCard);
		displayPanel.add(SHOW_DEED, showDeedCard);
		displayPanel.add(SHOW_ELEMENT, showElementCard);
		displayPanel.add(ANALYSES, reportPage);
		displayPanel.add(IMPORT_EXPORT, importExportCard);

		card.show(displayPanel, DEFINE_SEQUENCE);
		controls.operationsChoice.select(ControlMenus.DEFINE_SEQUENCE);

		// Graph parameters.
		generalizationsAllInstantiated = false;

	} // End jbInit.

	/** Format one and two digit numbers. */
	static String format0_99(int number) {
		if (number < 10) {
			return (" " + number);
		} else {
			Integer num = new Integer(number);
			return (num.toString());
		}
	} // End format0_99.

	/** Remove characters that confound export-import. */
	static String cleanTheString(String input) {
		input = input.replace('\r', ' ');
		input = input.replace('\n', ' ');
		input = input.replace('#', ' ');
		input = input.replace('@', ' ');
		return input;
	} // End cleanTheString.

}