
import java.util.Vector;

/**
 * Class for entities, persons, and actions; <code>deed</code> is an extension.
 */
class Percept {

	/** The shortest name of the element. */
	String shortName;

	/** Bucket for AKO elements, each specified as a Relation. */
	Vector implications;

	/* Constructor method. */
	public Percept() {
		shortName = new String("");
		implications = new Vector(2);
	} // End constructor.

	/** Transform percept to text version. */
	public String toString() {
		Relation link;
		String output = "";
		output = "#" + this.shortName + "\n";
		for (int j = 0; j < this.implications.size(); j++) {
			link = (Relation) this.implications.elementAt(j);
			output = output + link.implicated + "\n";
			output = output + link.depletes + "\n";
			output = output + link.commutes + "\n";
		}
		return output;
	}

	/** Transform percept to XML text version. */
	String toXML() {
		Relation link;
		String output =
			"<" + SavedData.name + ">" + this.shortName + "</" + SavedData.name
				+ ">\n";
		for (int j = 0; j < this.implications.size(); j++) {
			link = (Relation) this.implications.elementAt(j);
			output =
				output + "<" + SavedData.implied + ">" + link.implicated + "</"
					+ SavedData.implied + ">\n";
		}
		return output;
	}

	/** Transform text version to percept; kept for legacy datasets. */
	public int fromString(String data, int first) {
		int last;
		String implicated;
		boolean depletes, commutes;
		if (data.charAt(first) != '#') {
			Ethno.reportPage.results.append(
					"\n\nPercept expected but not found.\n\n");
			return (first + 1);
		} else
			first++;
		if (data.charAt(first) == '\n') {
			Ethno.reportPage.results.append(
					"\n\nPercept with no name.\n\n");
			return (first + 1);
		} else {
			last = data.indexOf('\n', first);
			this.shortName = data.substring(first, last);
			first = ++last;
		}
		while ((data.charAt(first) != '#') && (data.charAt(first) != '@')) {
			if (data.charAt(first) == '\n') {
				Ethno.reportPage.results.append(
						"\n\nPercept implication with no name.\n\n");
				return (last + 1);
			} else {
				last = data.indexOf('\n', first);
				implicated = data.substring(first, last);
				first = ++last;
			}
			if (data.charAt(first) == '\n') {
				Ethno.reportPage.results.append(
						"\n\nPercept depletes unspecified.\n\n");
				return (last + 1);
			} else {
				last = data.indexOf('\n', first);
				depletes = (data.substring(first, last).equals("true"));
				first = ++last;
			}
			if (data.charAt(first) == '\n') {
				Ethno.reportPage.results.append(
						"\n\nPercept commutes unspecified.\n\n");
				return (last + 1);
			} else {
				last = data.indexOf('\n', first);
				commutes = (data.substring(first, last).equals("true"));
				first = ++last;
			}
			this.implications.addElement(new Relation(implicated, depletes,
				commutes));
		}
		return first;
	} // End fromString.

	void fromXMLString(String text) {
		String name, implicated;
		if (!(text.substring(0, SavedData.open(SavedData.name).length()))
			.equalsIgnoreCase(SavedData.open(SavedData.name))) {
			Ethno.reportPage.results.append(
				"\n\n" + Ethno.interfaceTexts.getString("unnamedPercept") + "\n\n");
			return;
		}
		// Lop off <NAME>
		text =
			text.substring(text.indexOf(SavedData.open(SavedData.name))
				+ SavedData.open(SavedData.name).length());
		// Get the name.
		name = text.substring(0, text.indexOf(SavedData.close(SavedData.name)));
		if (name.length() == 0) {
			Ethno.reportPage.results.append(
				"\n\n" + Ethno.interfaceTexts.getString("unnamedPercept") + "\n\n");
			return;
		}
		this.shortName = name;
		while (text.indexOf(SavedData.open(SavedData.implied)) != -1) {
			// Lop off <IMPLIED> and prior text.
			text =
				text.substring(text.indexOf(SavedData.open(SavedData.implied))
					+ SavedData.open(SavedData.implied).length());
			// Get the name.
			implicated =
				text.substring(0,
					text.indexOf(SavedData.close(SavedData.implied)));
			if (name.length() == 0) {
				Ethno.reportPage.results.append(
					"\n\n" + name
					+ Ethno.interfaceTexts.getString("unnamedImplication") + "\n\n");
				return;
			}
			// No depletion or commutation for percepts.
			boolean depletes = false;
			boolean commutes = false;
			this.implications.addElement(new Relation(implicated, depletes,
				commutes));
		} // End while.
	} // End fromXMLString.

} // End class Percept.

