
import java.util.Vector;

/**
 * Variables and methods relating to a set of events, entities, persons, or
 * actions.
 */
class PerceptList extends Vector {
	/*
	 * This class contains: UTILITY METHODS. getPerceptList, odd,
	 * isGeneralizedModel METHODS FOR DEEDS. insertAlphabetic, reposition,
	 * positionOfDeed METHODS FOR PERCEPTS. insertAlphabetic, sortElementNames,
	 * reposition, duplicatedPercept, positionOfElement, deedsContaining,
	 * downwardLinks, upwardLinks METHODS FOR BARYPOSITIONING. topoSort,
	 * baryRanks, shellSort, baryPositions, constrainTier, maxEventsOnTier
	 * METHODS FOR SEARCHING. quickLinks, findDeedWithLinks
	 */

	public static final int MAX_ITERATIONS_FOR_RANKING = 50;

	public static final int MAX_ITERATIONS_FOR_POSITIONING = 20;

	public static final int MAX_NUMBER_OF_TIERS = 250;

	public static final int SEARCH_DOWN = 0;

	public static final int SEARCH_UP = 1;

	public PerceptList() { // Constructor method.
		super();
	}

	/* UTILITY METHODS. */

	/** Get the current list of elements of the given type. */
	public static PerceptList getPerceptList(int typeOfElement) {
		PerceptList elementList = new PerceptList();
		elementList = Ethno.entities; // Dummy initialization.
		if (typeOfElement == Ethno.ENTITY) {
			elementList = Ethno.entities;
		}
		if (typeOfElement == Ethno.PERSON) {
			elementList = Ethno.people;
		}
		if (typeOfElement == Ethno.ACTION) {
			elementList = Ethno.actions;
		}
		return elementList;
	} // getPerceptList.

	/** Return true if the integer is odd. */
	public boolean isOdd(int number) {
		if (2 * (number / 2) == number) {
			return false;
		} else {
			return true;
		}
	} // End isOdd.

	/** Return true if the model of events has generalizations. */
	public boolean isGeneralizedModel() {
		Deed aDeed;
		boolean haveGeneralization = false;
		for (int i = 0; i < this.size(); i++) {
			aDeed = (Deed) this.elementAt(i);
			haveGeneralization = haveGeneralization || aDeed.isGeneralization;
		}
		return haveGeneralization;
	} // End isGeneralizedModel.

	/** Return true if all events are generalizations. */
	public boolean isGeneralizedModelOnly() {
		Deed aDeed;
		boolean haveGeneralization = true;
		for (int i = 0; i < this.size(); i++) {
			aDeed = (Deed) this.elementAt(i);
			haveGeneralization = haveGeneralization & aDeed.isGeneralization;
		}
		return haveGeneralization;
	} // End isGeneralizedModel.

	/**
	 * Returns position of element inserted alphabetically in <code>this</code>,
	 * after an alphabetical insertion on short names..
	 */
	public int insertAlphabetically(Percept x) {
		Percept trialElement;
		if (duplicatedPercept(-1, x.shortName)) return -1;
		if (this.isEmpty()) {
			this.addElement(x);
			return 0;
		} else {
			for (int i = 0; i < this.size(); i++) {
				trialElement = (Percept) this.elementAt(i);
				if (x.shortName.compareTo(trialElement.shortName) < 0) {
					this.insertElementAt(x, i);
					return i;
				}
			}
			this.addElement(x); // Beyond all existing elements.
			return (this.size() - 1);
		}
	} // End insertAlphabetic.

	/** Order elements alphabetically by their short names. */
	public PerceptList sortElementNames() {
		PerceptList sorted = new PerceptList();
		Percept tempElement1, tempElement2;
		LoopThroughOriginal: for (int index = 0; index < this.size(); index++) {
			tempElement1 = (Percept) this.elementAt(index);
			if (sorted.isEmpty()) {
				sorted.addElement(tempElement1);
				continue LoopThroughOriginal;
			}
			for (int i = 0; i < sorted.size(); i++) {
				tempElement2 = (Percept) sorted.elementAt(i);
				if (tempElement1.shortName.compareTo(tempElement2.shortName) < 0) {
					sorted.insertElementAt(tempElement1, i);
					continue LoopThroughOriginal;
				}
			}
			sorted.addElement(tempElement1); // Beyond all existing elements.
		}
		return sorted;
	} // End sortElementNames.

	/**
	 * Returns new position in <code>this</code> of percept with a revised short
	 * name.
	 */
	public int repositionElement(int oldPosition, Percept newVersion) {
		Percept trialElement;
		if (duplicatedPercept(oldPosition, newVersion.shortName)) return -1;
		// Insert the new version and take out the old.
		for (int i = 0; i < this.size(); i++) {
			if (i != oldPosition) {
				trialElement = (Percept) this.elementAt(i);
				if (newVersion.shortName.compareTo(trialElement.shortName) < 0) {
					insertElementAt(newVersion, i);
					if (i <= oldPosition) {
						this.removeElementAt(oldPosition + 1);
						return i;
					} else {
						this.removeElementAt(oldPosition);
						return i - 1;
					}
				}
			}
		}
		this.addElement(newVersion); // Beyond all existing elements.
		this.removeElementAt(oldPosition);
		return (this.size() - 1);
	} // End reposition.

	/** Check for duplication of an element name. */
	public boolean duplicatedPercept(int oldPosition, String newName) {
		Percept y;
		for (int i = 0; i < this.size(); i++) {
			if (!(i == oldPosition)) {
				y = (Percept) this.elementAt(i);
				if (newName.compareTo(y.shortName) == 0) { return true; // Duplicate.
				}
			}
		}
		return false;
	} // End duplicatedPercept.

	/** Returns position of the element that has elementName. */
	public int findPositionOfElement(String elementName) {
		Percept trialElement;
		if (this.isEmpty()) {
			return -1; // Can't be in an empty vector.
		} else {
			for (int i = 0; i < this.size(); i++) {
				trialElement = (Percept) this.elementAt(i);
				if (trialElement.shortName.equals(elementName)) { return i; }
			}
		}
		return (-1); // Not found.
	} // End positionOfElement.

	/**
	 * Collect deeds from Ethno.deeds that contain a percept in any event-frame
	 * category.
	 */
	public Vector deedsContaining(
		Percept referent, boolean wantConcreteDeedsRatherThanGeneralizations) {
		Deed aDeed;
		Percept constituent;
		Vector facet, setOfDeeds;
		setOfDeeds = new Vector();
		for (int i = 0; i < this.size(); i++) { // Traverse the list of deeds.
			aDeed = (Deed) this.elementAt(i);
			if (!(aDeed.isGeneralization & wantConcreteDeedsRatherThanGeneralizations)) {
				// Traverse Event Frame categories.
				search: for (int j = 0; j < 8; j++) {
					facet = aDeed.eventFrame[j];
					for (int k = 0; k < facet.size(); k++) {
						// Tranverse percepts in each category.
						constituent = (Percept) facet.elementAt(k);
						if (constituent.shortName
							.equalsIgnoreCase(referent.shortName)) {
							setOfDeeds.addElement(aDeed);
							break search;
						}
					}
				} // End search.
			} // End if.
		} // End traversing list of deeds.
		return setOfDeeds;
	} // End deedsContaining.

	/**
	 * Collect percepts that, directly or indirectly, imply or generalize to the
	 * target.
	 */
	public Vector downwardLinks(Percept targetPercept) {
		Vector toBeChecked, targetItems, cleaned;
		Percept targetItem, testItem;
		Relation r;
		boolean anythingNew;
		// Make a vector to contain target percepts.
		targetItems = new Vector();
		targetItems.addElement(targetPercept);
		// Copy the PerceptList in use and remove the target percept.
		toBeChecked = new Vector();
		toBeChecked = (Vector) this.clone();
		toBeChecked.removeElement(targetPercept);
		anythingNew = true;
		BuildingChains: while (anythingNew) {
			// Keep doing this until there is no addition to the targetItems.
			anythingNew = false;
			for (int j = 0; j < targetItems.size(); j++) {
				// Take a target item.
				targetItem = (Percept) targetItems.elementAt(j);
				CheckingAllItems: for (int i = 0; i < toBeChecked.size(); i++) {
					// Consider each item in the perceptList to be checked.
					testItem = (Percept) toBeChecked.elementAt(i);
					for (int k = 0; k < testItem.implications.size(); k++) {
						// Consider each of the test items upward links.
						r = (Relation) testItem.implications.elementAt(k);
						if (r.implicated.equals(targetItem.shortName)) {
							// One of the test item's up links is to a target.
							anythingNew = true;
							// The test item is in the down chain, so include it
							// as a target.
							targetItems.addElement(testItem);
							// And don't examine it anymore to see if it is in
							// the chain.
							toBeChecked.removeElement(testItem);
							if (toBeChecked.isEmpty()) {
								break BuildingChains;
							} else {
								// Shift back the index to allow for the
								// deletion.
								i = i - 1;
								// And consider the next item in the
								// perceptList.
								continue CheckingAllItems;
							}
						}
					}
				}
			}
		}
		// Eliminate duplicates.
		cleaned = new PerceptList();
		// Take out the original target.
		while (targetItems.removeElement(targetPercept))
			;
		// Return the downward chains, with each element listed only once.
		for (int i = 0; i < targetItems.size(); i++) {
			testItem = (Percept) targetItems.elementAt(i);
			if (cleaned.indexOf(testItem) < 0) { // if not already in cleaned
				cleaned.addElement(testItem);
			}
		}
		return cleaned;
	} // End downwardLinks.

	/**
	 * Collect generalizations or implications, direct or indirect, of the
	 * source.
	 */
	public Vector upwardLinks(Percept sourcePercept) {
		Relation r;
		Percept testItem, foundItem;
		Vector retrievals;
		int index;
		retrievals = new Vector();
		retrievals.addElement(sourcePercept);
		int currentElement = 0;
		for (int i = currentElement++; i < retrievals.size(); i++) {
			testItem = (Percept) retrievals.elementAt(i);
			for (int j = 0; j < testItem.implications.size(); j++) {
				r = (Relation) testItem.implications.elementAt(j);
				index = this.findPositionOfElement(r.implicated);
				if (index >= 0) {
					// The upward link exists in the PerceptList being examined.
					foundItem = (Percept) this.elementAt(index);
					if (retrievals.indexOf(foundItem) < 0) {
						// Not in retrievals yet.
						retrievals.addElement(this.elementAt(index));
					}
				}
			}
		}
		// Take out the original source.
		while (retrievals.removeElement(sourcePercept))
			;
		return retrievals;
	} // End upwardLinks.

	/* METHODS FOR BARYPOSITIONING. */

	/** Perform a topological sort of the deeds in <code>this</code>. */
	public PerceptList topoSort() {
		PerceptList reducedTHIS, topoList;
		int index;
		int numberOfTiers;
		int[] hierarchy;
		Deed focalDeed, upperDeed, implicatedDeed;
		Relation r;
		int numberProcessed;
		// Remove any summarized events.
		reducedTHIS = new PerceptList();
		for (int i = 0; i < this.size(); i++) {
			focalDeed = (Deed) this.elementAt(i);
			if (!focalDeed.isSummarized) {
				reducedTHIS.addElement(focalDeed);
			}
		}
		// Count the number of prerequisites for each element,
		// and get the first tier.
		topoList = new PerceptList();
		numberProcessed = 0;
		for (int i = 0; i < reducedTHIS.size(); i++) {
			focalDeed = (Deed) reducedTHIS.elementAt(i);
			focalDeed.upCount = focalDeed.implications.size();
			if (focalDeed.upCount == 0) {
				focalDeed.tier = 1;
				topoList.addElement(focalDeed);
				numberProcessed++;
			}
		}
		if (numberProcessed == 0) { return topoList; // Empty list.
		}
		hierarchy = new int[MAX_NUMBER_OF_TIERS];
		hierarchy[0] = -1;
		numberOfTiers = 0;
		// Get the lower tiers.
		while (numberProcessed < reducedTHIS.size()) {
			numberOfTiers++;
			// Index to topolist:
			hierarchy[numberOfTiers] = numberProcessed - 1;
			NextUnprocessedDeed: for (int j = 0; j < reducedTHIS.size(); j++) {
				// Consider each deed in this set.
				focalDeed = (Deed) reducedTHIS.elementAt(j);
				if (focalDeed.upCount > 0) {
					// The tier of this deed has not been found yet.
					for (int k = 0; k < focalDeed.implications.size(); k++) {
						// Consider each implication of the focal deed.
						r = (Relation) focalDeed.implications.elementAt(k);
						index = Ethno.deeds.findPositionOfElement(r.implicated);
						implicatedDeed = (Deed) Ethno.deeds.elementAt(index);
						for (int i = hierarchy[numberOfTiers - 1] + 1; i <= hierarchy[numberOfTiers]; i++) {
							// Consider deed in last tier.
							upperDeed = (Deed) topoList.elementAt(i);
							if (implicatedDeed == upperDeed) {
								// The implicated deed is in last tier, so
								// decrement count.
								focalDeed.upCount--;
								if (focalDeed.upCount == 0) {
									// All implications in upper tiers, so add
									// deed to this tier.
									focalDeed.tier = numberOfTiers + 1;
									topoList.addElement(focalDeed);
									numberProcessed++;
									continue NextUnprocessedDeed;
								}
							}
						}
					}
				}
			}
		}
		return topoList;
	} // End topoSort.

	/**
	 * Rank nodes in each tier of <code>this</code> topographically-ordered list
	 * (including dummy nodes) for minimum crossed lines.
	 */
	public void baryRank() {
		Deed focalDeed, testDeed;
		int currentTier, rank, cycle, sum, totalNodes;
		boolean changed;
		// Initialize.
		for (int i = 0; i < this.size(); i++) {
			focalDeed = (Deed) this.elementAt(i);
			focalDeed.upCount = focalDeed.upNodes.size();
			focalDeed.downCount = focalDeed.downNodes.size();
			focalDeed.horizontalPosition = 0;
		}
		cycle = 1;
		changed = false;
		do { // Iterate.
				// Assign ranks to nodes at each level.
			currentTier = 1;
			rank = 1;
			for (int i = 0; i < this.size(); i++) {
				// Rank nodes in each tier alphabetically.
				focalDeed = (Deed) this.elementAt(i);
				if (focalDeed.tier != currentTier) {
					currentTier++;
					rank = 1;
				}
				focalDeed.tierRank = rank++;
			}
			// Get the average rank of connected nodes.
			for (int i = 0; i < this.size(); i++) {
				sum = 0;
				totalNodes = 0;
				focalDeed = (Deed) this.elementAt(i);
				if (isOdd(cycle) && ((cycle < 10) || (focalDeed.tier == 1))) {
					// On early odd cycles or in the first tier average the
					// ranks of downwardly connected nodes.
					for (int j = 0; j < focalDeed.downCount; j++) {
						testDeed = (Deed) focalDeed.downNodes.elementAt(j);
						sum = sum + 100 * testDeed.tierRank;
						totalNodes++;
					}
				}
				// Average rank of every upwardly connected node.
				for (int j = 0; j < focalDeed.upCount; j++) {
					testDeed = (Deed) focalDeed.upNodes.elementAt(j);
					sum = sum + 100 * testDeed.tierRank;
					totalNodes++;
				}
				if (totalNodes > 0) {
					focalDeed.horizontalPosition = Math.round(sum / totalNodes);
				} else {
					// Isolated nodes go on right.
					focalDeed.horizontalPosition = 100000;
				}
			}
			// Sort by average rank of connected nodes:
			changed = this.sortHorizontalPositions();
		} while (changed && (++cycle < MAX_ITERATIONS_FOR_RANKING));
	} // End baryRanks.

	/** Performs a shell sort of horizontal positions of deeds in each tier. */
	public boolean sortHorizontalPositions() {
		Deed deed1, deed2;
		int currentTier, first, last, gap, i, j, k;
		boolean changed;
		changed = false;
		currentTier = 1;
		first = 0;
		while (first < this.size()) {
			// Find last element in this tier.
			last = first;
			do {
				deed1 = (Deed) this.elementAt(last);
				// Next statement drops thru with a tier change, before changing
				// last.
			} while ((deed1.tier == currentTier) && (++last < this.size()));
			last--; // Convert tier rank to vector index.
			// Place all generalizations on right.
			int rightmostDistance =
				((Deed) this.elementAt(last)).horizontalPosition;
			for (int m = first; m <= last; m++) {
				deed1 = (Deed) this.elementAt(m);
				if (deed1.isGeneralization) {
					deed1.horizontalPosition = rightmostDistance + m;
				}
			}
			gap = (last - first + 1) / 2;
			while (gap > 0) {
				for (i = (first + gap); i <= last; i++) {
					j = i - gap;
					while (j >= first) {
						deed1 = (Deed) this.elementAt(j);
						k = j + gap;
						deed2 = (Deed) this.elementAt(k);
						if (deed1.horizontalPosition <= deed2.horizontalPosition) {
							j = 0;
						} else {
							// Switch elements.
							this.insertElementAt(deed2, j);
							this.removeElementAt(j + 1);
							this.insertElementAt(deed1, k);
							this.removeElementAt(k + 1);
							changed = true;
						}
						j = j - gap;
					}
				}
				gap = gap / 2;
			}
			currentTier++;
			first = last + 1;
		}
		return changed;
	} // End shellSort.

	/** Position nodes in each tier to minimize crossing lines. */
	public void baryPositionDeeds(int width, int height, boolean namedNodes) {
		// "this" has nodes assembled by tiers, starting with highest.
		Deed focalDeed, testDeed;
		int currentTier, first, last, lastNodePosition, realNodeRank, cycle;
		int distanceBetweenNodeCenters, distanceToLeftMargin;
		int nameSum, positionsSum, N, maxNodeHalfWidth, totalNodes;
		// Initial spacing.
		currentTier = 1;
		first = 0;
		do {
			// Get tier information.
			last = first; // Find last element in this tier.
			N = 0; // Find number of visible nodes in this tier.
			nameSum = 0; // Find sum of name widths in this tier.
			testDeed = (Deed) this.elementAt(last);
			while (testDeed.tier == currentTier) {
				if (!testDeed.isADummy) { // Only visible nodes.
					N++;
					nameSum = nameSum + 2 * testDeed.halfNameWidth;
				}
				if (++last < this.size()) {
					testDeed = (Deed) this.elementAt(last);
				} else {
					break;
				}
			}
			// Make sure all the deed names in this tier fit the line width.
			if ((nameSum - width) > 0) {
				// Have to reduce some name widths.
				maxNodeHalfWidth = Math.round(width / (2 * N));
				for (int i = first; i < last; i++) {
					testDeed = (Deed) this.elementAt(i);
					if (testDeed.isADummy) continue;
					if (testDeed.halfNameWidth > maxNodeHalfWidth) {
						testDeed.halfNameWidth = maxNodeHalfWidth;
					}
				}
			} // End reducing name widths.
				// Set initial spacing.
			distanceBetweenNodeCenters = width / N;
			distanceToLeftMargin = distanceBetweenNodeCenters / 2;
			realNodeRank = 0;
			lastNodePosition = 0;
			for (int i = first; i < last; i++) {
				testDeed = (Deed) this.elementAt(i);
				if (testDeed.isADummy) {
					// Dummy node gets position of its source in higher tier.
					Deed sourceDeed = (Deed) testDeed.upNodes.elementAt(0);
					lastNodePosition = sourceDeed.horizontalPosition;
				} else {
					lastNodePosition =
						distanceToLeftMargin + realNodeRank++
							* distanceBetweenNodeCenters;
				}
				testDeed.horizontalPosition = lastNodePosition;
			}
			// Go to next tier.
			currentTier++;
			first = last;
		} while (first < this.size()); // End of initial spacing.

		// Align connected nodes.
		cycle = 1; // Cycle through maxPositioningIterations.
		do {
			currentTier = 1;
			first = 0;
			for (int i = 0; i < this.size(); i++) {
				focalDeed = (Deed) this.elementAt(i);
				if (focalDeed.tier != currentTier) {
					// This is first node of next tier.
					// Compute finished positioning of nodes on tier above
					// focalDeed.
					this.spaceNodesInTier(first, i - 1, width);
					first = i;
					currentTier++;
				}
				// Get the average position of up-connected nodes.
				positionsSum = 0;
				totalNodes = 0;
				for (int j = 0; j < focalDeed.upCount; j++) {
					testDeed = (Deed) focalDeed.upNodes.elementAt(j);
					positionsSum = positionsSum + testDeed.horizontalPosition;
					totalNodes++;
				}
				if (((!focalDeed.isADummy) && isOdd(cycle))
					|| (focalDeed.tier == 1)) {
					// Average in the ranks of down connected nodes, too.
					for (int j = 0; j < focalDeed.downCount; j++) {
						testDeed = (Deed) focalDeed.downNodes.elementAt(j);
						positionsSum =
							positionsSum + testDeed.horizontalPosition;
						totalNodes++;
					}
				}
				if (totalNodes > 0) {
					focalDeed.horizontalPosition = positionsSum / totalNodes;
				}
				if (i == (this.size() - 1)) { // Last node?
					// Compute finished positioning of nodes on bottom tier.
					this.spaceNodesInTier(first, i, width);
				}
			}
		} while (++cycle <= MAX_ITERATIONS_FOR_POSITIONING);
	} // End baryPositionDeeds.

	/** Adjust the horizontal positions of a tier's nodes to prevent overlaps. */
	public void spaceNodesInTier(
		int firstNodeInTier, int lastNodeInTier, int widthOfLine) {
		int increment, linePosition, goLeft, available, goRight;
		Deed thisNode, nodeOnRightSide, leftwardLeftNode, rightwardLeftNode, aDeed;
		boolean firstRealNode = true;
		// Spread the nodes on the tier.
		long divideOutcome =
			Math.round(widthOfLine / (lastNodeInTier - firstNodeInTier + 2));
		increment = (int) divideOutcome;
		linePosition = increment;
		for (int i = firstNodeInTier; i <= lastNodeInTier; i++) {
			((Deed) this.elementAt(i)).horizontalPosition = linePosition;
			linePosition = linePosition + increment;
		}
		for (int thisNodeIndex = firstNodeInTier; thisNodeIndex <= lastNodeInTier; thisNodeIndex++) {
			// Check positioning of each node on this tier.
			thisNode = (Deed) this.elementAt(thisNodeIndex);
			if (thisNode.isADummy) continue; // Ignore dummy nodes in
												// positioning.
			if (firstRealNode
				& ((thisNode.horizontalPosition - thisNode.halfNameWidth) < 0)) {
				// Get first node within left margin.
				thisNode.horizontalPosition = thisNode.halfNameWidth;
				firstRealNode = false;
			}
			nodeOnRightSide = thisNode; // Initialization for compiler.
			boolean haveRealNodeAtRight = false;
			for (int j = thisNodeIndex + 1; j <= lastNodeInTier; j++) {
				// See if another visible node is to the right of thisNode.
				nodeOnRightSide = (Deed) this.elementAt(j);
				if (!nodeOnRightSide.isADummy) {
					haveRealNodeAtRight = true;
					break;
				}
			}
			if (haveRealNodeAtRight) {
				// Distance between center positions of nodes must be enough for
				// their names and spacing.
				increment =
					(thisNode.halfNameWidth + nodeOnRightSide.halfNameWidth)
						- (nodeOnRightSide.horizontalPosition - thisNode.horizontalPosition);
			} else {
				// No real nodes right of thisNode. Check that distance to right
				// margin is enough for thisNode's name.
				increment =
					(thisNode.horizontalPosition + thisNode.halfNameWidth)
						- widthOfLine;
			}
			if (increment > 0) {
				// Need more space between nodes, or between thisNode and right
				// margin. Try left side first.
				goLeft = increment;
				rightwardLeftNode = thisNode;
				leftwardLeftNode = rightwardLeftNode; // Initialize for
														// compiler.
				for (int j = thisNodeIndex - 1; j >= firstNodeInTier - 1; j--) {
					available = 0;
					// Consider each real node leftward of this node.
					if (j < firstNodeInTier) {
						// No more left nodes on this tier. See if there is
						// excess space at the left margin.
						available =
							rightwardLeftNode.horizontalPosition
								- rightwardLeftNode.halfNameWidth;
					} else {
						leftwardLeftNode = (Deed) this.elementAt(j);
						if (leftwardLeftNode.isADummy) {
							continue;
						}
						// Found a leftward real node. See if there is extra
						// space between nodes.
						available =
							(rightwardLeftNode.horizontalPosition - leftwardLeftNode.horizontalPosition)
								- (rightwardLeftNode.halfNameWidth + leftwardLeftNode.halfNameWidth);
					}
					if (available > 0) {
						// Found some available space.
						int movement = Math.min(available, goLeft);
						for (int k = j + 1; k <= thisNodeIndex; k++) {
							// Move all nodes from rightwardLeftNode to thisNode
							// leftward the required amount.
							aDeed = (Deed) this.elementAt(k);
							if (!aDeed.isADummy) {
								aDeed.horizontalPosition =
									aDeed.horizontalPosition - movement;
							}
						}
						goLeft = Math.max(0, goLeft - available);
					}
					// The leftmost left-side node becomes the rightmost one
					// next round:
					rightwardLeftNode = leftwardLeftNode;
					if (goLeft == 0) break; // Enough space has been found to
					// the left.
				}
				if (goLeft == 0) {
					continue;
				} // Finished positioning thisNode relative to rightward
					// constraint.
				/*
				 * No more space on left so move visible rightward nodes further
				 * right to finish the spacing. Half-widths were set so that
				 * enough space must be available.
				 */
				goRight = goLeft;
				int positionOfLeftwardNode = thisNode.horizontalPosition;
				int halfNameWidthOfLeftwardNode = thisNode.halfNameWidth;
				for (int j = thisNodeIndex + 1; j <= lastNodeInTier; j++) {
					aDeed = (Deed) this.elementAt(j);
					if (aDeed.isADummy) {
						continue;
					}
					available =
						Math.max(
							0,
							(aDeed.horizontalPosition - positionOfLeftwardNode)
								- (aDeed.halfNameWidth + halfNameWidthOfLeftwardNode));
					if (available >= goRight) {
						break; // Extra space between the current node and the
						// one on left has absorbed movement of leftward node.
					} else {
						aDeed.horizontalPosition =
							aDeed.horizontalPosition + goRight; // Move the
																// current node
																// rightward.
						// Account for space that was available with the current
						// node:
						goRight = goRight - available;
						// Make the current node into the one on the left for/
						// next round.
						positionOfLeftwardNode = aDeed.horizontalPosition;
						halfNameWidthOfLeftwardNode = aDeed.halfNameWidth;
					}
				} // End of moving rightward nodes.
			} // End if (increment > 0).
		} // End for thisNodeIndex.
	} // End spaceNodesInTier.

	/** Return the number of deeds in the widest tier. */
	int getMaxEventsOnAnyTier() {
		Deed aDeed;
		int tier[] = new int[MAX_NUMBER_OF_TIERS];
		for (int i = 0; i < tier.length; i++) {
			tier[i] = 0;
		}
		int numberOfNodesOnTier, numberOfTiers;
		numberOfNodesOnTier = numberOfTiers = 0;
		for (int i = 0; i < this.size(); i++) { // Traverse list of deeds.
			aDeed = (Deed) this.elementAt(i);
			if (!aDeed.isLinked) continue;
			if (aDeed.isADummy) continue;
			if (aDeed.tier < 0) continue;
			numberOfTiers = Math.max(numberOfTiers, aDeed.tier);
			++tier[aDeed.tier];
		}
		for (int i = 1; i <= numberOfTiers; i++) {
			numberOfNodesOnTier = Math.max(numberOfNodesOnTier, tier[i]);
		}
		return numberOfNodesOnTier;
	} // End maxEventsOnTier.

	/** Return the number of tiers in the model of deeds. */
	int getNumberOfTiers() {
		Deed aDeed;
		int numberOfTiers = 0;
		for (int i = 0; i < this.size(); i++) { // Traverse list of deeds.
			aDeed = (Deed) this.elementAt(i);
			if (aDeed.isADummy) continue;
			if (aDeed.tier < 0) continue;
			numberOfTiers = Math.max(numberOfTiers, aDeed.tier);
		}
		return numberOfTiers;
	}

	/** Return the average length of deeds' short names. */
	int getAverageNameLength() {
		Deed aDeed;
		int sumOfLengths = 0;
		if (this.size() < 1) return(0);
		for (int i = 0; i < this.size(); i++) { // Traverse list of deeds.
			aDeed = (Deed) this.elementAt(i);
			if (aDeed.isADummy) continue;
			if (aDeed.tier < 0) continue;
			sumOfLengths = sumOfLengths + aDeed.shortName.length();
		}
		return Math.round(sumOfLengths / this.size());
	}

	/* METHODS FOR SEARCHING. */

	/**
	 * Same as upwardLinks and downwardLinks BUT works with a vector of
	 * DeedWithLinks. Uses special records for deeds: DeedWithLinks.
	 */
	public Vector quickLinks(Deed source, int direction) {
		DeedWithLinks aLinkedDeed;
		Object obj;
		Vector retrievals, deedRetrievals, cleaned;
		retrievals = new Vector();
		retrievals.addElement(findDeedWithLinks(source));
		for (int i = 0; i < retrievals.size(); i++) {
			aLinkedDeed = (DeedWithLinks) retrievals.elementAt(i);
			if (direction == SEARCH_DOWN) {
				for (int j = 0; j < aLinkedDeed.downLinks.size(); j++) {
					retrievals
						.addElement(findDeedWithLinks(aLinkedDeed.downLinks
							.elementAt(j)));
				}
			} else {
				for (int jj = 0; jj < aLinkedDeed.upLinks.size(); jj++) {
					retrievals.addElement(findDeedWithLinks(aLinkedDeed.upLinks
						.elementAt(jj)));
				}
			}
		}
		// Convert each element from DeedWithLinks to standard Deed.
		deedRetrievals = new Vector();
		for (int i = 0; i < retrievals.size(); i++) {
			aLinkedDeed = (DeedWithLinks) retrievals.elementAt(i);
			deedRetrievals.addElement(aLinkedDeed.theDeed);
		}
		cleaned = new Vector();
		// Take out the original source.
		while (deedRetrievals.removeElement(source))
			;
		// Return the chains, with each element listed only once.
		while (!deedRetrievals.isEmpty()) {
			obj = deedRetrievals.elementAt(0);
			cleaned.addElement(obj);
			// Remove all instances of this element from targetItems.
			while (deedRetrievals.removeElement(obj))
				;
		}
		return cleaned;
	} // End quickLinks.

	/**
	 * Find the DeedWithLinks version of a deed in "this" vector of
	 * DeedWithLinks.
	 */
	public DeedWithLinks findDeedWithLinks(Object aDeed) {
		DeedWithLinks aLinkedDeed;
		aLinkedDeed = (DeedWithLinks) this.elementAt(0); // Gratuitous
		// initialization.
		for (int i = 0; i < this.size(); i++) {
			aLinkedDeed = (DeedWithLinks) this.elementAt(i);
			if (aLinkedDeed.theDeed == aDeed) { return aLinkedDeed; }
		}
		return null;
	} // End findDeedWithLinks.

	/**
	 * Determine which implication relation is a commuter; returns -1 if none
	 * are.
	 */
	int findCommuterLink(Deed node1, Deed node2) {
		int relationThatIsCommuter = -1;
		for (int index = 0; index < node1.implications.size(); index++) {
			Relation r = (Relation) node1.implications.elementAt(index);
			if (r.implicated.equals(node2.shortName) & r.commutes) {
				relationThatIsCommuter = index;
				break;
			}
		}
		return relationThatIsCommuter;
	}

	/**
	 * Determine which implication relation of Node1 implies Node2; return -1 if
	 * none do..
	 */
	int relationIndexForNode1ImpliesNode2(Deed node1, Deed node2) {
		int implicationExistsForWhichRelation = -1;
		for (int index = 0; index < node1.implications.size(); index++) {
			Relation r = (Relation) node1.implications.elementAt(index);
			if (r.implicated.equals(node2.shortName) & !r.commutes) {
				implicationExistsForWhichRelation = index;
				break;
			}
		}
		return implicationExistsForWhichRelation;
	}

} // End class PerceptList.
