
import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Choice;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Point;
import java.awt.TextArea;
import java.awt.TextField;
import java.awt.event.ActionEvent;
import java.awt.event.WindowEvent;
import java.util.Vector;

/** Dialog offering possible solutions to a problem in processing events. */
public class RefineModelDialog extends Dialog {
	final int TRY_CUT_LINK = 1, TRY_NONDEPLETING_LINK = 2,
			TRY_COMMUTING_CONSEQUENCE = 3;

	final int TRY_UNRECORDED_PREREQUISITE = 4, TRY_DISJUNCTIVE = 5,
			TRY_NEW_DISJUNCTIVE_PREREQUISITE = 6;

	final int TRY_ADD_LINK = 7, TRY_UNRECORDED_CONSEQUENCE = 8,
			TRY_REPEATABLE = 9, TRY_NEW_EVENT = 10;

	Panel dialogPanel = new Panel();

	Panel problemPanel = new Panel();

	Panel solutionsPanel = new Panel();

	Panel overviewPanel = new Panel();

	Panel offeredSolutionArea = new Panel();

	Panel inputPanel = new Panel();

	Panel buttonArea = new Panel();

	Label regardingDeedLabel = new Label();

	Label problemLabel1 = new Label();

	Label problemLabel2 = new Label();

	Label solutionsTitle = new Label();

	Label offeredSolutionTitle = new Label();

	Label offeredSolutionLabel = new Label();

	TextArea solutionsOverviewBox = new TextArea(4, 40);

	TextField newEventTextField = new TextField(20);

	Choice relevantDeedsChoice = new Choice();

	Button nextButton = new Button();

	Button doneButton = new Button();

	Button adoptButton = new Button();

	BorderLayout borderLayout1 = new BorderLayout();

	BorderLayout borderLayout2 = new BorderLayout();

	BorderLayout borderLayout3 = new BorderLayout();

	BorderLayout borderLayout4 = new BorderLayout();

	BorderLayout borderLayout5 = new BorderLayout();

	int currentlyOfferedSolution;

	String problemText, solutionText;

	Deed focalDeed;

	DeedWithLinks linkedFocalDeed, linkedOtherDeed;

	boolean initializing, unusedRatherThanUnprimed;

	public RefineModelDialog(Frame frame, Deed d) {
		super(frame, Ethno.interfaceTexts.getString("problemHeaderText"), true);
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		focalDeed = d;
		try {
			jbInit();
			add(dialogPanel);
			pack();
			Point upperLeftCorner = frame.getLocation();
			Dimension parentSize = frame.getSize();
			Dimension dialogSize = this.getSize();
			upperLeftCorner.translate(
				(parentSize.width - dialogSize.width) / 2,
				(parentSize.height - dialogSize.height) / 2);
			this.setLocation(upperLeftCorner);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	}

	void jbInit() throws Exception {
		focalDeed.isSelected = true;
		linkedFocalDeed = new DeedWithLinks(focalDeed);
		unusedRatherThanUnprimed = focalDeed.isBlocked;

		dialogPanel.setBackground(Color.yellow);
		problemPanel.setBackground(Color.yellow);
		overviewPanel.setBackground(Color.yellow);
		solutionsPanel.setBackground(Color.yellow);
		solutionsPanel.setLayout(borderLayout5);
		offeredSolutionArea.setBackground(Color.yellow);
		regardingDeedLabel.setBackground(Color.yellow);
		problemLabel1.setBackground(Color.yellow);
		problemLabel2.setBackground(Color.yellow);
		solutionsTitle.setBackground(Color.yellow);
		offeredSolutionTitle.setBackground(Color.yellow);
		offeredSolutionLabel.setBackground(Color.yellow);
		buttonArea.setBackground(Color.yellow);
		solutionsOverviewBox.setBackground(Color.yellow);
		nextButton.setBackground(Color.blue);
		adoptButton.setBackground(Color.blue);
		doneButton.setBackground(Color.blue);
		doneButton.setForeground(Color.white);
		nextButton.setForeground(Color.white);
		adoptButton.setForeground(Color.white);

		dialogPanel.setLayout(borderLayout2);
		borderLayout2.setVgap(10);
		problemPanel.setLayout(borderLayout3);
		overviewPanel.setLayout(borderLayout1);
		offeredSolutionArea.setLayout(borderLayout4);
		solutionsTitle.setAlignment(Label.CENTER);
		regardingDeedLabel.setAlignment(Label.CENTER);
		problemLabel1.setAlignment(Label.CENTER);
		problemLabel2.setAlignment(Label.CENTER);
		solutionsOverviewBox.setEditable(false);

		regardingDeedLabel.setText(Ethno.interfaceTexts
			.getString("regardingText") + focalDeed.shortName);
		solutionsTitle.setText(Ethno.interfaceTexts
			.getString("solutionsTitleText"));
		offeredSolutionTitle.setText(Ethno.interfaceTexts
			.getString("proposedSolutionTitleText"));
		nextButton.setLabel(Ethno.interfaceTexts.getString("nextButtonText"));
		adoptButton.setLabel(Ethno.interfaceTexts.getString("adoptButtonText"));
		doneButton.setLabel(Ethno.interfaceTexts.getString("doneButtonText"));

		dialogPanel.add(problemPanel, BorderLayout.NORTH);
		dialogPanel.add(solutionsPanel, BorderLayout.CENTER);
		dialogPanel.add(buttonArea, BorderLayout.SOUTH);
		problemPanel.add(regardingDeedLabel, BorderLayout.NORTH);
		problemPanel.add(problemLabel1, BorderLayout.CENTER);
		problemPanel.add(problemLabel2, BorderLayout.SOUTH);
		solutionsPanel.add(overviewPanel, BorderLayout.NORTH);
		solutionsPanel.add(offeredSolutionArea, BorderLayout.CENTER);
		overviewPanel.add(solutionsTitle, BorderLayout.NORTH);
		overviewPanel.add(solutionsOverviewBox, BorderLayout.CENTER);
		offeredSolutionArea.add(offeredSolutionTitle, BorderLayout.NORTH);
		offeredSolutionArea.add(offeredSolutionLabel, BorderLayout.CENTER);
		offeredSolutionArea.add(inputPanel, BorderLayout.SOUTH);
		inputPanel.add(newEventTextField, null);
		inputPanel.add(relevantDeedsChoice, null);
		buttonArea.add(nextButton, null);
		buttonArea.add(adoptButton, null);
		buttonArea.add(doneButton, null);

		// Fill in the solutions.
		String newLine = Ethno.interfaceTexts.getString("newLine");
		solutionsOverviewBox.setText("");
		solutionText = "";
		initializing = true;
		if (unusedRatherThanUnprimed) { // Unused.
			problemLabel1
				.setText(Ethno.interfaceTexts.getString("unusedText1"));
			problemLabel2
				.setText(Ethno.interfaceTexts.getString("unusedText2"));
			if (tryAddingDepletingLink()) {
				solutionsOverviewBox.append(solutionText + newLine);
			}
			if (tryUnrecordedConsequence()) {
				solutionsOverviewBox.append(solutionText + newLine);
			}
			if (tryRepeatable()) {
				solutionsOverviewBox.append(solutionText + newLine);
			}
			if (tryNewEvent()) {
				solutionsOverviewBox.append(solutionText + newLine);
			}
			// Set up for nextButton_actionPerformed below:
			solutionText = Ethno.interfaceTexts.getString("newEventSolution");
		} else { // Unprimed.
			problemLabel1.setText(Ethno.interfaceTexts
				.getString("unprimedText"));
			if (tryCuttingLinks()) {
				solutionsOverviewBox.append(solutionText + newLine);
			}
			if (tryNondepletingLink()) {
				solutionsOverviewBox.append(solutionText + newLine);
			}
			if (tryCommutingConsequence()) {
				solutionsOverviewBox.append(solutionText + newLine);
			}
			if (tryUnrecordedPrerequisite()) {
				solutionsOverviewBox.append(solutionText + newLine);
			}
			if (tryDisjunctive()) {
				solutionsOverviewBox.append(solutionText);
			}
			if (tryAddingDisjunctivePrerequisite()) {
				solutionsOverviewBox.append(solutionText);
			}
			// Set up for nextButton_actionPerformed below:
			solutionText =
				Ethno.interfaceTexts.getString("newDisjunctivePrerequisite");
		}
		initializing = false;
		ChartCard.adoptedSomeSolution = false;
		cycleThroughFeasibleFixes(); // Present the first solution.

		nextButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				cycleThroughFeasibleFixes();
			}
		});
		doneButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				exitFromDialog();
			}
		});
		adoptButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				implementChosenSolution();
			}
		});
	} // End jbInit.

	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			cancel();
		}
		super.processWindowEvent(e);
	} // End processWindowEvent.

	void cancel() {
		dispose();
	}

	/*
	 * UNPRIMED.
	 */

	/** Try changing each unaccomplished prerequisite's consequences. */
	boolean tryCuttingLinks() {
		Deed aDeed, descendantDeed, pastDeed;
		Relation r;
		solutionText = "";
		relevantDeedsChoice.removeAll();
		for (int i = 0; i < linkedFocalDeed.upLinks.size(); i++) {
			aDeed = (Deed) linkedFocalDeed.upLinks.elementAt(i);
			String repetitionCheck = "";
			if (!aDeed.isAccomplished) {
				solutionText =
					Ethno.interfaceTexts.getString("stubText")
						+ aDeed.shortName
						+ Ethno.interfaceTexts.getString("cuttingLinkSolution");
				linkedOtherDeed = new DeedWithLinks(aDeed);
				for (int j = 0; j < linkedOtherDeed.downLinks.size(); j++) {
					descendantDeed =
						(Deed) linkedOtherDeed.downLinks.elementAt(j);
					if (descendantDeed == focalDeed) {
						relevantDeedsChoice.addItem(focalDeed.shortName);
						repetitionCheck = repetitionCheck + focalDeed.shortName;
						continue;
					}
					// Add events that depleted aDeed since last occurrence of
					// focalDeed.
					for (int k = Ethno.currentEvent - 1; k >= 0; k--) {
						pastDeed = (Deed) Ethno.sequence.elementAt(k);
						if (pastDeed == focalDeed) {
							break;
						}
						if (repetitionCheck.indexOf(pastDeed.shortName) >= 0) {
							continue;
						} else {
							repetitionCheck =
								repetitionCheck + pastDeed.shortName;
						}
						if (linkedOtherDeed.downLinks.contains(pastDeed)) {
							for (int m = 0; m < pastDeed.implications.size(); m++) {
								r =
									(Relation) pastDeed.implications
										.elementAt(m);
								if (r.implicated.equals(aDeed.shortName)
									&& r.depletes && !r.commutes) {
									relevantDeedsChoice
										.addItem(pastDeed.shortName);
								}
							}
						}
					}
				}
				break;
			}
		}
		if (!solutionText.equals("")) {
			currentlyOfferedSolution = TRY_CUT_LINK;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	} // End tryCuttingLinks.

	/** Try making an edge nondepletive so a prerequisite stays accomplished. */
	boolean tryNondepletingLink() {
		Deed aPrerequisite, pastDeed;
		DeedWithLinks aDeedLinked;
		Relation r;
		boolean deedAdded;
		solutionText = "";
		relevantDeedsChoice.removeAll();
		aDeedLinked = null; // Compiler initialization.
		deedAdded = false;
		Prerequisites: for (int i = 0; i < linkedFocalDeed.upLinks.size(); i++) {
			aPrerequisite = (Deed) linkedFocalDeed.upLinks.elementAt(i);
			if (!aPrerequisite.isAccomplished) {
				// Offer to give this prerequiste non-depleting consequences if
				// the relation to focal deed depletes
				// and if focal deed occurred previously, after an occurrence of
				// this prerequisite.
				if (Ethno.sequence.indexOf(focalDeed,
					Ethno.sequence.indexOf(aPrerequisite)) < Ethno.currentEvent) {
					// Focal deed did occur previously, after an occurrence of
					// this prerequisite.
					for (int n = 0; n < focalDeed.implications.size(); n++) {
						r = (Relation) focalDeed.implications.elementAt(n);
						if (r.implicated.equals(aPrerequisite.shortName)
							&& r.depletes) { // It depletes.
							relevantDeedsChoice.addItem(focalDeed.shortName);
							solutionText =
								Ethno.interfaceTexts.getString("stubText")
									+ aPrerequisite.shortName
									+ Ethno.interfaceTexts
										.getString("nondepletingLinkSolution");
							deedAdded = true;
						}
					}
				}
				// Find the prerequisites consequences.
				aDeedLinked = new DeedWithLinks(aPrerequisite);
				for (int j = 0; j < aDeedLinked.downLinks.size(); j++) {
					// Check if this deed happened since last occurrence of
					// focal deed.
					for (int k = Ethno.currentEvent - 1; k >= 0; k--) {
						pastDeed = (Deed) Ethno.sequence.elementAt(k);
						if (pastDeed == focalDeed) {
							break;
						}
						if (aDeedLinked.downLinks.contains(pastDeed)) {
							// Candidate for nondepleting link if not already
							// nondepleting, or commutative.
							for (int m = 0; m < pastDeed.implications.size(); m++) {
								r =
									(Relation) pastDeed.implications
										.elementAt(m);
								if (r.implicated
									.equals(aPrerequisite.shortName)
									&& r.depletes && !r.commutes) {
									relevantDeedsChoice
										.addItem(pastDeed.shortName);
									solutionText =
										Ethno.interfaceTexts
											.getString("stubText")
											+ aPrerequisite.shortName
											+ Ethno.interfaceTexts
												.getString("nondepletingLinkSolution");
									deedAdded = true;
								}
							}
						}
						if (deedAdded) {
							// Deal with one prerequiste at a time. Leave others
							// to replay:
							break Prerequisites;
						}
					}
				}
			}
		}
		if (!solutionText.equals("")) {
			linkedOtherDeed = aDeedLinked;
			currentlyOfferedSolution = TRY_NONDEPLETING_LINK;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	}

	// Try adding a disjunctive commutation with an accomplished consequence.
	boolean tryCommutingConsequence() {
		Deed aDeed;
		solutionText = "";
		relevantDeedsChoice.removeAll();
		for (int i = 0; i < linkedFocalDeed.downLinks.size(); i++) {
			aDeed = (Deed) linkedFocalDeed.downLinks.elementAt(i);
			if (aDeed.isAccomplished) {
				relevantDeedsChoice.addItem(aDeed.shortName);
				solutionText =
					Ethno.interfaceTexts
						.getString("commutingConsequenceSolution");
			}
		}
		if (solutionText.equals(Ethno.interfaceTexts
			.getString("commutingConsequenceSolution"))) {
			currentlyOfferedSolution = TRY_COMMUTING_CONSEQUENCE;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	} // End tryCommutingConsequence.

	/**
	 * If an unaccomplished prerequisite's conditions are fulfilled, then maybe
	 * it happened without being observed.
	 */
	boolean tryUnrecordedPrerequisite() {
		Deed aDeed, aConcreteDeed;
		solutionText = "";
		String repetitionCheck = "";
		relevantDeedsChoice.removeAll();
		for (int i = 0; i < linkedFocalDeed.upLinks.size(); i++) {
			aDeed = (Deed) linkedFocalDeed.upLinks.elementAt(i);
			if (Deed.deedsAreAtDifferentGeneralizationLevels(focalDeed, aDeed)) {
				continue;
			}
			if (aDeed.isFeasible && !aDeed.isAccomplished) {
				if (aDeed.isGeneralization) {
					// Focal deed is generalized. Check this generalized
					// prerequisite's instantiators.
					Vector concreteDeeds = aDeed.getInstantiators();
					for (int j = 0; j < concreteDeeds.size(); j++) {
						aConcreteDeed = (Deed) concreteDeeds.elementAt(j);
						if (aConcreteDeed.isFeasible
							&& !aConcreteDeed.isAccomplished
							&& repetitionCheck.indexOf(aDeed.shortName) < 0) {
							// Found a performable instantiator.
							solutionText =
								Ethno.interfaceTexts
									.getString("unrecordedPrerequisiteSolution");
							relevantDeedsChoice.addItem(aDeed.shortName);
							repetitionCheck = repetitionCheck + aDeed.shortName;
						}
					}
				} else {
					// This is an unaccomplished but performable prerequisite of
					// the concrete focal deed.
					solutionText =
						Ethno.interfaceTexts
							.getString("unrecordedPrerequisiteSolution");
					relevantDeedsChoice.addItem(aDeed.shortName);
				}
			}
		}
		if (solutionText.equals(Ethno.interfaceTexts
			.getString("unrecordedPrerequisiteSolution"))) {
			currentlyOfferedSolution = TRY_UNRECORDED_PREREQUISITE;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	} // End tryUnrecordedPrerequisite.

	/** Try making prerequisites disjunctive. */
	boolean tryDisjunctive() {
		Deed aDeed;
		solutionText = "";
		relevantDeedsChoice.removeAll();
		if ((!focalDeed.isDisjunctive) && (linkedFocalDeed.upLinks.size() > 1)) {
			for (int i = 0; i < linkedFocalDeed.upLinks.size(); i++) {
				aDeed = (Deed) linkedFocalDeed.upLinks.elementAt(i);
				relevantDeedsChoice.addItem(aDeed.shortName);
				if (aDeed.isAccomplished) {
					solutionText =
						Ethno.interfaceTexts.getString("disjunctiveSolution");
				}
			}
		}
		if (solutionText.equals(Ethno.interfaceTexts
			.getString("disjunctiveSolution"))) {
			currentlyOfferedSolution = TRY_DISJUNCTIVE;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	} // End tryDisjunctive.

	/** If the focal event is disjunctive, try adding a new prerequisite. */
	boolean tryAddingDisjunctivePrerequisite() {
		Deed aDeed;
		if (!focalDeed.isDisjunctive) { return false; }
		solutionText = "";
		relevantDeedsChoice.removeAll();
		// Any accomplished deed that is not a consequence of this event might
		// enable this disjunctive event.
		lookingForNonConsequences: for (int i = 0; i < Ethno.deeds.size(); i++) {
			aDeed = (Deed) Ethno.deeds.elementAt(i);
			if ((!aDeed.isAccomplished) || (aDeed == focalDeed)) {
				continue;
			}
			for (int j = 0; j < linkedFocalDeed.downLinks.size(); j++) {
				if (aDeed == linkedFocalDeed.downLinks.elementAt(j)) {
					continue lookingForNonConsequences;
				}
			}
			// This is an accomplished possible prerequisite of the disjunctive
			// focal deed.
			solutionText =
				Ethno.interfaceTexts.getString("newDisjunctivePrerequisite");
			relevantDeedsChoice.addItem(aDeed.shortName);
		}
		if (solutionText.equals(Ethno.interfaceTexts
			.getString("newDisjunctivePrerequisite"))) {
			currentlyOfferedSolution = TRY_NEW_DISJUNCTIVE_PREREQUISITE;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	} // End tryAddingDisjunctivePrerequisite.

	/*
	 * UNUSED.
	 */

	/** Try adding a depleting edge to use up an unused event. */
	boolean tryAddingDepletingLink() {
		Deed aDeed, pastDeed;
		Vector possibilities, repetitions;
		solutionText = "";
		relevantDeedsChoice.removeAll();
		possibilities = new Vector();
		repetitions = new Vector();
		// Only consider events happening since last occurrence of focal event.
		for (int k = Ethno.currentEvent - 1; k >= 0; k--) {
			pastDeed = (Deed) Ethno.sequence.elementAt(k);
			if (pastDeed == focalDeed) {
				break;
			}
			if (Deed.deedsAreAtDifferentGeneralizationLevels(focalDeed,
				pastDeed)) {
				continue;
			}
			if (possibilities.contains(pastDeed)) {
				// Have to disregard an event that happened more than once.
				repetitions.addElement(pastDeed);
			}
			// Don't try connecting the focal event to its superordinates or
			// subordinates.
			if (!(linkedFocalDeed.upLinks.contains(pastDeed) | linkedFocalDeed.downLinks
				.contains(pastDeed))) {
				// Found a possible new consequence.
				possibilities.addElement(pastDeed);
			}
		}
		for (int i = 0; i < possibilities.size(); i++) {
			if (!repetitions.contains(possibilities.elementAt(i))) {
				aDeed = (Deed) possibilities.elementAt(i);
				relevantDeedsChoice.addItem(aDeed.shortName);
				solutionText =
					Ethno.interfaceTexts.getString("addingLinkSolution");
			}
		}
		if (solutionText.equals(Ethno.interfaceTexts
			.getString("addingLinkSolution"))) {
			currentlyOfferedSolution = TRY_ADD_LINK;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	} // End tryAddingDepletingLink.

	/**
	 * If an unaccomplished consequence's conditions are fulfilled, then maybe
	 * it happened and depleted the focal event without being observed.
	 */
	boolean tryUnrecordedConsequence() {
		Deed aDeed;
		solutionText = "";
		relevantDeedsChoice.removeAll();
		for (int i = 0; i < linkedFocalDeed.downLinks.size(); i++) {
			aDeed = (Deed) linkedFocalDeed.downLinks.elementAt(i);
			if (aDeed.isFeasible && !aDeed.isAccomplished) {
				// This is an unaccomplished but performable consequence.
				solutionText =
					Ethno.interfaceTexts
						.getString("unrecordedConsequenceSolution");
				relevantDeedsChoice.addItem(aDeed.shortName);
			}
		}
		if (solutionText.equals(Ethno.interfaceTexts
			.getString("unrecordedConsequenceSolution"))) {
			currentlyOfferedSolution = TRY_UNRECORDED_CONSEQUENCE;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	} // End tryUnrecordedConsequence.

	/** Try making the focal deed repeatable. */
	boolean tryRepeatable() {
		solutionText = "";
		relevantDeedsChoice.removeAll();
		if (!focalDeed.isRepeatable) {
			solutionText = Ethno.interfaceTexts.getString("repeatableSolution");
			relevantDeedsChoice.addItem(focalDeed.shortName);
			currentlyOfferedSolution = TRY_REPEATABLE;
			if (!initializing) {
				newEventTextField.setVisible(false);
				relevantDeedsChoice.setVisible(true);
			}
			return true;
		}
		return false;
	} // End tryRepeatable.

	/** Try adding a new event to deplete focal event. */
	boolean tryNewEvent() {
		solutionText = Ethno.interfaceTexts.getString("newEventSolution");
		if (!initializing) {
			relevantDeedsChoice.setVisible(false);
			newEventTextField.setVisible(true);
		}
		currentlyOfferedSolution = TRY_NEW_EVENT;
		return true;
	} // End tryNewEvent.

	/*
	 * BUTTONS.
	 */

	/**
	 * Get the first feasible fix after the one just tried, which still is
	 * stored in solutionText.
	 */
	void cycleThroughFeasibleFixes() {
		boolean theSolution = false;
		offeredSolutionLabel.invalidate();
		relevantDeedsChoice.invalidate();
		newEventTextField.invalidate();
		offeredSolutionArea.invalidate();
		inputPanel.invalidate();
		if (unusedRatherThanUnprimed) { // Unused.
			if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("newEventSolution")) >= 0) { // Cycle from end.
				theSolution =
					tryAddingDepletingLink() || tryUnrecordedConsequence()
						|| tryRepeatable() || tryNewEvent();
			} else if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("addingLinkSolution")) >= 0) {
				theSolution =
					tryUnrecordedConsequence() || tryRepeatable()
						|| tryNewEvent() || tryAddingDepletingLink();
			} else if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("unrecordedConsequenceSolution")) >= 0) {
				theSolution =
					tryRepeatable() || tryNewEvent()
						|| tryAddingDepletingLink()
						|| tryUnrecordedConsequence();
			} else if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("repeatableSolution")) >= 0) {
				theSolution =
					tryNewEvent() || tryAddingDepletingLink()
						|| tryUnrecordedConsequence() || tryRepeatable();
			}
		} else {
			// Unprimed.
			// Cycle from end:
			if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("newDisjunctivePrerequisite")) >= 0) {
				theSolution =
					tryCuttingLinks() || tryNondepletingLink()
						|| tryCommutingConsequence()
						|| tryUnrecordedPrerequisite() || tryDisjunctive()
						|| tryAddingDisjunctivePrerequisite();
			} else if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("cuttingLinkSolution")) >= 0) {
				theSolution =
					tryNondepletingLink() || tryCommutingConsequence()
						|| tryUnrecordedPrerequisite() || tryDisjunctive()
						|| tryAddingDisjunctivePrerequisite()
						|| tryCuttingLinks();
			} else if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("nondepletingLinkSolution")) >= 0) {
				theSolution =
					tryCommutingConsequence() || tryUnrecordedPrerequisite()
						|| tryDisjunctive()
						|| tryAddingDisjunctivePrerequisite()
						|| tryCuttingLinks() || tryNondepletingLink();
			} else if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("commutingConsequenceSolution")) >= 0) {
				theSolution =
					tryUnrecordedPrerequisite() || tryDisjunctive()
						|| tryAddingDisjunctivePrerequisite()
						|| tryCuttingLinks() || tryNondepletingLink()
						|| tryCommutingConsequence();
			} else if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("unrecordedPrerequisiteSolution")) >= 0) {
				theSolution =
					tryDisjunctive() || tryAddingDisjunctivePrerequisite()
						|| tryCuttingLinks() || tryNondepletingLink()
						|| tryCommutingConsequence()
						|| tryUnrecordedPrerequisite();
			} else if (solutionText.indexOf(Ethno.interfaceTexts
				.getString("disjunctiveSolution")) >= 0) {
				theSolution =
					tryAddingDisjunctivePrerequisite() || tryCuttingLinks()
						|| tryNondepletingLink() || tryCommutingConsequence()
						|| tryUnrecordedPrerequisite() || tryDisjunctive();
			}
		}
		offeredSolutionLabel.setText(solutionText);
		offeredSolutionLabel.validate();
		relevantDeedsChoice.validate();
		newEventTextField.validate();
		offeredSolutionArea.validate();
		inputPanel.validate();
		this.validate();
		if (theSolution)
		; // Get rid of not-read error.
	} // End cycleThroughFeasibleFixes.

	/** Respond to user clicking the Adopt button. */
	void implementChosenSolution() {
		String theLine;
		Deed aDeed, aGeneralizedDeed;
		Relation r;
		ChartCard.adoptedSomeSolution = true;
		switch (currentlyOfferedSolution) {
		case TRY_CUT_LINK:
			theLine = relevantDeedsChoice.getSelectedItem();
			aDeed =
				(Deed) Ethno.deeds.elementAt(Ethno.deeds
					.findPositionOfElement(theLine));
			for (int n = 0; n < aDeed.implications.size(); n++) {
				r = (Relation) aDeed.implications.elementAt(n);
				if (linkedOtherDeed.theDeed.shortName.equals(r.implicated)) {
					aDeed.implications.removeElementAt(n);
					Ethno.linkingHistory =
						Ethno.linkingHistory
							+ linkedOtherDeed.theDeed.shortName
							+ Ethno.interfaceTexts.getString("cutLink")
							+ theLine
							+ Ethno.interfaceTexts.getString("newLine");
					Ethno.defineLinksCard.elicitationLog
						.setText(Ethno.linkingHistory);
					break;
				}
			}
			break;
		case TRY_NONDEPLETING_LINK:
			theLine = relevantDeedsChoice.getSelectedItem();
			aDeed =
				(Deed) Ethno.deeds.elementAt(Ethno.deeds
					.findPositionOfElement(theLine));
			for (int n = 0; n < aDeed.implications.size(); n++) {
				r = (Relation) aDeed.implications.elementAt(n);
				if (linkedOtherDeed.theDeed.shortName.equals(r.implicated)) {
					r.depletes = false;
				}
			}
			break;
		case TRY_COMMUTING_CONSEQUENCE:
			focalDeed.isDisjunctive = true;
			// Continue to next in order to add relation.
		case TRY_ADD_LINK:
			boolean commutation =
				(currentlyOfferedSolution == TRY_COMMUTING_CONSEQUENCE);
			theLine = relevantDeedsChoice.getSelectedItem();
			aDeed =
				(Deed) Ethno.deeds.elementAt(Ethno.deeds
					.findPositionOfElement(theLine));
			aDeed.implications.addElement(new Relation(focalDeed.shortName,
				true, commutation));
			if (commutation) {
				Ethno.linkingHistory =
					Ethno.linkingHistory + focalDeed.shortName
						+ Ethno.interfaceTexts.getString("space")
						+ aDeed.shortName
						+ Ethno.interfaceTexts.getString("colonSpace")
						+ Ethno.interfaceTexts.getString("addedCommuter")
						+ Ethno.interfaceTexts.getString("newLine");
			} else {
				Ethno.linkingHistory =
					Ethno.linkingHistory
						+ Ethno.interfaceTexts.getString("linkedText")
						+ focalDeed.shortName
						+ Ethno.interfaceTexts.getString("commaSpace")
						+ aDeed.shortName
						+ Ethno.interfaceTexts.getString("newLine");
			}
			break;
		case TRY_NEW_DISJUNCTIVE_PREREQUISITE:
			theLine = relevantDeedsChoice.getSelectedItem();
			aDeed =
				(Deed) Ethno.deeds.elementAt(Ethno.deeds
					.findPositionOfElement(theLine));
			focalDeed.implications.addElement(new Relation(aDeed.shortName,
				true, currentlyOfferedSolution == TRY_COMMUTING_CONSEQUENCE));
			Ethno.linkingHistory =
				Ethno.linkingHistory + focalDeed.shortName
					+ Ethno.interfaceTexts.getString("addLink")
					+ aDeed.shortName
					+ Ethno.interfaceTexts.getString("newLine");
			Ethno.defineLinksCard.elicitationLog.setText(Ethno.linkingHistory);
			break;
		case TRY_UNRECORDED_PREREQUISITE:
		case TRY_UNRECORDED_CONSEQUENCE:
			theLine = relevantDeedsChoice.getSelectedItem();
			aDeed =
				(Deed) Ethno.deeds.elementAt(Ethno.deeds
					.findPositionOfElement(theLine));
			int insertionPoint = Ethno.currentEvent;
			Ethno.sequence.insertElementAt(aDeed, insertionPoint++);
			for (int i = 0; i < aDeed.generalizations.size(); i++) {
				r = (Relation) aDeed.generalizations.elementAt(i);
				aGeneralizedDeed =
					(Deed) Ethno.deeds.elementAt(Ethno.deeds
						.findPositionOfElement(r.implicated));
				Ethno.sequence.insertElementAt(aGeneralizedDeed,
					insertionPoint++);
			}
			break;
		case TRY_DISJUNCTIVE:
			focalDeed.isDisjunctive = true;
			break;
		case TRY_REPEATABLE:
			focalDeed.isRepeatable = true;
			break;
		case TRY_NEW_EVENT:
			if (!newEventTextField.getText().equals("")) {
				aDeed = new Deed();
				aDeed.shortName = newEventTextField.getText();
				aDeed.isGeneralization = focalDeed.isGeneralization;
				Ethno.deeds.insertAlphabetically(aDeed);
				RelationDialog query =
					new RelationDialog((Frame) this.getParent().getParent(), focalDeed,
						aDeed, true);
				query.setVisible(true);
				if (aDeed.isGeneralization) {
					AbstractionDialog.setSummarizing(false);
					AbstractionDialog summary =
						new AbstractionDialog((Frame) this.getParent().getParent());
					summary.setVisible(true);
				} else {
					Ethno.sequence.insertElementAt(aDeed, Ethno.currentEvent);
				}
				Ethno.currentEvent++;
			}
			break;
		}
	} // End implementChosenSolution.

	/** Respond to user pressing Done button. */
	void exitFromDialog() {
		focalDeed.isSelected = false;
		cancel();
	}

} // End RefineModelDialog.
