
import java.awt.AWTEvent;
import java.awt.BorderLayout;
import java.awt.Button;
import java.awt.Checkbox;
import java.awt.CheckboxGroup;
import java.awt.Color;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridLayout;
import java.awt.Label;
import java.awt.Panel;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ItemEvent;
import java.awt.event.WindowEvent;

/**
 * Dialog for adding/deleting links among events, and for setting event
 * properties.
 */
public class RelationDialog extends Dialog {

	Panel dialogPanel = new Panel();

	Panel responsesPanel = new Panel();

	Panel propertiesPanel = new Panel();

	Panel descriptionPanel = new Panel();

	Label linkFromLabel = new Label();

	Label topNodeLabel = new Label();

	Label toLabel = new Label();

	Label bottomNodeLabel = new Label();

	Label spacerLabel = new Label();

	Label commutationQuestionLabel = new Label();

	CheckboxGroup defineEnabler = new CheckboxGroup();

	Checkbox upperEnablesCheckbox = new Checkbox("", defineEnabler, true);

	Checkbox lowerEnablesCheckbox = new Checkbox("", defineEnabler, false);

	Checkbox depletesCheckbox = new Checkbox();

	Button leftButton = new Button();

	Button rightButton = new Button();

	BorderLayout borderLayout1 = new BorderLayout();

	GridLayout gridLayout1 = new GridLayout(0, 1);

	GridLayout gridLayout2 = new GridLayout(0, 1);

	Deed firstClickedNode, secondClickedNode, bottomEventInDialog,
			topEventInDialog;

	Relation r;

	int relationIndex, commuterRelationIndex;

	boolean shiftKeyDown, secondClickedNodeIsImplier, existingLink;

	boolean originalRelationCommutes;

	public RelationDialog(Frame frame, Deed deed1, Deed deed2,
			boolean shiftkeydown) {
		super(frame, "", true);
		enableEvents(AWTEvent.WINDOW_EVENT_MASK);
		firstClickedNode = deed1;
		secondClickedNode = deed2;
		shiftKeyDown = shiftkeydown;
		try {
			jbInit();
			add(dialogPanel);
			pack();
			Point upperLeftCorner = frame.getLocation();
			Dimension parentSize = frame.getSize();
			Dimension dialogSize = this.getSize();
			upperLeftCorner.translate(
				(parentSize.width - dialogSize.width) / 2,
				(parentSize.height - dialogSize.height) / 2);
			this.setLocation(upperLeftCorner);
		} catch (Exception ex) {
			ex.printStackTrace();
		}
	} // End constructor that passes parameters.

	void jbInit() throws Exception {
		dialogPanel.setBackground(Color.yellow);
		responsesPanel.setBackground(Color.yellow);
		propertiesPanel.setBackground(Color.yellow);
		descriptionPanel.setBackground(Color.yellow);
		linkFromLabel.setBackground(Color.yellow);
		topNodeLabel.setBackground(Color.yellow);
		toLabel.setBackground(Color.yellow);
		bottomNodeLabel.setBackground(Color.yellow);
		spacerLabel.setBackground(Color.yellow);
		upperEnablesCheckbox.setBackground(Color.yellow);
		lowerEnablesCheckbox.setBackground(Color.yellow);
		depletesCheckbox.setBackground(Color.yellow);
		leftButton.setBackground(Color.green);
		rightButton.setBackground(Color.green);

		examineRelationOfSelectedEvents();

		linkFromLabel.setAlignment(Label.CENTER);
		topNodeLabel.setAlignment(Label.CENTER);
		toLabel.setAlignment(Label.CENTER);
		bottomNodeLabel.setAlignment(Label.CENTER);
		upperEnablesCheckbox.setState(!originalRelationCommutes);
		lowerEnablesCheckbox.setState(originalRelationCommutes);
		depletesCheckbox.setState(true);

		linkFromLabel.setText(Ethno.interfaceTexts.getString("fromText"));
		topNodeLabel.setText(topEventInDialog.shortName);
		toLabel.setText(Ethno.interfaceTexts.getString("toText"));
		bottomNodeLabel.setText(bottomEventInDialog.shortName);
		commutationQuestionLabel.setText(Ethno.interfaceTexts
			.getString("newCommuter"));
		upperEnablesCheckbox.setLabel(Ethno.interfaceTexts
			.getString("lowerText"));
		lowerEnablesCheckbox.setLabel(Ethno.interfaceTexts
			.getString("upperText"));
		if (originalRelationCommutes) {
			depletesCheckbox.setLabel(Ethno.interfaceTexts
				.getString("U_DepletesLText"));
		} else {
			depletesCheckbox.setLabel(Ethno.interfaceTexts
				.getString("L_DepletesUText"));
		}
		if (existingLink) {
			depletesCheckbox
				.setState(((Relation) bottomEventInDialog.implications
					.elementAt(relationIndex)).depletes);
			leftButton.setLabel(Ethno.interfaceTexts
				.getString("deleteLinkText"));
			rightButton.setLabel(Ethno.interfaceTexts.getString("linkOkText"));
		} else {
			leftButton.setLabel(Ethno.interfaceTexts.getString("addLinkText"));
			rightButton.setLabel(Ethno.interfaceTexts.getString("cancelText"));
		}

		dialogPanel.setLayout(borderLayout1);
		propertiesPanel.setLayout(gridLayout2);
		descriptionPanel.setLayout(gridLayout1);

		borderLayout1.setHgap(5);
		dialogPanel.add(responsesPanel, BorderLayout.SOUTH);
		dialogPanel.add(propertiesPanel, BorderLayout.CENTER);
		dialogPanel.add(descriptionPanel, BorderLayout.NORTH);
		descriptionPanel.add(linkFromLabel, null);
		descriptionPanel.add(topNodeLabel, null);
		descriptionPanel.add(toLabel, null);
		descriptionPanel.add(bottomNodeLabel, null);
		if (shiftKeyDown) {
			propertiesPanel.add(commutationQuestionLabel);
			leftButton.setLabel(Ethno.interfaceTexts.getString("okText"));
			rightButton.setLabel(Ethno.interfaceTexts.getString("cancelText"));
		} else {
			propertiesPanel.add(upperEnablesCheckbox, null);
			propertiesPanel.add(lowerEnablesCheckbox, null);
			propertiesPanel.add(spacerLabel, null);
			propertiesPanel.add(depletesCheckbox, null);
		}
		responsesPanel.add(leftButton, null);
		responsesPanel.add(rightButton, null);

		upperEnablesCheckbox.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				toggleLabelOfDepletionCheckbox();
			}
		});
		lowerEnablesCheckbox.addItemListener(new java.awt.event.ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				toggleLabelOfDepletionCheckbox();
			}
		});
		leftButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				toggleLinkExistence();
			}
		});
		rightButton.addActionListener(new java.awt.event.ActionListener() {
			public void actionPerformed(ActionEvent e) {
				recordLinkProperties();
			}
		});
	} // End jbInit.

	protected void processWindowEvent(WindowEvent e) {
		if (e.getID() == WindowEvent.WINDOW_CLOSING) {
			cancel();
		}
		super.processWindowEvent(e);
	} // End processWindowEvent.

	void cancel() {
		dispose();
	}

	/**
	 * Find out if events are directly linked, indirectly connected, or
	 * unconnected.
	 */
	void examineRelationOfSelectedEvents() {
		originalRelationCommutes = false;
		secondClickedNodeIsImplier = false;
		existingLink = false;
		// See if second-clicked event implies first.
		relationIndex =
			Ethno.deeds.relationIndexForNode1ImpliesNode2(
				(Deed) secondClickedNode, (Deed) firstClickedNode);
		if (relationIndex != -1) {
			secondClickedNodeIsImplier = true;
			existingLink = true;
			bottomEventInDialog = secondClickedNode;
			topEventInDialog = firstClickedNode;
			commuterRelationIndex =
				Ethno.deeds.findCommuterLink((Deed) secondClickedNode,
					(Deed) firstClickedNode);
			originalRelationCommutes = (commuterRelationIndex != -1);
		} else {
			// See if first-clicked event implies second.
			relationIndex =
				Ethno.deeds.relationIndexForNode1ImpliesNode2(
					(Deed) firstClickedNode, (Deed) secondClickedNode);
			if (relationIndex != -1) {
				existingLink = true;
				bottomEventInDialog = firstClickedNode;
				topEventInDialog = secondClickedNode;
				commuterRelationIndex =
					Ethno.deeds.findCommuterLink((Deed) firstClickedNode,
						(Deed) secondClickedNode);
				originalRelationCommutes = (commuterRelationIndex != -1);
			}
		}
		if (!existingLink) {
			// Check for indirect connection.
			if (Ethno.deeds.upwardLinks(firstClickedNode).contains(
				secondClickedNode)) {
				// First clicked node indirectly implies second clicked node.
				bottomEventInDialog = firstClickedNode;
				topEventInDialog = secondClickedNode;
			} else if (Ethno.deeds.upwardLinks(secondClickedNode).contains(
				firstClickedNode)) {
				// Second clicked node indirectly implies first clicked
				// node.
				secondClickedNodeIsImplier = true;
				bottomEventInDialog = secondClickedNode;
				topEventInDialog = firstClickedNode;
			} else {
				// The events are unconnected. Make upper
				// the event that was clicked first.
				topEventInDialog = firstClickedNode;
				bottomEventInDialog = secondClickedNode;
				// Don't confuse user with radio buttons.
				upperEnablesCheckbox.setVisible(false);
				lowerEnablesCheckbox.setVisible(false);
			}
		}
	} // End examineRelationOfSelectedEvents.

	void toggleLabelOfDepletionCheckbox() {
		depletesCheckbox.invalidate();
		if (lowerEnablesCheckbox.getState()) {
			depletesCheckbox.setLabel(Ethno.interfaceTexts
				.getString("U_DepletesLText"));
		} else {
			depletesCheckbox.setLabel(Ethno.interfaceTexts
				.getString("L_DepletesUText"));
		}
		depletesCheckbox.validate();
	} // End toggleLabelOfDepletionCheckbox.

	void toggleLinkExistence() {
		if (shiftKeyDown) { // Dealing with commutation.
			if (originalRelationCommutes) {
				// Remove the commuter relation.
				if (secondClickedNodeIsImplier) {
					secondClickedNode.implications
						.removeElementAt(commuterRelationIndex);
				} else {
					firstClickedNode.implications
						.removeElementAt(commuterRelationIndex);
				}
				Ethno.linkingHistory =
					Ethno.linkingHistory + topEventInDialog.shortName
						+ Ethno.interfaceTexts.getString("space")
						+ bottomEventInDialog.shortName
						+ Ethno.interfaceTexts.getString("removedCommuter")
						+ Ethno.interfaceTexts.getString("newLine");
			} else {
				// Add a second relation in the same direction, this one marked
				// disjunctive.
				// The source node in a commutation has to be disjunctive,
				// and has to changed back manually if commutation is
				// removed.
				if (secondClickedNodeIsImplier) {
					secondClickedNode.implications.addElement(new Relation(
						firstClickedNode.shortName, true, true));
					firstClickedNode.isDisjunctive = true;
				} else {
					firstClickedNode.implications.addElement(new Relation(
						secondClickedNode.shortName, true, true));
					secondClickedNode.isDisjunctive = true;
				}
				Ethno.linkingHistory =
					Ethno.linkingHistory + topEventInDialog.shortName
						+ Ethno.interfaceTexts.getString("space")
						+ bottomEventInDialog.shortName
						+ Ethno.interfaceTexts.getString("colonSpace")
						+ Ethno.interfaceTexts.getString("addedCommuter")
						+ Ethno.interfaceTexts.getString("newLine");
			}
		} else {
			if (existingLink) {
				// Delete the relation.
				if (secondClickedNodeIsImplier) {
					secondClickedNode.implications
						.removeElementAt(relationIndex);
				} else {
					firstClickedNode.implications
						.removeElementAt(relationIndex);
				}
				Ethno.linkingHistory =
					Ethno.linkingHistory + topEventInDialog.shortName
						+ Ethno.interfaceTexts.getString("space")
						+ bottomEventInDialog.shortName
						+ Ethno.interfaceTexts.getString("cutLink")
						+ Ethno.interfaceTexts.getString("newLine");
			} else { // Add link.
				bottomEventInDialog.implications.addElement(new Relation(
					topEventInDialog.shortName, depletesCheckbox.getState(),
					lowerEnablesCheckbox.getState()));
				Ethno.linkingHistory =
					Ethno.linkingHistory + bottomEventInDialog.shortName
						+ Ethno.interfaceTexts.getString("addLink")
						+ topEventInDialog.shortName
						+ Ethno.interfaceTexts.getString("newLine");
				bottomEventInDialog.isLinked = true;
				topEventInDialog.isLinked = true;
			}
		}
		Ethno.defineLinksCard.elicitationLog.setText(Ethno.linkingHistory);
		bottomEventInDialog.isSelected = false;
		topEventInDialog.isSelected = false;
		Ethno.graphCard.theGraph.repaint();
		cancel();
	}// End toggleLinkExistence.

	void recordLinkProperties() {
		Relation r =
			(Relation) bottomEventInDialog.implications
				.elementAt(relationIndex);
		if (existingLink) {
			r.commutes = lowerEnablesCheckbox.getState();
			r.depletes = depletesCheckbox.getState();
		}
		bottomEventInDialog.isSelected = false;
		topEventInDialog.isSelected = false;
		// Ethno.needNewGraph = true;
		Ethno.graphCard.theGraph.repaint();
		cancel();
	} // End recordLinkProperties.

}